@@ -1,10 +1,9 @@
 /////////////////////////////////////////////////////////////////////////////
-// Name:        wx/colrdlgg.h
+// Name:        wx/colordlg.h
 // Purpose:     wxColourDialog
-// Author:      Vadim Zeitiln
+// Author:      Vadim Zeitlin
 // Modified by:
 // Created:     01/02/97
-// RCS-ID:      $Id$
 // Copyright:   (c) wxWidgets team
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -16,22 +15,22 @@
 
 #if wxUSE_COLOURDLG
 
+#include "wx/colourdata.h"
+
 #if defined(__WXMSW__) && !defined(__WXUNIVERSAL__)
     #include "wx/msw/colordlg.h"
 #elif defined(__WXMAC__) && !defined(__WXUNIVERSAL__)
     #include "wx/osx/colordlg.h"
 #elif defined(__WXGTK20__) && !defined(__WXUNIVERSAL__)
     #include "wx/gtk/colordlg.h"
-#elif defined(__WXPALMOS__)
-    #include "wx/palmos/colordlg.h"
+#elif defined(__WXQT__)
+    #include "wx/qt/colordlg.h"
 #else
     #include "wx/generic/colrdlgg.h"
 
     #define wxColourDialog wxGenericColourDialog
 #endif
 
-class WXDLLIMPEXP_FWD_CORE wxColourData;
-
 // get the colour from user and return it
 WXDLLIMPEXP_CORE wxColour wxGetColourFromUser(wxWindow *parent = NULL,
                                               const wxColour& colInit = wxNullColour,
