@@ -17,6 +17,7 @@
 #include "wx/html/htmltag.h"
 #include "wx/html/htmldefs.h"
 #include "wx/window.h"
+#include "wx/brush.h"
 
 
 class WXDLLIMPEXP_FWD_HTML wxHtmlWindowInterface;
@@ -81,7 +82,7 @@ enum wxHtmlSelectionState
 class WXDLLIMPEXP_HTML wxHtmlRenderingState
 {
 public:
-    wxHtmlRenderingState() : m_selState(wxHTML_SEL_OUT) { m_bgMode = wxSOLID; }
+    wxHtmlRenderingState() : m_selState(wxHTML_SEL_OUT) { m_bgMode = wxBRUSHSTYLE_SOLID; }
 
     void SetSelectionState(wxHtmlSelectionState s) { m_selState = s; }
     wxHtmlSelectionState GetSelectionState() const { return m_selState; }
@@ -232,7 +233,7 @@ public:
     virtual wxHtmlCell* GetFirstChild() const { return NULL; }
 
     // members writing methods
-    virtual void SetPos(int x, int y) {m_PosX = x, m_PosY = y;}
+    virtual void SetPos(int x, int y) {m_PosX = x; m_PosY = y;}
     void SetLink(const wxHtmlLinkInfo& link);
     void SetNext(wxHtmlCell *cell) {m_Next = cell;}
 
@@ -279,17 +280,20 @@ public:
                                    const wxPoint& pos,
                                    const wxMouseEvent& event);
 
-    // This method used to adjust pagebreak position. The parameter is variable
-    // that contains y-coordinate of page break (= horizontal line that should
-    // not be crossed by words, images etc.). If this cell cannot be divided
-    // into two pieces (each one on another page) then it moves the pagebreak
-    // few pixels up.
+    // This method is called when paginating HTML, e.g. when printing.
+    //
+    // On input, pagebreak contains y-coordinate of page break (i.e. the
+    // horizontal line that should not be crossed by words, images etc.)
+    // relative to the parent cell on entry and may be modified to request a
+    // page break at a position before it if this cell cannot be divided into
+    // two pieces (each one on its own page).
+    //
+    // Note that page break must still happen on the current page, i.e. the
+    // returned value must be strictly greater than "*pagebreak - pageHeight"
+    // and less or equal to "*pagebreak" for the value of pagebreak on input.
     //
     // Returned value : true if pagebreak was modified, false otherwise
-    // Usage : while (container->AdjustPagebreak(&p)) {}
-    virtual bool AdjustPagebreak(int *pagebreak,
-                                 const wxArrayInt& known_pagebreaks,
-                                 int pageHeight) const;
+    virtual bool AdjustPagebreak(int *pagebreak, int pageHeight) const;
 
     // Sets cell's behaviour on pagebreaks (see AdjustPagebreak). Default
     // is true - the cell can be split on two pages
@@ -442,7 +446,7 @@ protected:
 class WXDLLIMPEXP_HTML wxHtmlContainerCell : public wxHtmlCell
 {
 public:
-    wxHtmlContainerCell(wxHtmlContainerCell *parent);
+    explicit wxHtmlContainerCell(wxHtmlContainerCell *parent);
     virtual ~wxHtmlContainerCell();
 
     virtual void Layout(int w) wxOVERRIDE;
@@ -451,13 +455,16 @@ public:
     virtual void DrawInvisible(wxDC& dc, int x, int y,
                                wxHtmlRenderingInfo& info) wxOVERRIDE;
 
-    virtual bool AdjustPagebreak(int *pagebreak,
-                                 const wxArrayInt& known_pagebreaks,
-                                 int pageHeight) const wxOVERRIDE;
+    virtual bool AdjustPagebreak(int *pagebreak, int pageHeight) const wxOVERRIDE;
 
     // insert cell at the end of m_Cells list
     void InsertCell(wxHtmlCell *cell);
 
+    // Detach a child cell. After calling this method, it's the caller
+    // responsibility to destroy this cell (possibly by calling InsertCell()
+    // with it to attach it elsewhere).
+    void Detach(wxHtmlCell *cell);
+
     // sets horizontal/vertical alignment
     void SetAlignHor(int al) {m_AlignHor = al; m_LastLayout = -1;}
     int GetAlignHor() const {return m_AlignHor;}
@@ -486,7 +493,7 @@ public:
     // returns background colour (of wxNullColour if none set), so that widgets can
     // adapt to it:
     wxColour GetBackgroundColour();
-    void SetBorder(const wxColour& clr1, const wxColour& clr2, int border = 1) {m_Border = border; m_BorderColour1 = clr1, m_BorderColour2 = clr2;}
+    void SetBorder(const wxColour& clr1, const wxColour& clr2, int border = 1) {m_Border = border; m_BorderColour1 = clr1; m_BorderColour2 = clr2;}
     virtual wxHtmlLinkInfo* GetLink(int x = 0, int y = 0) const wxOVERRIDE;
     virtual const wxHtmlCell* Find(int condition, const void* param) const wxOVERRIDE;
 
@@ -648,16 +655,12 @@ protected:
 class WXDLLIMPEXP_HTML wxHtmlLinkInfo : public wxObject
 {
 public:
-    wxHtmlLinkInfo() : wxObject()
-          { m_Href = m_Target = wxEmptyString; m_Event = NULL, m_Cell = NULL; }
-    wxHtmlLinkInfo(const wxString& href, const wxString& target = wxEmptyString) : wxObject()
-          { m_Href = href; m_Target = target; m_Event = NULL, m_Cell = NULL; }
-    wxHtmlLinkInfo(const wxHtmlLinkInfo& l) : wxObject()
-          { m_Href = l.m_Href, m_Target = l.m_Target, m_Event = l.m_Event;
-            m_Cell = l.m_Cell; }
-    wxHtmlLinkInfo& operator=(const wxHtmlLinkInfo& l)
-          { m_Href = l.m_Href, m_Target = l.m_Target, m_Event = l.m_Event;
-            m_Cell = l.m_Cell; return *this; }
+    wxHtmlLinkInfo()
+        { m_Event = NULL; m_Cell = NULL; }
+    wxHtmlLinkInfo(const wxString& href, const wxString& target = wxString())
+        : m_Href(href)
+        , m_Target(target)
+        { m_Event = NULL; m_Cell = NULL; }
 
     void SetEvent(const wxMouseEvent *e) { m_Event = e; }
     void SetHtmlCell(const wxHtmlCell *e) { m_Cell = e; }
