@@ -4,7 +4,6 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     01/02/97
-// RCS-ID:      $Id: dc.h,v 1.59 2006/10/29 10:46:49 SC Exp $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -13,6 +12,7 @@
 #define _WX_MSW_DC_H_
 
 #include "wx/defs.h"
+#include "wx/dc.h"
 
 // ---------------------------------------------------------------------------
 // macros
@@ -42,50 +42,58 @@ public:
 
 // this is an ABC: use one of the derived classes to create a DC associated
 // with a window, screen, printer and so on
-class WXDLLEXPORT wxDC : public wxDCBase
+class WXDLLIMPEXP_CORE wxMSWDCImpl: public wxDCImpl
 {
 public:
-    wxDC(WXHDC hDC) { Init(); m_hDC = hDC; }
-    virtual ~wxDC();
+    wxMSWDCImpl(wxDC *owner, WXHDC hDC);
+    virtual ~wxMSWDCImpl();
 
     // implement base class pure virtuals
     // ----------------------------------
 
-    virtual void Clear();
+    virtual void Clear() wxOVERRIDE;
 
-    virtual bool StartDoc(const wxString& message);
-    virtual void EndDoc();
+    virtual bool StartDoc(const wxString& message) wxOVERRIDE;
+    virtual void EndDoc() wxOVERRIDE;
 
-    virtual void StartPage();
-    virtual void EndPage();
+    virtual void StartPage() wxOVERRIDE;
+    virtual void EndPage() wxOVERRIDE;
 
-    virtual void SetFont(const wxFont& font);
-    virtual void SetPen(const wxPen& pen);
-    virtual void SetBrush(const wxBrush& brush);
-    virtual void SetBackground(const wxBrush& brush);
-    virtual void SetBackgroundMode(int mode);
+    virtual void SetFont(const wxFont& font) wxOVERRIDE;
+    virtual void SetPen(const wxPen& pen) wxOVERRIDE;
+    virtual void SetBrush(const wxBrush& brush) wxOVERRIDE;
+    virtual void SetBackground(const wxBrush& brush) wxOVERRIDE;
+    virtual void SetBackgroundMode(int mode) wxOVERRIDE;
 #if wxUSE_PALETTE
-    virtual void SetPalette(const wxPalette& palette);
+    virtual void SetPalette(const wxPalette& palette) wxOVERRIDE;
 #endif // wxUSE_PALETTE
 
-    virtual void DestroyClippingRegion();
+    virtual void DestroyClippingRegion() wxOVERRIDE;
 
-    virtual wxCoord GetCharHeight() const;
-    virtual wxCoord GetCharWidth() const;
+    virtual wxCoord GetCharHeight() const wxOVERRIDE;
+    virtual wxCoord GetCharWidth() const wxOVERRIDE;
 
-    virtual bool CanDrawBitmap() const;
-    virtual bool CanGetTextExtent() const;
-    virtual int GetDepth() const;
-    virtual wxSize GetPPI() const;
+    virtual bool CanDrawBitmap() const wxOVERRIDE;
+    virtual bool CanGetTextExtent() const wxOVERRIDE;
+    virtual int GetDepth() const wxOVERRIDE;
+    virtual wxSize GetPPI() const wxOVERRIDE;
 
-    virtual void SetMapMode(int mode);
-    virtual void SetUserScale(double x, double y);
-    virtual void SetSystemScale(double x, double y);
-    virtual void SetLogicalScale(double x, double y);
-    virtual void SetLogicalOrigin(wxCoord x, wxCoord y);
-    virtual void SetDeviceOrigin(wxCoord x, wxCoord y);
-    virtual void SetAxisOrientation(bool xLeftRight, bool yBottomUp);
-    virtual void SetLogicalFunction(int function);
+
+    virtual void SetMapMode(wxMappingMode mode) wxOVERRIDE;
+    virtual void SetUserScale(double x, double y) wxOVERRIDE;
+    virtual void SetLogicalScale(double x, double y) wxOVERRIDE;
+    virtual void SetLogicalOrigin(wxCoord x, wxCoord y) wxOVERRIDE;
+    virtual void SetDeviceOrigin(wxCoord x, wxCoord y) wxOVERRIDE;
+    virtual void SetAxisOrientation(bool xLeftRight, bool yBottomUp) wxOVERRIDE;
+
+#if wxUSE_DC_TRANSFORM_MATRIX
+    virtual bool CanUseTransformMatrix() const wxOVERRIDE;
+    virtual bool SetTransformMatrix(const wxAffineMatrix2D& matrix) wxOVERRIDE;
+    virtual wxAffineMatrix2D GetTransformMatrix() const wxOVERRIDE;
+    virtual void ResetTransformMatrix() wxOVERRIDE;
+#endif // wxUSE_DC_TRANSFORM_MATRIX
+
+    virtual void SetLogicalFunction(wxRasterOperationMode function) wxOVERRIDE;
 
     // implementation from now on
     // --------------------------
@@ -93,10 +101,9 @@ public:
     virtual void SetRop(WXHDC cdc);
     virtual void SelectOldObjects(WXHDC dc);
 
-    wxWindow *GetWindow() const { return m_canvas; }
     void SetWindow(wxWindow *win)
     {
-        m_canvas = win;
+        m_window = win;
 
 #if wxUSE_PALETTE
         // if we have palettes use the correct one for this window
@@ -114,12 +121,13 @@ public:
         // return it if asked -- but avoid calling ::GetClipBox() right now as
         // it could be unnecessary wasteful
         m_clipping = true;
-        m_clipX1 =
-        m_clipX2 = 0;
+        m_isClipBoxValid = false;
     }
 
-    const wxBitmap& GetSelectedBitmap() const { return m_selectedBitmap; }
-    wxBitmap& GetSelectedBitmap() { return m_selectedBitmap; }
+    void* GetHandle() const wxOVERRIDE { return (void*)GetHDC(); }
+    
+    const wxBitmap& GetSelectedBitmap() const wxOVERRIDE { return m_selectedBitmap; }
+    wxBitmap& GetSelectedBitmap() wxOVERRIDE { return m_selectedBitmap; }
 
     // update the internal clip box variables
     void UpdateClipBox();
@@ -138,13 +146,12 @@ public:
 
     // get or change the layout direction (LTR or RTL) for this dc,
     // wxLayout_Default is returned if layout direction is not supported
-    virtual wxLayoutDirection GetLayoutDirection() const;
-    virtual void SetLayoutDirection(wxLayoutDirection dir);
+    virtual wxLayoutDirection GetLayoutDirection() const wxOVERRIDE;
+    virtual void SetLayoutDirection(wxLayoutDirection dir) wxOVERRIDE;
 
 protected:
     void Init()
     {
-        m_canvas = NULL;
         m_bOwnsDC = false;
         m_hDC = NULL;
 
@@ -156,84 +163,103 @@ protected:
 #if wxUSE_PALETTE
         m_oldPalette = NULL;
 #endif // wxUSE_PALETTE
+        m_isClipBoxValid = false;
     }
 
     // create an uninitialized DC: this should be only used by the derived
     // classes
-    wxDC() { Init(); }
+    wxMSWDCImpl( wxDC *owner ) : wxDCImpl( owner ) { Init(); }
 
+    void RealizeScaleAndOrigin();
+
+public:
+    virtual void DoGetFontMetrics(int *height,
+                                  int *ascent,
+                                  int *descent,
+                                  int *internalLeading,
+                                  int *externalLeading,
+                                  int *averageWidth) const wxOVERRIDE;
     virtual void DoGetTextExtent(const wxString& string,
                                  wxCoord *x, wxCoord *y,
                                  wxCoord *descent = NULL,
                                  wxCoord *externalLeading = NULL,
-                                 wxFont *theFont = NULL) const;
-    virtual bool DoGetPartialTextExtents(const wxString& text, wxArrayInt& widths) const;
+                                 const wxFont *theFont = NULL) const wxOVERRIDE;
+    virtual bool DoGetPartialTextExtents(const wxString& text, wxArrayInt& widths) const wxOVERRIDE;
 
     virtual bool DoFloodFill(wxCoord x, wxCoord y, const wxColour& col,
-                             int style = wxFLOOD_SURFACE);
+                             wxFloodFillStyle style = wxFLOOD_SURFACE) wxOVERRIDE;
 
     virtual void DoGradientFillLinear(const wxRect& rect,
                                       const wxColour& initialColour,
                                       const wxColour& destColour,
-                                      wxDirection nDirection = wxEAST);
+                                      wxDirection nDirection = wxEAST) wxOVERRIDE;
 
-    virtual bool DoGetPixel(wxCoord x, wxCoord y, wxColour *col) const;
+    virtual bool DoGetPixel(wxCoord x, wxCoord y, wxColour *col) const wxOVERRIDE;
 
-    virtual void DoDrawPoint(wxCoord x, wxCoord y);
-    virtual void DoDrawLine(wxCoord x1, wxCoord y1, wxCoord x2, wxCoord y2);
+    virtual void DoDrawPoint(wxCoord x, wxCoord y) wxOVERRIDE;
+    virtual void DoDrawLine(wxCoord x1, wxCoord y1, wxCoord x2, wxCoord y2) wxOVERRIDE;
 
     virtual void DoDrawArc(wxCoord x1, wxCoord y1,
                            wxCoord x2, wxCoord y2,
-                           wxCoord xc, wxCoord yc);
+                           wxCoord xc, wxCoord yc) wxOVERRIDE;
     virtual void DoDrawCheckMark(wxCoord x, wxCoord y,
-                                 wxCoord width, wxCoord height);
+                                 wxCoord width, wxCoord height) wxOVERRIDE;
     virtual void DoDrawEllipticArc(wxCoord x, wxCoord y, wxCoord w, wxCoord h,
-                                   double sa, double ea);
+                                   double sa, double ea) wxOVERRIDE;
 
-    virtual void DoDrawRectangle(wxCoord x, wxCoord y, wxCoord width, wxCoord height);
+    virtual void DoDrawRectangle(wxCoord x, wxCoord y, wxCoord width, wxCoord height) wxOVERRIDE;
     virtual void DoDrawRoundedRectangle(wxCoord x, wxCoord y,
                                         wxCoord width, wxCoord height,
-                                        double radius);
-    virtual void DoDrawEllipse(wxCoord x, wxCoord y, wxCoord width, wxCoord height);
+                                        double radius) wxOVERRIDE;
+    virtual void DoDrawEllipse(wxCoord x, wxCoord y, wxCoord width, wxCoord height) wxOVERRIDE;
 
 #if wxUSE_SPLINES
-    virtual void DoDrawSpline(wxList *points);
+    virtual void DoDrawSpline(const wxPointList *points) wxOVERRIDE;
 #endif
 
-    virtual void DoCrossHair(wxCoord x, wxCoord y);
+    virtual void DoCrossHair(wxCoord x, wxCoord y) wxOVERRIDE;
 
-    virtual void DoDrawIcon(const wxIcon& icon, wxCoord x, wxCoord y);
+    virtual void DoDrawIcon(const wxIcon& icon, wxCoord x, wxCoord y) wxOVERRIDE;
     virtual void DoDrawBitmap(const wxBitmap &bmp, wxCoord x, wxCoord y,
-                              bool useMask = false);
+                              bool useMask = false) wxOVERRIDE;
 
-    virtual void DoDrawText(const wxString& text, wxCoord x, wxCoord y);
+    virtual void DoDrawText(const wxString& text, wxCoord x, wxCoord y) wxOVERRIDE;
     virtual void DoDrawRotatedText(const wxString& text, wxCoord x, wxCoord y,
-                                   double angle);
+                                   double angle) wxOVERRIDE;
 
     virtual bool DoBlit(wxCoord xdest, wxCoord ydest, wxCoord width, wxCoord height,
                         wxDC *source, wxCoord xsrc, wxCoord ysrc,
-                        int rop = wxCOPY, bool useMask = false, wxCoord xsrcMask = wxDefaultCoord, wxCoord ysrcMask = wxDefaultCoord);
+                        wxRasterOperationMode rop = wxCOPY, bool useMask = false,
+                        wxCoord xsrcMask = wxDefaultCoord, wxCoord ysrcMask = wxDefaultCoord) wxOVERRIDE;
+
+    virtual bool DoStretchBlit(wxCoord xdest, wxCoord ydest,
+                               wxCoord dstWidth, wxCoord dstHeight,
+                               wxDC *source,
+                               wxCoord xsrc, wxCoord ysrc,
+                               wxCoord srcWidth, wxCoord srcHeight,
+                               wxRasterOperationMode rop = wxCOPY, bool useMask = false,
+                               wxCoord xsrcMask = wxDefaultCoord, wxCoord ysrcMask = wxDefaultCoord) wxOVERRIDE;
 
-    // this is gnarly - we can't even call this function DoSetClippingRegion()
-    // because of virtual function hiding
-    virtual void DoSetClippingRegionAsRegion(const wxRegion& region);
     virtual void DoSetClippingRegion(wxCoord x, wxCoord y,
-                                     wxCoord width, wxCoord height);
-    virtual void DoGetClippingBox(wxCoord *x, wxCoord *y,
-                                  wxCoord *w, wxCoord *h) const;
+                                     wxCoord width, wxCoord height) wxOVERRIDE;
+    virtual void DoSetDeviceClippingRegion(const wxRegion& region) wxOVERRIDE;
+    virtual bool DoGetClippingRect(wxRect& rect) const wxOVERRIDE;
 
-    virtual void DoGetSizeMM(int* width, int* height) const;
+    virtual void DoGetSizeMM(int* width, int* height) const wxOVERRIDE;
 
-    virtual void DoDrawLines(int n, wxPoint points[],
-                             wxCoord xoffset, wxCoord yoffset);
-    virtual void DoDrawPolygon(int n, wxPoint points[],
+    virtual void DoDrawLines(int n, const wxPoint points[],
+                             wxCoord xoffset, wxCoord yoffset) wxOVERRIDE;
+    virtual void DoDrawPolygon(int n, const wxPoint points[],
                                wxCoord xoffset, wxCoord yoffset,
-                               int fillStyle = wxODDEVEN_RULE);
-    virtual void DoDrawPolyPolygon(int n, int count[], wxPoint points[],
+                               wxPolygonFillMode fillStyle = wxODDEVEN_RULE) wxOVERRIDE;
+    virtual void DoDrawPolyPolygon(int n, const int count[], const wxPoint points[],
                                    wxCoord xoffset, wxCoord yoffset,
-                                   int fillStyle = wxODDEVEN_RULE);
-    virtual wxBitmap DoGetAsBitmap(const wxRect *subrect) const 
-    { return subrect == NULL ? GetSelectedBitmap() : GetSelectedBitmap().GetSubBitmap(*subrect); }
+                                   wxPolygonFillMode fillStyle = wxODDEVEN_RULE) wxOVERRIDE;
+    virtual wxBitmap DoGetAsBitmap(const wxRect *subrect) const wxOVERRIDE
+    {
+        return subrect == NULL ? GetSelectedBitmap()
+                               : GetSelectedBitmap().GetSubBitmap(*subrect);
+    }
 
 
 #if wxUSE_PALETTE
@@ -247,10 +273,11 @@ protected:
     void InitializePalette();
 #endif // wxUSE_PALETTE
 
+protected:
     // common part of DoDrawText() and DoDrawRotatedText()
     void DrawAnyText(const wxString& text, wxCoord x, wxCoord y);
 
-    // common part of DoSetClippingRegion() and DoSetClippingRegionAsRegion()
+    // common part of DoSetClippingRegion() and DoSetDeviceClippingRegion()
     void SetClippingHrgn(WXHRGN hrgn);
 
     // implementation of DoGetSize() for wxScreen/PrinterDC: this simply
@@ -291,12 +318,14 @@ protected:
 #endif // wxUSE_PALETTE
 
 #if wxUSE_DC_CACHEING
-    static wxList     sm_bitmapCache;
-    static wxList     sm_dcCache;
+    static wxObjectList     sm_bitmapCache;
+    static wxObjectList     sm_dcCache;
 #endif
 
-    DECLARE_DYNAMIC_CLASS(wxDC)
-    DECLARE_NO_COPY_CLASS(wxDC)
+    bool m_isClipBoxValid;
+
+    wxDECLARE_CLASS(wxMSWDCImpl);
+    wxDECLARE_NO_COPY_CLASS(wxMSWDCImpl);
 };
 
 // ----------------------------------------------------------------------------
@@ -304,28 +333,27 @@ protected:
 // only/mainly)
 // ----------------------------------------------------------------------------
 
-class WXDLLEXPORT wxDCTemp : public wxDC
+class WXDLLIMPEXP_CORE wxDCTempImpl : public wxMSWDCImpl
 {
 public:
     // construct a temporary DC with the specified HDC and size (it should be
     // specified whenever we know it for this HDC)
-    wxDCTemp(WXHDC hdc, const wxSize& size = wxDefaultSize)
-        : wxDC(hdc),
+    wxDCTempImpl(wxDC *owner, WXHDC hdc, const wxSize& size )
+        : wxMSWDCImpl( owner, hdc ),
           m_size(size)
     {
     }
 
-    virtual ~wxDCTemp()
+    virtual ~wxDCTempImpl()
     {
         // prevent base class dtor from freeing it
         SetHDC((WXHDC)NULL);
     }
 
-protected:
-    virtual void DoGetSize(int *w, int *h) const
+    virtual void DoGetSize(int *w, int *h) const wxOVERRIDE
     {
         wxASSERT_MSG( m_size.IsFullySpecified(),
-                      _T("size of this DC hadn't been set and is unknown") );
+                      wxT("size of this DC hadn't been set and is unknown") );
 
         if ( w )
             *w = m_size.x;
@@ -338,7 +366,16 @@ private:
     // find it ourselves
     const wxSize m_size;
 
-    DECLARE_NO_COPY_CLASS(wxDCTemp)
+    wxDECLARE_NO_COPY_CLASS(wxDCTempImpl);
+};
+
+class WXDLLIMPEXP_CORE wxDCTemp : public wxDC
+{
+public:
+    wxDCTemp(WXHDC hdc, const wxSize& size = wxDefaultSize)
+        : wxDC(new wxDCTempImpl(this, hdc, size))
+    {
+    }
 };
 
 #endif // _WX_MSW_DC_H_
