@@ -40,6 +40,13 @@
  */
 #define wxHAVE_FSEVENTS_FILE_NOTIFICATIONS 1
 
+/*
+ * turn off old style icon format if not asked for
+ */
+#ifndef wxOSX_USE_ICONREF
+    #define wxOSX_USE_ICONREF 0
+#endif
+
 /*
  * turning off capabilities that don't work under cocoa yet
  */
