@@ -19,17 +19,8 @@
 
 #include "wx/osx/core/cfstring.h"
 #include "wx/osx/core/cfdataref.h"
-
-// Define helper macros allowing to insert small snippets of code to be
-// compiled for high enough OS X version only: this shouldn't be abused for
-// anything big but it's handy for e.g. specifying OS X 10.6-only protocols in
-// the Objective C classes declarations when they're not supported under the
-// previous versions
-#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_6
-    #define wxOSX_10_6_AND_LATER(x) x
-#else
-    #define wxOSX_10_6_AND_LATER(x)
-#endif
+#include "wx/osx/core/cfarray.h"
+#include "wx/osx/core/cfdictionary.h"
 
 // platform specific Clang analyzer support
 #ifndef NS_RETURNS_RETAINED
@@ -54,8 +45,6 @@
 // wxBase part of it).
 #include <Carbon/Carbon.h>
 
-WXDLLIMPEXP_BASE long UMAGetSystemVersion() ;
-
 void WXDLLIMPEXP_CORE wxMacStringToPascal( const wxString&from , unsigned char * to );
 wxString WXDLLIMPEXP_CORE wxMacMakeStringFromPascal( const unsigned char * from );
 
@@ -115,8 +104,6 @@ protected :
 
 // Quartz
 
-WXDLLIMPEXP_CORE CGImageRef wxMacCreateCGImageFromBitmap( const wxBitmap& bitmap );
-
 WXDLLIMPEXP_CORE CGDataProviderRef wxMacCGDataProviderCreateWithCFData( CFDataRef data );
 WXDLLIMPEXP_CORE CGDataConsumerRef wxMacCGDataConsumerCreateWithCFData( CFMutableDataRef data );
 WXDLLIMPEXP_CORE CGDataProviderRef wxMacCGDataProviderCreateWithMemoryBuffer( const wxMemoryBuffer& buf );
@@ -125,6 +112,16 @@ WXDLLIMPEXP_CORE CGColorSpaceRef wxMacGetGenericRGBColorSpace(void);
 
 WXDLLIMPEXP_CORE double wxOSXGetMainScreenContentScaleFactor();
 
+// UI
+
+CGSize WXDLLIMPEXP_CORE wxOSXGetImageSize(WXImage image);
+CGImageRef WXDLLIMPEXP_CORE wxOSXCreateCGImageFromImage( WXImage nsimage, double *scale = NULL );
+CGImageRef WXDLLIMPEXP_CORE wxOSXGetCGImageFromImage( WXImage nsimage, CGRect* r, CGContextRef cg);
+CGContextRef WXDLLIMPEXP_CORE wxOSXCreateBitmapContextFromImage( WXImage nsimage, bool *isTemplate = NULL);
+WXImage WXDLLIMPEXP_CORE wxOSXGetImageFromCGImage( CGImageRef image, double scale = 1.0, bool isTemplate = false);
+double WXDLLIMPEXP_CORE wxOSXGetImageScaleFactor(WXImage image);
+
+
 class wxWindowMac;
 // to
 extern wxWindow* g_MacLastWindow;
@@ -138,14 +135,12 @@ class wxComboBox;
 class wxNotebook;
 class wxTextCtrl;
 class wxSearchCtrl;
+class wxMenuItem;
+class wxAcceleratorEntry;
 
 WXDLLIMPEXP_CORE wxWindowMac * wxFindWindowFromWXWidget(WXWidget inControl );
 
-#if wxOSX_USE_CARBON
-typedef wxMacControl wxWidgetImplType;
-#else
 typedef wxWidgetImpl wxWidgetImplType;
-#endif
 
 #if wxUSE_MENUS
 class wxMenuItemImpl : public wxObject
@@ -179,7 +174,7 @@ public :
 protected :
     wxMenuItem* m_peer;
 
-    DECLARE_ABSTRACT_CLASS(wxMenuItemImpl)
+    wxDECLARE_ABSTRACT_CLASS(wxMenuItemImpl);
 } ;
 
 class wxMenuImpl : public wxObject
@@ -202,13 +197,18 @@ public :
     wxMenu* GetWXPeer() { return m_peer ; }
 
     virtual void PopUp( wxWindow *win, int x, int y ) = 0;
+    
+    virtual void GetMenuBarDimensions(int &x, int &y, int &width, int &height) const
+    {
+        x = y = width = height = -1;
+    }
 
     static wxMenuImpl* Create( wxMenu* peer, const wxString& title );
     static wxMenuImpl* CreateRootMenu( wxMenu* peer );
 protected :
     wxMenu* m_peer;
 
-    DECLARE_ABSTRACT_CLASS(wxMenuItemImpl)
+    wxDECLARE_ABSTRACT_CLASS(wxMenuImpl);
 } ;
 #endif
 
@@ -300,6 +300,8 @@ public :
 #if wxUSE_MARKUP && wxOSX_USE_COCOA
     virtual void        SetLabelMarkup( const wxString& WXUNUSED(markup) ) { }
 #endif
+    virtual void        SetInitialLabel( const wxString& title, wxFontEncoding encoding )
+                            { SetLabel(title, encoding); }
 
     virtual void        SetCursor( const wxCursor & cursor ) = 0;
     virtual void        CaptureMouse() = 0;
@@ -334,6 +336,8 @@ public :
 
     virtual void        InstallEventHandler( WXWidget control = NULL ) = 0;
 
+    virtual bool        EnableTouchEvents(int eventsMask) = 0;
+
     // Mechanism used to keep track of whether a change should send an event
     // Do SendEvents(false) when starting actions that would trigger programmatic events
     // and SendEvents(true) at the end of the block.
@@ -353,6 +357,7 @@ public :
                         FindBestFromWXWidget(WXWidget control);
     
     static void         RemoveAssociations( wxWidgetImpl* impl);
+    static void         RemoveAssociation(WXWidget control);
 
     static void         Associate( WXWidget control, wxWidgetImpl *impl );
 
@@ -553,6 +558,15 @@ public :
                                     long extraStyle);
 #endif
 
+    static wxWidgetImplType*    CreateStaticBitmap( wxWindowMac* wxpeer,
+                                                   wxWindowMac* parent,
+                                                   wxWindowID id,
+                                                   const wxBitmap& bitmap,
+                                                   const wxPoint& pos,
+                                                   const wxSize& size,
+                                                   long style,
+                                                   long extraStyle);
+
     // converts from Toplevel-Content relative to local
     static void Convert( wxPoint *pt , wxWidgetImpl *from , wxWidgetImpl *to );
 protected :
@@ -563,7 +577,7 @@ protected :
     bool                m_needsFrame;
     bool                m_shouldSendEvents;
 
-    DECLARE_ABSTRACT_CLASS(wxWidgetImpl)
+    wxDECLARE_ABSTRACT_CLASS(wxWidgetImpl);
 };
 
 //
@@ -621,6 +635,8 @@ public:
     // display
 
     virtual void            ListScrollTo( unsigned int n ) = 0;
+    virtual int             ListGetTopItem() const = 0;
+    virtual int             ListGetCountPerPage() const = 0;
     virtual void            UpdateLine( unsigned int n, wxListWidgetColumn* col = NULL ) = 0;
     virtual void            UpdateLineToEnd( unsigned int n) = 0;
 
@@ -663,7 +679,10 @@ public :
 
     virtual bool CanClipMaxLength() const { return false; }
     virtual void SetMaxLength(unsigned long WXUNUSED(len)) {}
-    
+
+    virtual bool CanForceUpper() { return false; }
+    virtual void ForceUpper() {}
+
     virtual bool GetStyle( long position, wxTextAttr& style);
     virtual void SetStyle( long start, long end, const wxTextAttr& style ) ;
     virtual void Copy() ;
@@ -690,21 +709,24 @@ public :
     virtual int GetNumberOfLines() const ;
     virtual long XYToPosition(long x, long y) const;
     virtual bool PositionToXY(long pos, long *x, long *y) const ;
-    virtual void ShowPosition(long WXUNUSED(pos)) ;
+    virtual void ShowPosition(long pos) ;
     virtual int GetLineLength(long lineNo) const ;
     virtual wxString GetLineText(long lineNo) const ;
     virtual void CheckSpelling(bool WXUNUSED(check)) { }
+    virtual void EnableAutomaticQuoteSubstitution(bool WXUNUSED(enable)) {}
+    virtual void EnableAutomaticDashSubstitution(bool WXUNUSED(enable)) {}
 
     virtual wxSize GetBestSize() const { return wxDefaultSize; }
 
     virtual bool SetHint(const wxString& WXUNUSED(hint)) { return false; }
+    virtual void SetJustification();
 private:
     wxTextEntry * const m_entry;
 
     wxDECLARE_NO_COPY_CLASS(wxTextWidgetImpl);
 };
 
-// common interface for all implementations
+// common interface for all combobox implementations
 class WXDLLIMPEXP_CORE wxComboWidgetImpl
 
 {
@@ -731,6 +753,41 @@ public :
     virtual int FindString(const wxString& WXUNUSED(text)) const { return -1; }
 };
 
+//
+// common interface for choice
+//
+
+class WXDLLIMPEXP_CORE wxChoiceWidgetImpl
+
+{
+public :
+    wxChoiceWidgetImpl() {}
+
+    virtual ~wxChoiceWidgetImpl() {}
+
+    virtual int GetSelectedItem() const { return -1; }
+
+    virtual void SetSelectedItem(int WXUNUSED(item)) {}
+
+    virtual size_t GetNumberOfItems() const = 0;
+
+    virtual void InsertItem(size_t pos, int itemid, const wxString& text) = 0;
+
+    virtual void RemoveItem(size_t pos) = 0;
+
+    virtual void Clear()
+    {
+        size_t count = GetNumberOfItems();
+        for ( size_t i = 0 ; i < count ; i++ )
+        {
+            RemoveItem( 0 );
+        }
+    }
+
+    virtual void SetItem(int pos, const wxString& item) = 0;
+};
+
+
 //
 // common interface for buttons
 //
@@ -855,6 +912,10 @@ public :
 
     virtual void SetTitle( const wxString& title, wxFontEncoding encoding ) = 0;
 
+    virtual bool EnableCloseButton(bool enable) = 0;
+    virtual bool EnableMaximizeButton(bool enable) = 0;
+    virtual bool EnableMinimizeButton(bool enable) = 0;
+
     virtual bool IsMaximized() const = 0;
 
     virtual bool IsIconized() const= 0;
@@ -867,6 +928,8 @@ public :
 
     virtual void ShowWithoutActivating() { Show(true); }
 
+    virtual bool EnableFullScreenView(bool enable) = 0;
+
     virtual bool ShowFullScreen(bool show, long style)= 0;
 
     virtual void RequestUserAttention(int flags) = 0;
@@ -906,7 +969,7 @@ public :
     virtual void RestoreWindowLevel() {}
 protected :
     wxNonOwnedWindow*   m_wxPeer;
-    DECLARE_ABSTRACT_CLASS(wxNonOwnedWindowImpl)
+    wxDECLARE_ABSTRACT_CLASS(wxNonOwnedWindowImpl);
 };
 
 #endif // wxUSE_GUI
