@@ -2,7 +2,6 @@
 // Name:        cmdline.h
 // Purpose:     interface of wxCmdLineParser
 // Author:      wxWidgets team
-// RCS-ID:      $Id$
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
 
@@ -14,7 +13,7 @@
     parameter - this may be changed by giving non-default flags to it, i.e. use
     @c wxCMD_LINE_OPTION_MANDATORY to require that the option is given and
     @c wxCMD_LINE_PARAM_OPTIONAL to make a parameter optional.
-    
+
     Also, @c wxCMD_LINE_PARAM_MULTIPLE may be specified if the programs accepts a
     variable number of parameters - but it only can be given for the last
     parameter in the command line description. If you use this flag, you will
@@ -25,6 +24,17 @@
     @c wxCMD_LINE_NEEDS_SEPARATOR can be specified to require a separator (either
     a colon, an equal sign or white space) between the option name and its
     value. By default, no separator is required.
+
+    @c wxCMD_LINE_SWITCH_NEGATABLE can be specified if you want to allow the
+    user to specify the switch in both normal form and in negated one (e.g.
+    /R-). You will need to use wxCmdLineParser::FoundSwitch() to distinguish
+    between the normal and negated forms of the switch. This flag is new since
+    wxWidgets 2.9.2.
+
+    @c wxCMD_LINE_HIDDEN can be specified for arguments that should exist but
+    are not to be included in the output of Usage(). These could be, for
+    example, diagnostics switches that are not useful to the end user.
+    This flags is new since wxWidgets 3.1.1.
 */
 enum wxCmdLineEntryFlags
 {
@@ -32,7 +42,9 @@ enum wxCmdLineEntryFlags
     wxCMD_LINE_PARAM_OPTIONAL   = 0x02, ///< The parameter may be omitted.
     wxCMD_LINE_PARAM_MULTIPLE   = 0x04, ///< The parameter may be repeated.
     wxCMD_LINE_OPTION_HELP      = 0x08, ///< This option is a help request.
-    wxCMD_LINE_NEEDS_SEPARATOR  = 0x10  ///< Must have a separator before the value.
+    wxCMD_LINE_NEEDS_SEPARATOR  = 0x10, ///< Must have a separator before the value.
+    wxCMD_LINE_SWITCH_NEGATABLE = 0x20, ///< This switch can be negated (e.g. /S-)
+    wxCMD_LINE_HIDDEN           = 0x40  ///< This switch is not listed by Usage()
 };
 
 /**
@@ -55,20 +67,38 @@ enum wxCmdLineEntryType
 {
     /// A boolean argument of the program; e.g. @c -v to enable verbose mode.
     wxCMD_LINE_SWITCH,
-    
+
     /// An argument with an associated value; e.g. @c "-o filename" to specify
     /// an optional output filename.
     wxCMD_LINE_OPTION,
-    
+
     /// A parameter: a required program argument.
     wxCMD_LINE_PARAM,
-    
+
     /// Additional usage text. See wxCmdLineParser::AddUsageText.
     wxCMD_LINE_USAGE_TEXT,
-    
+
     wxCMD_LINE_NONE     ///< Use this to terminate the list.
 };
 
+/**
+    The state of a switch as returned by wxCmdLineParser::FoundSwitch().
+
+    @since 2.9.2
+*/
+enum wxCmdLineSwitchState
+{
+    /// The switch was found in negated form, i.e. followed by a '-'.
+    wxCMD_SWITCH_OFF,
+
+    /// The switch was not found at all on the command line.
+    wxCMD_SWITCH_NOT_FOUND
+
+    /// The switch was found (and was not negated)
+    wxCMD_SWITCH_ON
+};
+
+
 /**
     Flags determining wxCmdLineParser::ConvertStringToArgs() behaviour.
  */
@@ -96,7 +126,7 @@ struct wxCmdLineEntryDesc
 
     /**
         The usual, short, name of the switch or the option.
-        
+
         It may contain only letters, digits and the underscores.
         This field is unused if <tt>kind == wxCMD_LINE_PARAM</tt>.
     */
@@ -105,7 +135,7 @@ struct wxCmdLineEntryDesc
     /**
         The long name for this program argument (may be empty if the option
         has no long name).
-        
+
         It may contain only letters, digits and the underscores.
         This field is unused if <tt>kind == wxCMD_LINE_PARAM</tt>.
     */
@@ -122,13 +152,176 @@ struct wxCmdLineEntryDesc
         See ::wxCmdLineParamType for more info.
     */
     wxCmdLineParamType type;
-    
+
     /**
         A combination of one or more ::wxCmdLineEntryFlags enum values.
     */
     int flags;
 };
 
+/**
+    The interface wxCmdLineArg provides information for an instance of argument
+    passed on command line.
+
+    Example of use:
+
+    @code
+    wxCmdLineParser parser;
+
+    for (wxCmdLineArgs::const_iterator itarg=parser.GetArguments().begin();
+                                       itarg!=parser.GetArguments().end();
+                                       ++itarg)
+    {
+        wxString optionName;
+        switch (itarg->GetKind())
+        {
+        case wxCMD_LINE_SWITCH:
+            if (itarg->IsNegated()) {
+            }
+            else {
+            }
+            break;
+
+        case wxCMD_LINE_OPTION:
+            // assuming that all the options have a short name
+            optionName = itarg->GetShortName();
+
+            switch (itarg->GetType()) {
+                case wxCMD_LINE_VAL_NUMBER:
+                    // do something with itarg->GetLongVal();
+                    break;
+
+                case wxCMD_LINE_VAL_DOUBLE:
+                    // do something with itarg->GetDoubleVal();
+                    break;
+
+                case wxCMD_LINE_VAL_DATE:
+                    // do something with itarg->GetDateVal();
+                    break;
+
+                case wxCMD_LINE_VAL_STRING:
+                    // do something with itarg->GetStrVal();
+                    break;
+            }
+            break;
+
+        case wxCMD_LINE_PARAM:
+            // do something with itarg->GetStrVal();
+            break;
+        }
+    }
+    @endcode
+
+    With C++11, the for loop could be written:
+    @code
+    for (const auto &arg : parser.GetArguments()) {
+        // working on arg as with *itarg above
+    }
+    @endcode
+
+    @since 3.1.0
+*/
+class wxCmdLineArg
+{
+public:
+    virtual ~wxCmdLineArg() {}
+
+    /**
+        Returns the command line argument value as a wxDateTime.
+
+        @note This call works only for @c wxCMD_LINE_VAL_DATE options
+    */
+    virtual const wxDateTime& GetDateVal() const = 0;
+
+    /**
+        Returns the command line argument value as a double.
+
+        @note This call works only for @c wxCMD_LINE_VAL_DOUBLE options
+    */
+    virtual double GetDoubleVal() const = 0;
+
+    /**
+        Returns the command line argument entry kind.
+
+        @note Parameters can only be retrieved as strings, with GetStrVal()
+
+        @see wxCmdLineEntryType, GetType()
+    */
+    virtual wxCmdLineEntryType GetKind() const = 0;
+
+    /**
+        Returns the command line argument value as a long.
+
+        @note This call works only for @c wxCMD_LINE_VAL_NUMBER options
+    */
+    virtual long GetLongVal() const = 0;
+
+    /**
+        Returns the command line argument long name if any.
+
+        @note This call makes sense only for options and switches
+    */
+    virtual wxString GetLongName() const = 0;
+
+    /**
+        Returns the command line argument short name if any.
+
+        @note This call makes sense only for options and switches
+    */
+    virtual wxString GetShortName() const = 0;
+
+    /**
+        Returns the command line argument value as a string.
+
+        @note This call works only for @c wxCMD_LINE_VAL_STRING options
+        and parameters
+    */
+    virtual const wxString& GetStrVal() const = 0;
+
+    /**
+        Returns the command line argument parameter type
+
+        @note This call makes sense only for options
+        (i.e. GetKind() == @c wxCMD_LINE_OPTION).
+
+        @see wxCmdLineParamType, GetKind()
+    */
+    virtual wxCmdLineParamType GetType() const = 0;
+
+    /**
+        Returns true if the switch was negated
+
+        @note This call works only for switches.
+    */
+    virtual bool IsNegated() const = 0;
+};
+
+/**
+    An ordered collection of wxCmdLineArg providing an iterator to enumerate
+    the arguments passed on command line.
+
+    @see wxCmdLineParser::GetArguments()
+
+    @since 3.1.0
+*/
+class wxCmdLineArgs
+{
+public:
+    /**
+        A bidirectional constant iterator exposing the command line arguments as
+        wxCmdLineArg references.
+    */
+    class const_iterator;
+
+    const_iterator begin() const;
+    const_iterator end() const;
+
+    /**
+        Returns the number of command line arguments in this collection.
+    */
+    size_t size() const;
+};
+
 /**
     @class wxCmdLineParser
 
@@ -155,12 +348,12 @@ struct wxCmdLineEntryDesc
 
     In the documentation below the following terminology is used:
 
-    - @b switch: a boolean option which can be given or not, but which doesn't have 
+    - @b switch: a boolean option which can be given or not, but which doesn't have
                  any value. We use the word @e switch to distinguish
                  such boolean options from more generic options like those
                  described below. For example, @c "-v" might be a switch
                  meaning "enable verbose mode".
-    - @b option: a switch with a value associated to it. 
+    - @b option: a switch with a value associated to it.
                  For example, @c "-o filename" might be an
                  option for specifying the name of the output file.
     - @b parameter: a required program argument.
@@ -201,7 +394,7 @@ struct wxCmdLineEntryDesc
 
     First global option is the support for long (also known as GNU-style)
     options. The long options are the ones which start with two dashes and look
-    like "--verbose", i.e. they generally are complete words and not some
+    like "\--verbose", i.e. they generally are complete words and not some
     abbreviations of them. As long options are used by more and more
     applications, they are enabled by default, but may be disabled with
     DisableLongOptions().
@@ -311,6 +504,32 @@ public:
     */
     ~wxCmdLineParser();
 
+    /**
+        Adds an option with only long form.
+
+        This is just a convenient wrapper for AddOption() passing an empty
+        string as short option name.
+
+        @since 2.9.3
+     */
+    void AddLongOption(const wxString& lng,
+                       const wxString& desc = wxEmptyString,
+                       wxCmdLineParamType type = wxCMD_LINE_VAL_STRING,
+                       int flags = 0);
+
+    /**
+        Adds a switch with only long form.
+
+        This is just a convenient wrapper for AddSwitch() passing an empty
+        string as short switch name.
+
+        @since 2.9.3
+     */
+
+    void AddLongSwitch(const wxString& lng,
+                       const wxString& desc = wxEmptyString,
+                       int flags = 0);
+
     /**
         Add an option @a name with an optional long name @a lng (no long name
         if it is empty, which is default) taking a value of the given type
@@ -389,6 +608,26 @@ public:
     */
     bool Found(const wxString& name) const;
 
+    /**
+        Returns whether the switch was found on the command line and whether it
+        was negated.
+
+        This method can be used for any kind of switch but is especially useful
+        for switches that can be negated, i.e. were added with
+        wxCMD_LINE_SWITCH_NEGATABLE flag, as otherwise Found() is simpler to
+        use.
+
+        However Found() doesn't allow to distinguish between switch specified
+        normally, i.e. without dash following it, and negated switch, i.e. with
+        the following dash. This method will return @c wxCMD_SWITCH_ON or @c
+        wxCMD_SWITCH_OFF depending on whether the switch was negated or not.
+        And if the switch was not found at all, @c wxCMD_SWITCH_NOT_FOUND is
+        returned.
+
+        @since 2.9.2
+    */
+    wxCmdLineSwitchState FoundSwitch(const wxString& name) const;
+
     /**
         Returns true if an option taking a string value was found and stores
         the value in the provided pointer (which should not be @NULL).
@@ -425,7 +664,18 @@ public:
     size_t GetParamCount() const;
 
     /**
-        Parse the command line, return 0 if ok, -1 if @c "-h" or @c "--help"
+        Returns the collection of arguments
+
+        @note The returned object just refers to the command line parser. The
+        command line parser must live longer than it.
+
+        @see wxCmdLineArgs
+        @since 3.1.0
+    */
+    wxCmdLineArgs GetArguments() const;
+
+    /**
+        Parse the command line, return 0 if ok, -1 if @c "-h" or @c "\--help"
         option was encountered and the help message was given or a positive
         value if a syntax error occurred.
 
