@@ -28,6 +28,12 @@ of MSW, MAC and OS2
     #define USE_WXUNIVERSAL 0
 #endif
 
+#ifdef WXUSINGDLL
+    #define USE_DLL 1
+#else
+    #define USE_DLL 0
+#endif
+
 #if defined(__WXMSW__)
     #define USE_WXMSW 1
 #else
@@ -52,14 +58,16 @@ of MSW, MAC and OS2
     #define USE_WXGTK 0
 #endif
 
+#define USE_GENERIC_DIALOGS (!USE_WXUNIVERSAL && !USE_DLL)
+
 #define USE_COLOURDLG_GENERIC \
-    ((USE_WXMSW || USE_WXMAC) && USE_WXUNIVERSAL && wxUSE_COLOURDLG)
+    ((USE_WXMSW || USE_WXMAC) && USE_GENERIC_DIALOGS && wxUSE_COLOURDLG)
 #define USE_DIRDLG_GENERIC \
-    ((USE_WXMSW || USE_WXMAC) && USE_WXUNIVERSAL && wxUSE_DIRDLG)
+    ((USE_WXMSW || USE_WXMAC) && USE_GENERIC_DIALOGS && wxUSE_DIRDLG)
 #define USE_FILEDLG_GENERIC \
-    ((USE_WXMSW || USE_WXMAC) && USE_WXUNIVERSAL  && wxUSE_FILEDLG)
+    ((USE_WXMSW || USE_WXMAC) && USE_GENERIC_DIALOGS  && wxUSE_FILEDLG)
 #define USE_FONTDLG_GENERIC \
-    ((USE_WXMSW || USE_WXMACFONTDLG) && USE_WXUNIVERSAL && wxUSE_FONTDLG)
+    ((USE_WXMSW || USE_WXMACFONTDLG) && USE_GENERIC_DIALOGS && wxUSE_FONTDLG)
 
 // Turn USE_MODAL_PRESENTATION to 0 if there is any reason for not presenting difference
 // between modal and modeless dialogs (ie. not implemented it in your port yet)
