@@ -4,7 +4,6 @@
 // Author:      Julian Smart
 // Modified by: Vadim Zeitlin: merge with the MDI version and general cleanup
 // Created:     04/01/98
-// RCS-ID:      $Id$
 // Copyright:   (c) 1998 Julian Smart
 //              (c) 2008 Vadim Zeitlin
 // Licence:     wxWindows licence
@@ -50,17 +49,22 @@
 #include "wx/docview.h"
 #include "wx/docmdi.h"
 
+#if wxUSE_AUI
+    #include "wx/aui/tabmdi.h"
+#endif // wxUSE_AUI
+
 #include "docview.h"
 #include "doc.h"
 #include "view.h"
 
 #include "wx/cmdline.h"
+#include "wx/config.h"
 
 #ifdef __WXMAC__
     #include "wx/filename.h"
 #endif
 
-#if !defined(__WXMSW__) && !defined(__WXPM__)
+#ifndef wxHAS_IMAGES_IN_RESOURCES
     #include "doc.xpm"
     #include "chart.xpm"
     #include "notepad.xpm"
@@ -70,11 +74,11 @@
 // MyApp implementation
 // ----------------------------------------------------------------------------
 
-IMPLEMENT_APP(MyApp)
+wxIMPLEMENT_APP(MyApp);
 
-BEGIN_EVENT_TABLE(MyApp, wxApp)
+wxBEGIN_EVENT_TABLE(MyApp, wxApp)
     EVT_MENU(wxID_ABOUT, MyApp::OnAbout)
-END_EVENT_TABLE()
+wxEND_EVENT_TABLE()
 
 MyApp::MyApp()
 {
@@ -92,7 +96,12 @@ MyApp::MyApp()
 namespace CmdLineOption
 {
 
+#if wxUSE_MDI_ARCHITECTURE
 const char * const MDI = "mdi";
+#endif // wxUSE_MDI_ARCHITECTURE
+#if wxUSE_AUI
+const char * const AUI = "aui";
+#endif // wxUSE_AUI
 const char * const SDI = "sdi";
 const char * const SINGLE = "single";
 
@@ -102,12 +111,22 @@ void MyApp::OnInitCmdLine(wxCmdLineParser& parser)
 {
     wxApp::OnInitCmdLine(parser);
 
+#if wxUSE_MDI_ARCHITECTURE
     parser.AddSwitch("", CmdLineOption::MDI,
                      "run in MDI mode: multiple documents, single window");
+#endif // wxUSE_MDI_ARCHITECTURE
+#if wxUSE_AUI
+    parser.AddSwitch("", CmdLineOption::AUI,
+                     "run in MDI mode using AUI: multiple documents, single window");
+#endif // wxUSE_AUI
     parser.AddSwitch("", CmdLineOption::SDI,
                      "run in SDI mode: multiple documents, multiple windows");
     parser.AddSwitch("", CmdLineOption::SINGLE,
                      "run in single document mode");
+
+    parser.AddParam("document-file",
+                    wxCMD_LINE_VAL_STRING,
+                    wxCMD_LINE_PARAM_MULTIPLE | wxCMD_LINE_PARAM_OPTIONAL);
 }
 
 bool MyApp::OnCmdLineParsed(wxCmdLineParser& parser)
@@ -122,6 +141,14 @@ bool MyApp::OnCmdLineParsed(wxCmdLineParser& parser)
     }
 #endif // wxUSE_MDI_ARCHITECTURE
 
+#if wxUSE_AUI
+    if ( parser.Found(CmdLineOption::AUI) )
+    {
+        m_mode = Mode_AUI;
+        numModeOptions++;
+    }
+#endif // wxUSE_AUI
+
     if ( parser.Found(CmdLineOption::SDI) )
     {
         m_mode = Mode_SDI;
@@ -140,9 +167,21 @@ bool MyApp::OnCmdLineParsed(wxCmdLineParser& parser)
         return false;
     }
 
+    // save any files given on the command line: we'll open them in OnInit()
+    // later, after creating the frame
+    for ( size_t i = 0; i != parser.GetParamCount(); ++i )
+        m_filesFromCmdLine.push_back(parser.GetParam(i));
+
     return wxApp::OnCmdLineParsed(parser);
 }
 
+#ifdef __WXMAC__
+void MyApp::MacNewFile()
+{
+    wxDocManager::GetDocumentManager()->CreateNewDocument();
+}
+#endif // __WXMAC__
+
 bool MyApp::OnInit()
 {
     if ( !wxApp::OnInit() )
@@ -150,7 +189,10 @@ bool MyApp::OnInit()
 
     ::wxInitAllImageHandlers();
 
-    SetAppName("DocView Sample");
+    // Fill in the application information fields before creating wxConfig.
+    SetVendorName("wxWidgets");
+    SetAppName("wx_docview_sample");
+    SetAppDisplayName("wxWidgets DocView Sample");
 
     //// Create a document manager
     wxDocManager *docManager = new wxDocManager;
@@ -159,9 +201,6 @@ bool MyApp::OnInit()
     new wxDocTemplate(docManager, "Drawing", "*.drw", "", "drw",
                       "Drawing Doc", "Drawing View",
                       CLASSINFO(DrawingDocument), CLASSINFO(DrawingView));
-#if defined( __WXMAC__ )  && wxOSX_USE_CARBON
-    wxFileName::MacRegisterDefaultTypeAndCreator("drw" , 'WXMB' , 'WXMA');
-#endif
 
     if ( m_mode == Mode_Single )
     {
@@ -175,9 +214,6 @@ bool MyApp::OnInit()
         new wxDocTemplate(docManager, "Text", "*.txt;*.text", "", "txt;text",
                           "Text Doc", "Text View",
                           CLASSINFO(TextEditDocument), CLASSINFO(TextEditView));
-#if defined( __WXMAC__ ) && wxOSX_USE_CARBON
-        wxFileName::MacRegisterDefaultTypeAndCreator("txt" , 'TEXT' , 'WXMA');
-#endif
         // Create a template relating image documents to their views
         new wxDocTemplate(docManager, "Image", "*.png;*.jpg", "", "png;jpg",
                           "Image Doc", "Image View",
@@ -185,22 +221,37 @@ bool MyApp::OnInit()
     }
 
     // create the main frame window
-    wxFrame *frame;
+    wxFrame *frame = NULL;
+    switch ( m_mode )
+    {
 #if wxUSE_MDI_ARCHITECTURE
-    if ( m_mode == Mode_MDI )
-    {
-        frame = new wxDocMDIParentFrame(docManager, NULL, wxID_ANY,
-                                        GetAppDisplayName(),
-                                        wxDefaultPosition,
-                                        wxSize(500, 400));
-    }
-    else
+        case Mode_MDI:
+            frame = new wxDocMDIParentFrame(docManager, NULL, wxID_ANY,
+                                            GetAppDisplayName(),
+                                            wxDefaultPosition,
+                                            wxSize(500, 400));
+            break;
 #endif // wxUSE_MDI_ARCHITECTURE
-    {
-        frame = new wxDocParentFrame(docManager, NULL, wxID_ANY,
-                                     GetAppDisplayName(),
-                                     wxDefaultPosition,
-                                     wxSize(500, 400));
+
+#if wxUSE_AUI
+        case Mode_AUI:
+            frame = new wxDocParentFrameAny<wxAuiMDIParentFrame>
+                        (
+                            docManager, NULL, wxID_ANY,
+                            GetAppDisplayName(),
+                            wxDefaultPosition,
+                            wxSize(500, 400)
+                        );
+            break;
+#endif // wxUSE_AUI
+
+        case Mode_SDI:
+        case Mode_Single:
+            frame = new wxDocParentFrame(docManager, NULL, wxID_ANY,
+                                         GetAppDisplayName(),
+                                         wxDefaultPosition,
+                                         wxSize(500, 400));
+            break;
     }
 
     // and its menu bar
@@ -217,27 +268,43 @@ bool MyApp::OnInit()
 
     // A nice touch: a history of files visited. Use this menu.
     docManager->FileHistoryUseMenu(menuFile);
+#if wxUSE_CONFIG
+    docManager->FileHistoryLoad(*wxConfig::Get());
+#endif // wxUSE_CONFIG
+
 
     if ( m_mode == Mode_Single )
     {
         m_canvas = new MyCanvas(NULL, frame);
         m_menuEdit = CreateDrawingEditMenu();
-        docManager->CreateNewDocument();
     }
 
     CreateMenuBarForFrame(frame, menuFile, m_menuEdit);
 
     frame->SetIcon(wxICON(doc));
-    frame->Centre(wxBOTH);
-    frame->Show(true);
+    frame->Centre();
+    frame->Show();
+
+    if ( m_filesFromCmdLine.empty() )
+    {
+        docManager->CreateNewDocument();
+    }
+    else // we have files to open on command line
+    {
+        for ( size_t i = 0; i != m_filesFromCmdLine.size(); ++i )
+            docManager->CreateDocument(m_filesFromCmdLine[i], wxDOC_SILENT);
+    }
 
-    SetTopWindow(frame);
     return true;
 }
 
 int MyApp::OnExit()
 {
-    delete wxDocManager::GetDocumentManager();
+    wxDocManager * const manager = wxDocManager::GetDocumentManager();
+#if wxUSE_CONFIG
+    manager->FileHistorySave(*wxConfig::Get());
+#endif // wxUSE_CONFIG
+    delete manager;
 
     return wxApp::OnExit();
 }
@@ -285,39 +352,58 @@ void MyApp::CreateMenuBarForFrame(wxFrame *frame, wxMenu *file, wxMenu *edit)
     frame->SetMenuBar(menubar);
 }
 
-wxFrame *MyApp::CreateChildFrame(wxDocument *doc, wxView *view, bool isCanvas)
+wxFrame *MyApp::CreateChildFrame(wxView *view, bool isCanvas)
 {
     // create a child frame of appropriate class for the current mode
-    wxFrame *subframe;
+    wxFrame *subframe = NULL;
+    wxDocument *doc = view->GetDocument();
+    switch ( GetMode() )
 #if wxUSE_MDI_ARCHITECTURE
-    if ( GetMode() == Mode_MDI )
     {
-        subframe = new wxDocMDIChildFrame
-                       (
-                            doc,
-                            view,
-                            wxStaticCast(GetTopWindow(), wxDocMDIParentFrame),
-                            wxID_ANY,
-                            "Child Frame",
-                            wxDefaultPosition,
-                            wxSize(300, 300)
-                       );
-    }
-    else
+        case Mode_MDI:
+            subframe = new wxDocMDIChildFrame
+                           (
+                                doc,
+                                view,
+                                wxStaticCast(GetTopWindow(), wxDocMDIParentFrame),
+                                wxID_ANY,
+                                "Child Frame",
+                                wxDefaultPosition,
+                                wxSize(300, 300)
+                           );
+            break;
 #endif // wxUSE_MDI_ARCHITECTURE
-    {
-        subframe = new wxDocChildFrame
-                       (
-                            doc,
-                            view,
-                            wxStaticCast(GetTopWindow(), wxDocParentFrame),
-                            wxID_ANY,
-                            "Child Frame",
-                            wxDefaultPosition,
-                            wxSize(300, 300)
-                       );
 
-        subframe->Centre(wxBOTH);
+#if wxUSE_AUI
+        case Mode_AUI:
+            subframe = new wxDocChildFrameAny<wxAuiMDIChildFrame, wxAuiMDIParentFrame>
+                           (
+                                doc,
+                                view,
+                                wxStaticCast(GetTopWindow(), wxAuiMDIParentFrame),
+                                wxID_ANY,
+                                "Child Frame",
+                                wxDefaultPosition,
+                                wxSize(300, 300)
+                           );
+            break;
+#endif // wxUSE_AUI
+
+        case Mode_SDI:
+        case Mode_Single:
+            subframe = new wxDocChildFrame
+                           (
+                                doc,
+                                view,
+                                wxStaticCast(GetTopWindow(), wxDocParentFrame),
+                                wxID_ANY,
+                                "Child Frame",
+                                wxDefaultPosition,
+                                wxSize(300, 300)
+                           );
+
+            subframe->Centre();
+            break;
     }
 
     wxMenu *menuFile = new wxMenu;
@@ -362,6 +448,12 @@ void MyApp::OnAbout(wxCommandEvent& WXUNUSED(event))
             break;
 #endif // wxUSE_MDI_ARCHITECTURE
 
+#if wxUSE_AUI
+        case Mode_AUI:
+            modeName = "AUI";
+            break;
+#endif // wxUSE_AUI
+
         case Mode_SDI:
             modeName = "SDI";
             break;
@@ -374,15 +466,19 @@ void MyApp::OnAbout(wxCommandEvent& WXUNUSED(event))
             wxFAIL_MSG( "unknown mode ");
     }
 
+    const int docsCount =
+        wxDocManager::GetDocumentManager()->GetDocumentsVector().size();
+
     wxLogMessage
     (
         "This is the wxWidgets Document/View Sample\n"
         "running in %s mode.\n"
+        "%d open documents.\n"
         "\n"
         "Authors: Julian Smart, Vadim Zeitlin\n"
         "\n"
-        "Usage: docview [--{mdi,sdi,single}]",
-        modeName
+        "Usage: docview [--{mdi,aui,sdi,single}]",
+        modeName,
+        docsCount
     );
 }
-
