@@ -35,6 +35,7 @@
 
 #include <wx/statline.h>
 #include <wx/log.h>
+#include "gestures.h"
 
 // ----------------------------------------------------------------------------
 // event constants
@@ -91,8 +92,7 @@ public:
         : wxButton(parent, BUTTON_ID, label)
     {
         // Add a dynamic handler for this button event to button itself
-        Connect(wxEVT_BUTTON,
-                wxCommandEventHandler(MyEvtTestButton::OnClickDynamicHandler));
+        Bind(wxEVT_BUTTON, &MyEvtTestButton::OnClickDynamicHandler, this);
     }
 
 private:
@@ -144,6 +144,9 @@ public:
     void OnClickDynamicHandlerButton(wxCommandEvent& event);
     void OnClickStaticHandlerFrame(wxCommandEvent& event);
 
+    // Gesture
+    void OnGesture(wxCommandEvent& event);
+
 private:
     // symbolic names for the status bar fields
     enum
@@ -178,6 +181,8 @@ private:
     // the button used to highlight the event handlers execution order
     MyEvtTestButton *m_testBtn;
 
+    wxWindowRef m_gestureFrame;
+
 
     // any class wishing to process wxWidgets events must use this macro
     wxDECLARE_EVENT_TABLE();
@@ -191,7 +196,7 @@ public:
 
     void OnTest(wxCommandEvent& event)
     {
-        wxLogMessage(wxT("This is the pushed test event handler #%u"), m_level);
+        wxLogMessage("This is the pushed test event handler #%u", m_level);
 
         // if we don't skip the event, the other event handlers won't get it:
         // try commenting out this line and see what changes
@@ -220,7 +225,8 @@ enum
     Event_Push,
     Event_Pop,
     Event_Custom,
-    Event_Test
+    Event_Test,
+    Event_Gesture
 };
 
 // ----------------------------------------------------------------------------
@@ -252,6 +258,7 @@ wxBEGIN_EVENT_TABLE(MyFrame, wxFrame)
     EVT_MENU(Event_Test, MyFrame::OnTest)
     EVT_MENU(Event_Push, MyFrame::OnPushEventHandler)
     EVT_MENU(Event_Pop, MyFrame::OnPopEventHandler)
+    EVT_MENU(Event_Gesture, MyFrame::OnGesture)
 
     EVT_UPDATE_UI(Event_Pop, MyFrame::OnUpdateUIPop)
 
@@ -291,7 +298,7 @@ bool MyApp::OnInit()
         return false;
 
     // create the main application window
-    MyFrame *frame = new MyFrame(wxT("Event wxWidgets Sample"),
+    MyFrame *frame = new MyFrame("Event wxWidgets Sample",
                                  wxPoint(50, 50), wxSize(600, 340));
 
     // and show it (the frames, unlike simple controls, are not shown when
@@ -299,8 +306,8 @@ bool MyApp::OnInit()
     frame->Show(true);
 
     // Add a dynamic handler at the application level for the test button
-    Connect(MyEvtTestButton::BUTTON_ID, wxEVT_BUTTON,
-            wxCommandEventHandler(MyApp::OnClickDynamicHandlerApp));
+    Bind(wxEVT_BUTTON, &MyApp::OnClickDynamicHandlerApp, this,
+         MyEvtTestButton::BUTTON_ID);
 
     // success: wxApp::OnRun() will be called which will enter the main message
     // loop and the application will run. If we returned false here, the
@@ -360,41 +367,43 @@ MyFrame::MyFrame(const wxString& title, const wxPoint& pos, const wxSize& size)
     // create a menu bar
     wxMenu *menuFile = new wxMenu;
 
-    menuFile->Append(Event_About, wxT("&About\tCtrl-A"), wxT("Show about dialog"));
+    menuFile->Append(Event_About, "&About\tCtrl-A", "Show about dialog");
     menuFile->AppendSeparator();
-    menuFile->Append(Event_Quit, wxT("E&xit\tAlt-X"), wxT("Quit this program"));
+    menuFile->Append(Event_Quit, "E&xit\tAlt-X", "Quit this program");
 
     wxMenu *menuEvent = new wxMenu;
     menuEvent->AppendCheckItem(Event_Bind, "&Bind\tCtrl-B",
                                "Bind or unbind a dynamic event handler");
-    menuEvent->AppendCheckItem(Event_Connect, wxT("&Connect\tCtrl-C"),
-                     wxT("Connect or disconnect the dynamic event handler"));
-    menuEvent->Append(Event_Dynamic, wxT("&Dynamic event\tCtrl-D"),
-                      wxT("Dynamic event sample - only works after Connect"));
+    menuEvent->AppendCheckItem(Event_Connect, "&Connect\tCtrl-C",
+                     "Connect or disconnect the dynamic event handler");
+    menuEvent->Append(Event_Dynamic, "&Dynamic event\tCtrl-D",
+                      "Dynamic event sample - only works after Connect");
     menuEvent->AppendSeparator();
-    menuEvent->Append(Event_Push, wxT("&Push event handler\tCtrl-P"),
-                      wxT("Push event handler for test event"));
-    menuEvent->Append(Event_Pop, wxT("P&op event handler\tCtrl-O"),
-                      wxT("Pop event handler for test event"));
-    menuEvent->Append(Event_Test, wxT("Test event\tCtrl-T"),
-                      wxT("Test event processed by pushed event handler"));
+    menuEvent->Append(Event_Push, "&Push event handler\tCtrl-P",
+                      "Push event handler for test event");
+    menuEvent->Append(Event_Pop, "P&op event handler\tCtrl-O",
+                      "Pop event handler for test event");
+    menuEvent->Append(Event_Test, "Test event\tCtrl-T",
+                      "Test event processed by pushed event handler");
     menuEvent->AppendSeparator();
-    menuEvent->Append(Event_Custom, wxT("Fire c&ustom event\tCtrl-U"),
-                      wxT("Generate a custom event"));
+    menuEvent->Append(Event_Custom, "Fire c&ustom event\tCtrl-U",
+                      "Generate a custom event");
+    menuEvent->Append(Event_Gesture, "&Gesture events\tCtrl-G",
+                    "Gesture event");
 
     // now append the freshly created menu to the menu bar...
     wxMenuBar *menuBar = new wxMenuBar();
-    menuBar->Append(menuFile, wxT("&File"));
-    menuBar->Append(menuEvent, wxT("&Event"));
+    menuBar->Append(menuFile, "&File");
+    menuBar->Append(menuEvent, "&Event");
 
     // ... and attach this menu bar to the frame
     SetMenuBar(menuBar);
 
 #if wxUSE_STATUSBAR
     CreateStatusBar(3);
-    SetStatusText(wxT("Welcome to wxWidgets event sample"));
-    SetStatusText(wxT("Dynamic: off"), Status_Dynamic);
-    SetStatusText(wxT("Push count: 0"), Status_Push);
+    SetStatusText("Welcome to wxWidgets event sample");
+    SetStatusText("Dynamic: off", Status_Dynamic);
+    SetStatusText("Push count: 0", Status_Push);
 #endif // wxUSE_STATUSBAR
 
     wxPanel * const panel = new wxPanel(this);
@@ -417,15 +426,12 @@ MyFrame::MyFrame(const wxString& title, const wxPoint& pos, const wxSize& size)
     // event handlers (see class definition);
 
     // Add a dynamic handler for this button event in the parent frame
-    Connect(m_testBtn->GetId(), wxEVT_BUTTON,
-            wxCommandEventHandler(MyFrame::OnClickDynamicHandlerFrame));
+    Bind(wxEVT_BUTTON, &MyFrame::OnClickDynamicHandlerFrame, this,
+         m_testBtn->GetId());
 
     // Bind a method of this frame (notice "this" argument!) to the button
     // itself
-    m_testBtn->Connect(wxEVT_BUTTON,
-                       wxCommandEventHandler(MyFrame::OnClickDynamicHandlerButton),
-                       NULL,
-                       this);
+    m_testBtn->Bind(wxEVT_BUTTON, &MyFrame::OnClickDynamicHandlerButton, this);
 
     mainSizer->Add(m_testBtn);
     panel->SetSizer(mainSizer);
@@ -447,7 +453,8 @@ MyFrame::~MyFrame()
 
 void MyFrame::OnQuit(wxCommandEvent& WXUNUSED(event))
 {
-    // true is to force the frame to close
+    if ( m_gestureFrame )
+        m_gestureFrame->Close(true);
     Close(true);
 }
 
@@ -534,19 +541,13 @@ void MyFrame::OnConnect(wxCommandEvent& event)
 {
     if ( event.IsChecked() )
     {
-        m_btnDynamic->Connect(wxID_ANY, wxEVT_BUTTON,
-                              wxCommandEventHandler(MyFrame::OnDynamic),
-                              NULL, this);
-        Connect(Event_Dynamic, wxEVT_MENU,
-                wxCommandEventHandler(MyFrame::OnDynamic));
+        m_btnDynamic->Bind(wxEVT_BUTTON, &MyFrame::OnDynamic, this);
+        Bind(wxEVT_MENU, &MyFrame::OnDynamic, this, Event_Dynamic);
     }
     else // disconnect
     {
-        m_btnDynamic->Disconnect(wxID_ANY, wxEVT_BUTTON,
-                                 wxCommandEventHandler(MyFrame::OnDynamic),
-                                 NULL, this);
-        Disconnect(Event_Dynamic, wxEVT_MENU,
-                   wxCommandEventHandler(MyFrame::OnDynamic));
+        m_btnDynamic->Unbind(wxEVT_BUTTON, &MyFrame::OnDynamic, this);
+        Unbind(wxEVT_MENU, &MyFrame::OnDynamic, this, Event_Dynamic);
     }
 
     UpdateDynamicStatus(event.IsChecked());
@@ -561,25 +562,38 @@ void MyFrame::OnPushEventHandler(wxCommandEvent& WXUNUSED(event))
     PushEventHandler(new MyEvtHandler(++m_nPush));
 
 #if wxUSE_STATUSBAR
-    SetStatusText(wxString::Format(wxT("Push count: %u"), m_nPush), Status_Push);
+    SetStatusText(wxString::Format("Push count: %u", m_nPush), Status_Push);
 #endif // wxUSE_STATUSBAR
 }
 
 void MyFrame::OnPopEventHandler(wxCommandEvent& WXUNUSED(event))
 {
-    wxCHECK_RET( m_nPush, wxT("this command should be disabled!") );
+    wxCHECK_RET( m_nPush, "this command should be disabled!" );
 
     PopEventHandler(true /* delete handler */);
     m_nPush--;
 
 #if wxUSE_STATUSBAR
-    SetStatusText(wxString::Format(wxT("Push count: %u"), m_nPush), Status_Push);
+    SetStatusText(wxString::Format("Push count: %u", m_nPush), Status_Push);
 #endif // wxUSE_STATUSBAR
 }
 
+void MyFrame::OnGesture(wxCommandEvent& WXUNUSED(event))
+{
+    if ( m_gestureFrame )
+    {
+        m_gestureFrame->Raise();
+    }
+    else
+    {
+        m_gestureFrame = new MyGestureFrame();
+        m_gestureFrame->Show(true);
+    }
+}
+
 void MyFrame::OnTest(wxCommandEvent& WXUNUSED(event))
 {
-    wxLogMessage(wxT("This is the test event handler in the main frame"));
+    wxLogMessage("This is the test event handler in the main frame");
 }
 
 void MyFrame::OnUpdateUIPop(wxUpdateUIEvent& event)
@@ -600,6 +614,5 @@ void MyFrame::OnFireCustom(wxCommandEvent& WXUNUSED(event))
 
 void MyFrame::OnProcessCustom(wxCommandEvent& WXUNUSED(event))
 {
-    wxLogMessage(wxT("Got a custom event!"));
+    wxLogMessage("Got a custom event!");
 }
-
