@@ -4,7 +4,6 @@
 // Author:      Vadim Zeitlin
 // Modified by:
 // Created:     2003-09-17
-// RCS-ID:      $Id$
 // Copyright:   (c) 2003-2005 Vadim Zeitlin
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -48,12 +47,14 @@
     #include "wx/thread.h"
 #endif
 
+#include "wx/uiaction.h"
+
 // ----------------------------------------------------------------------------
 // resources
 // ----------------------------------------------------------------------------
 
-// the application icon (under Windows and OS/2 it is in resources)
-#if defined(__WXGTK__) || defined(__WXMOTIF__) || defined(__WXMAC__) || defined(__WXMGL__) || defined(__WXX11__)
+// the application icon (under Windows it is in resources)
+#ifndef wxHAS_IMAGES_IN_RESOURCES
     #include "../sample.xpm"
 #endif
 
@@ -75,23 +76,38 @@ static void DoCrash()
 class MyApp : public wxApp
 {
 public:
+    MyApp()
+    {
+        m_numStoredExceptions = 0;
+    }
+
     // override base class virtuals
     // ----------------------------
 
     // program startup
-    virtual bool OnInit();
+    virtual bool OnInit() wxOVERRIDE;
 
     // 2nd-level exception handling: we get all the exceptions occurring in any
     // event handler here
-    virtual bool OnExceptionInMainLoop();
+    virtual bool OnExceptionInMainLoop() wxOVERRIDE;
+
+    // 2nd-level exception handling helpers: if we can't deal with the
+    // exception immediately, we may also store it and rethrow it later, when
+    // we're back from events processing loop.
+    //
+    // Notice that overriding these methods is not necessary when using C++11
+    // as they have a perfectly serviceable implementation inside the library
+    // itself in this case.
+    virtual bool StoreCurrentException() wxOVERRIDE;
+    virtual void RethrowStoredException() wxOVERRIDE;
 
     // 3rd, and final, level exception handling: whenever an unhandled
     // exception is caught, this function is called
-    virtual void OnUnhandledException();
+    virtual void OnUnhandledException() wxOVERRIDE;
 
     // and now for something different: this function is called in case of a
     // crash (e.g. dereferencing null pointer, division by 0, ...)
-    virtual void OnFatalException();
+    virtual void OnFatalException() wxOVERRIDE;
 
     // you can override this function to do something different (e.g. log the
     // assert to file) whenever an assertion fails
@@ -99,7 +115,12 @@ public:
                                  int line,
                                  const wxChar *func,
                                  const wxChar *cond,
-                                 const wxChar *msg);
+                                 const wxChar *msg) wxOVERRIDE;
+
+private:
+    // This stores the number of times StoreCurrentException() was called,
+    // typically at most 1.
+    int m_numStoredExceptions;
 };
 
 // Define a new frame type: this is going to be our main frame
@@ -118,8 +139,10 @@ public:
     void OnThrowString(wxCommandEvent& event);
     void OnThrowObject(wxCommandEvent& event);
     void OnThrowUnhandled(wxCommandEvent& event);
+    void OnThrowFromYield(wxCommandEvent& event);
 
     void OnCrash(wxCommandEvent& event);
+    void OnTrap(wxCommandEvent& event);
 #if wxUSE_ON_FATAL_EXCEPTION
     void OnHandleCrash(wxCommandEvent& event);
 #endif
@@ -128,7 +151,7 @@ protected:
 
     // 1st-level exception handling: we overload ProcessEvent() to be able to
     // catch exceptions which occur in MyFrame methods here
-    virtual bool ProcessEvent(wxEvent& event);
+    virtual bool ProcessEvent(wxEvent& event) wxOVERRIDE;
 
     // provoke assert in main or worker thread
     //
@@ -140,7 +163,7 @@ protected:
 
 private:
     // any class wishing to process wxWidgets events must use this macro
-    DECLARE_EVENT_TABLE()
+    wxDECLARE_EVENT_TABLE();
 };
 
 // A simple dialog which has only some buttons to throw exceptions
@@ -152,10 +175,11 @@ public:
     // event handlers
     void OnThrowInt(wxCommandEvent& event);
     void OnThrowObject(wxCommandEvent& event);
+    void OnThrowUnhandled(wxCommandEvent& event);
     void OnCrash(wxCommandEvent& event);
 
 private:
-    DECLARE_EVENT_TABLE()
+    wxDECLARE_EVENT_TABLE();
 };
 
 // A trivial exception class
@@ -171,6 +195,9 @@ private:
 };
 
 // Another exception class which just has to be different from anything else
+//
+// It is not handled by OnExceptionInMainLoop() but is still handled by
+// explicit try/catch blocks so it's not quite completely unhandled, actually.
 class UnhandledException
 {
 };
@@ -187,7 +214,9 @@ enum
     Except_ThrowString,
     Except_ThrowObject,
     Except_ThrowUnhandled,
+    Except_ThrowFromYield,
     Except_Crash,
+    Except_Trap,
 #if wxUSE_ON_FATAL_EXCEPTION
     Except_HandleCrash,
 #endif // wxUSE_ON_FATAL_EXCEPTION
@@ -208,7 +237,7 @@ enum
 // the event tables connect the wxWidgets events with the functions (event
 // handlers) which process them. It can be also done at run-time, but for the
 // simple menu events like this the static method is much simpler.
-BEGIN_EVENT_TABLE(MyFrame, wxFrame)
+wxBEGIN_EVENT_TABLE(MyFrame, wxFrame)
     EVT_MENU(Except_Quit,  MyFrame::OnQuit)
     EVT_MENU(Except_About, MyFrame::OnAbout)
     EVT_MENU(Except_Dialog, MyFrame::OnDialog)
@@ -216,7 +245,9 @@ BEGIN_EVENT_TABLE(MyFrame, wxFrame)
     EVT_MENU(Except_ThrowString, MyFrame::OnThrowString)
     EVT_MENU(Except_ThrowObject, MyFrame::OnThrowObject)
     EVT_MENU(Except_ThrowUnhandled, MyFrame::OnThrowUnhandled)
+    EVT_MENU(Except_ThrowFromYield, MyFrame::OnThrowFromYield)
     EVT_MENU(Except_Crash, MyFrame::OnCrash)
+    EVT_MENU(Except_Trap, MyFrame::OnTrap)
 #if wxUSE_ON_FATAL_EXCEPTION
     EVT_MENU(Except_HandleCrash, MyFrame::OnHandleCrash)
 #endif // wxUSE_ON_FATAL_EXCEPTION
@@ -224,20 +255,21 @@ BEGIN_EVENT_TABLE(MyFrame, wxFrame)
 #if wxUSE_THREADS
     EVT_MENU(Except_ShowAssertInThread, MyFrame::OnShowAssertInThread)
 #endif // wxUSE_THREADS
-END_EVENT_TABLE()
+wxEND_EVENT_TABLE()
 
-BEGIN_EVENT_TABLE(MyDialog, wxDialog)
+wxBEGIN_EVENT_TABLE(MyDialog, wxDialog)
     EVT_BUTTON(Except_ThrowInt, MyDialog::OnThrowInt)
     EVT_BUTTON(Except_ThrowObject, MyDialog::OnThrowObject)
+    EVT_BUTTON(Except_ThrowUnhandled, MyDialog::OnThrowUnhandled)
     EVT_BUTTON(Except_Crash, MyDialog::OnCrash)
-END_EVENT_TABLE()
+wxEND_EVENT_TABLE()
 
 // Create a new application object: this macro will allow wxWidgets to create
 // the application object during program execution (it's better than using a
 // static object for many reasons) and also implements the accessor function
 // wxGetApp() which will return the reference of the right type (i.e. MyApp and
 // not wxApp)
-IMPLEMENT_APP(MyApp)
+wxIMPLEMENT_APP(MyApp);
 
 // ============================================================================
 // MyApp implementation
@@ -270,11 +302,11 @@ bool MyApp::OnExceptionInMainLoop()
     }
     catch ( int i )
     {
-        wxLogWarning(wxT("Caught an int %d in MyApp."), i);
+        wxLogWarning("Caught an int %d in MyApp.", i);
     }
     catch ( MyException& e )
     {
-        wxLogWarning(wxT("Caught MyException(%s) in MyApp."), e.what());
+        wxLogWarning("Caught MyException(%s) in MyApp.", e.what());
     }
     catch ( ... )
     {
@@ -284,6 +316,41 @@ bool MyApp::OnExceptionInMainLoop()
     return true;
 }
 
+bool MyApp::StoreCurrentException()
+{
+    try
+    {
+        throw;
+    }
+    catch ( UnhandledException& )
+    {
+        if ( m_numStoredExceptions )
+        {
+            wxLogWarning("Unexpectedly many exceptions to store.");
+        }
+
+        m_numStoredExceptions++;
+
+        return true;
+    }
+    catch ( ... )
+    {
+        // Don't know how to store other exceptions.
+    }
+
+    return false;
+}
+
+void MyApp::RethrowStoredException()
+{
+    if ( m_numStoredExceptions )
+    {
+        m_numStoredExceptions = 0;
+
+        throw UnhandledException();
+    }
+}
+
 void MyApp::OnUnhandledException()
 {
     // this shows how we may let some exception propagate uncaught
@@ -297,15 +364,15 @@ void MyApp::OnUnhandledException()
     }
     catch ( ... )
     {
-        wxMessageBox(wxT("Unhandled exception caught, program will terminate."),
-                     wxT("wxExcept Sample"), wxOK | wxICON_ERROR);
+        wxMessageBox("Unhandled exception caught, program will terminate.",
+                     "wxExcept Sample", wxOK | wxICON_ERROR);
     }
 }
 
 void MyApp::OnFatalException()
 {
-    wxMessageBox(wxT("Program has crashed and will terminate."),
-                 wxT("wxExcept Sample"), wxOK | wxICON_ERROR);
+    wxMessageBox("Program has crashed and will terminate.",
+                 "wxExcept Sample", wxOK | wxICON_ERROR);
 }
 
 void MyApp::OnAssertFailure(const wxChar *file,
@@ -336,7 +403,7 @@ void MyApp::OnAssertFailure(const wxChar *file,
 
 // frame constructor
 MyFrame::MyFrame()
-       : wxFrame(NULL, wxID_ANY, wxT("Except wxWidgets App"),
+       : wxFrame(NULL, wxID_ANY, "Except wxWidgets App",
                  wxPoint(50, 50), wxSize(450, 340))
 {
     // set the frame icon
@@ -345,43 +412,47 @@ MyFrame::MyFrame()
 #if wxUSE_MENUS
     // create a menu bar
     wxMenu *menuFile = new wxMenu;
-    menuFile->Append(Except_Dialog, wxT("Show &dialog\tCtrl-D"));
+    menuFile->Append(Except_Dialog, "Show &dialog\tCtrl-D");
     menuFile->AppendSeparator();
-    menuFile->Append(Except_ThrowInt, wxT("Throw an &int\tCtrl-I"));
-    menuFile->Append(Except_ThrowString, wxT("Throw a &string\tCtrl-S"));
-    menuFile->Append(Except_ThrowObject, wxT("Throw an &object\tCtrl-O"));
+    menuFile->Append(Except_ThrowInt, "Throw an &int\tCtrl-I");
+    menuFile->Append(Except_ThrowString, "Throw a &string\tCtrl-S");
+    menuFile->Append(Except_ThrowObject, "Throw an &object\tCtrl-O");
     menuFile->Append(Except_ThrowUnhandled,
-                        wxT("Throw &unhandled exception\tCtrl-U"));
-    menuFile->Append(Except_Crash, wxT("&Crash\tCtrl-C"));
+                        "Throw &unhandled exception\tCtrl-U");
+    menuFile->Append(Except_ThrowFromYield,
+                        "Throw from wx&Yield()\tCtrl-Y");
+    menuFile->Append(Except_Crash, "&Crash\tCtrl-C");
+    menuFile->Append(Except_Trap, "&Trap\tCtrl-T",
+                     "Break into the debugger (if one is running)");
     menuFile->AppendSeparator();
 #if wxUSE_ON_FATAL_EXCEPTION
-    menuFile->AppendCheckItem(Except_HandleCrash, wxT("&Handle crashes\tCtrl-H"));
+    menuFile->AppendCheckItem(Except_HandleCrash, "&Handle crashes\tCtrl-H");
     menuFile->AppendSeparator();
 #endif // wxUSE_ON_FATAL_EXCEPTION
-    menuFile->Append(Except_ShowAssert, wxT("Provoke &assert failure\tCtrl-A"));
+    menuFile->Append(Except_ShowAssert, "Provoke &assert failure\tCtrl-A");
 #if wxUSE_THREADS
     menuFile->Append(Except_ShowAssertInThread,
-                     wxT("Assert failure in &thread\tShift-Ctrl-A"));
+                     "Assert failure in &thread\tShift-Ctrl-A");
 #endif // wxUSE_THREADS
     menuFile->AppendSeparator();
-    menuFile->Append(Except_Quit, wxT("E&xit\tCtrl-Q"), wxT("Quit this program"));
+    menuFile->Append(Except_Quit, "E&xit\tCtrl-Q", "Quit this program");
 
     wxMenu *helpMenu = new wxMenu;
-    helpMenu->Append(Except_About, wxT("&About...\tF1"), wxT("Show about dialog"));
+    helpMenu->Append(Except_About, "&About\tF1", "Show about dialog");
 
     // now append the freshly created menu to the menu bar...
     wxMenuBar *menuBar = new wxMenuBar();
-    menuBar->Append(menuFile, wxT("&File"));
-    menuBar->Append(helpMenu, wxT("&Help"));
+    menuBar->Append(menuFile, "&File");
+    menuBar->Append(helpMenu, "&Help");
 
     // ... and attach this menu bar to the frame
     SetMenuBar(menuBar);
 #endif // wxUSE_MENUS
 
-#if wxUSE_STATUSBAR && !defined(__WXWINCE__)
+#if wxUSE_STATUSBAR
     // create a status bar just for fun (by default with 1 pane only)
     CreateStatusBar(2);
-    SetStatusText(wxT("Welcome to wxWidgets!"));
+    SetStatusText("Welcome to wxWidgets!");
 #endif // wxUSE_STATUSBAR
 }
 
@@ -393,7 +464,7 @@ bool MyFrame::ProcessEvent(wxEvent& event)
     }
     catch ( const wxChar *msg )
     {
-        wxLogMessage(wxT("Caught a string \"%s\" in MyFrame"), msg);
+        wxLogMessage("Caught a string \"%s\" in MyFrame", msg);
 
         return true;
     }
@@ -413,9 +484,13 @@ void MyFrame::OnDialog(wxCommandEvent& WXUNUSED(event))
 
         dlg.ShowModal();
     }
+    catch ( UnhandledException& )
+    {
+        wxLogMessage("Caught unhandled exception inside the dialog.");
+    }
     catch ( ... )
     {
-        wxLogWarning(wxT("An exception in MyDialog"));
+        wxLogWarning("An exception in MyDialog");
 
         Destroy();
         throw;
@@ -429,12 +504,12 @@ void MyFrame::OnThrowInt(wxCommandEvent& WXUNUSED(event))
 
 void MyFrame::OnThrowString(wxCommandEvent& WXUNUSED(event))
 {
-    throw wxT("string thrown from MyFrame");
+    throw "string thrown from MyFrame";
 }
 
 void MyFrame::OnThrowObject(wxCommandEvent& WXUNUSED(event))
 {
-    throw MyException(wxT("Exception thrown from MyFrame"));
+    throw MyException("Exception thrown from MyFrame");
 }
 
 void MyFrame::OnThrowUnhandled(wxCommandEvent& WXUNUSED(event))
@@ -442,11 +517,43 @@ void MyFrame::OnThrowUnhandled(wxCommandEvent& WXUNUSED(event))
     throw UnhandledException();
 }
 
+void MyFrame::OnThrowFromYield(wxCommandEvent& WXUNUSED(event))
+{
+#if wxUSE_UIACTIONSIMULATOR
+    // Simulate selecting the "Throw unhandled" menu item, its handler will be
+    // executed from inside wxYield() and as the exception is not handled by
+    // our OnExceptionInMainLoop(), will call StoreCurrentException() and, when
+    // wxYield() regains control, RethrowStoredException().
+    //
+    // Notice that if we didn't override these methods we wouldn't be able to
+    // catch this exception here!
+    try
+    {
+        wxUIActionSimulator sim;
+        sim.Char('U', wxMOD_CONTROL);
+        wxYield();
+    }
+    catch ( UnhandledException& )
+    {
+        wxLogMessage("Caught unhandled exception inside wxYield().");
+    }
+#else // !wxUSE_UIACTIONSIMULATOR
+    wxLogError("Can't trigger an exception inside wxYield() "
+               "without wxUIActionSimulator, please rebuild "
+               "with wxUSE_UIACTIONSIMULATOR=1.");
+#endif // wxUSE_UIACTIONSIMULATOR/!wxUSE_UIACTIONSIMULATOR
+}
+
 void MyFrame::OnCrash(wxCommandEvent& WXUNUSED(event))
 {
     DoCrash();
 }
 
+void MyFrame::OnTrap(wxCommandEvent& WXUNUSED(event))
+{
+    wxTrap();
+}
+
 #if wxUSE_ON_FATAL_EXCEPTION
 
 void MyFrame::OnHandleCrash(wxCommandEvent& event)
@@ -476,7 +583,7 @@ void MyFrame::OnShowAssertInThread(wxCommandEvent& WXUNUSED(event))
         }
 
     protected:
-        virtual void *Entry()
+        virtual void *Entry() wxOVERRIDE
         {
             wxFAIL_MSG("Test assert in another thread.");
 
@@ -495,10 +602,10 @@ void MyFrame::OnShowAssertInThread(wxCommandEvent& WXUNUSED(event))
 void MyFrame::OnAbout(wxCommandEvent& WXUNUSED(event))
 {
     wxString msg;
-    msg.Printf( wxT("This is the About dialog of the except sample.\n")
-                wxT("Welcome to %s"), wxVERSION_STRING);
+    msg.Printf( "This is the About dialog of the except sample.\n"
+                "Welcome to %s", wxVERSION_STRING);
 
-    wxMessageBox(msg, wxT("About Except"), wxOK | wxICON_INFORMATION, this);
+    wxMessageBox(msg, "About Except", wxOK | wxICON_INFORMATION, this);
 }
 
 // ============================================================================
@@ -506,18 +613,20 @@ void MyFrame::OnAbout(wxCommandEvent& WXUNUSED(event))
 // ============================================================================
 
 MyDialog::MyDialog(wxFrame *parent)
-        : wxDialog(parent, wxID_ANY, wxString(wxT("Throw exception dialog")))
+        : wxDialog(parent, wxID_ANY, wxString("Throw exception dialog"))
 {
     wxSizer *sizerTop = new wxBoxSizer(wxVERTICAL);
 
-    sizerTop->Add(new wxButton(this, Except_ThrowInt, wxT("Throw &int")),
-                  0, wxCENTRE | wxALL, 5);
-    sizerTop->Add(new wxButton(this, Except_ThrowObject, wxT("Throw &object")),
-                  0, wxCENTRE | wxALL, 5);
-    sizerTop->Add(new wxButton(this, Except_Crash, wxT("&Crash")),
-                  0, wxCENTRE | wxALL, 5);
-    sizerTop->Add(new wxButton(this, wxID_CANCEL, wxT("&Cancel")),
-                  0, wxCENTRE | wxALL, 5);
+    sizerTop->Add(new wxButton(this, Except_ThrowInt, "Throw &int"),
+                  0, wxEXPAND | wxALL, 5);
+    sizerTop->Add(new wxButton(this, Except_ThrowObject, "Throw &object"),
+                  0, wxEXPAND | wxALL, 5);
+    sizerTop->Add(new wxButton(this, Except_ThrowUnhandled, "Throw &unhandled"),
+                  0, wxEXPAND | wxALL, 5);
+    sizerTop->Add(new wxButton(this, Except_Crash, "&Crash"),
+                  0, wxEXPAND | wxALL, 5);
+    sizerTop->Add(new wxButton(this, wxID_CANCEL, "&Cancel"),
+                  0, wxEXPAND | wxALL, 5);
 
     SetSizerAndFit(sizerTop);
 }
@@ -529,7 +638,12 @@ void MyDialog::OnThrowInt(wxCommandEvent& WXUNUSED(event))
 
 void MyDialog::OnThrowObject(wxCommandEvent& WXUNUSED(event))
 {
-    throw MyException(wxT("Exception thrown from MyDialog"));
+    throw MyException("Exception thrown from MyDialog");
+}
+
+void MyDialog::OnThrowUnhandled(wxCommandEvent& WXUNUSED(event))
+{
+    throw UnhandledException();
 }
 
 void MyDialog::OnCrash(wxCommandEvent& WXUNUSED(event))
