@@ -311,7 +311,7 @@ bool MyApp::OnInit()
 #endif // wxUSE_HTML
 
     // Create the main application window
-    MyFrame *frame = new MyFrame(wxT("HelpDemo wxWidgets App"),
+    MyFrame *frame = new MyFrame("HelpDemo wxWidgets App",
                                  wxPoint(50, 50), wxSize(450, 340));
 
 #if !USE_SIMPLE_HELP_PROVIDER
@@ -327,34 +327,34 @@ bool MyApp::OnInit()
     // initialise the help system: this means that we'll use doc.hlp file under
     // Windows and that the HTML docs are in the subdirectory doc for platforms
     // using HTML help
-    if ( !frame->GetHelpController().Initialize(wxT("doc")) )
+    if ( !frame->GetHelpController().Initialize("doc") )
     {
-        wxLogError(wxT("Cannot initialize the help system, aborting."));
+        wxLogError("Cannot initialize the help system, aborting.");
 
         return false;
     }
 
 #if wxUSE_MS_HTML_HELP && !defined(__WXUNIVERSAL__)
-    if( !frame->GetMSHtmlHelpController().Initialize(wxT("doc")) )
+    if( !frame->GetMSHtmlHelpController().Initialize("doc") )
     {
-        wxLogError(wxT("Cannot initialize the MS HTML Help system."));
+        wxLogError("Cannot initialize the MS HTML Help system.");
     }
 #endif
 
 #if wxUSE_MS_HTML_HELP && wxUSE_WXHTML_HELP && !defined(__WXUNIVERSAL__)
     // you need to call Initialize in order to use wxBestHelpController
-    if( !frame->GetBestHelpController().Initialize(wxT("doc")) )
+    if( !frame->GetBestHelpController().Initialize("doc") )
     {
-        wxLogError(wxT("Cannot initialize the best help system, aborting."));
+        wxLogError("Cannot initialize the best help system, aborting.");
     }
 #endif
 
 #if USE_HTML_HELP
     // initialise the advanced HTML help system: this means that the HTML docs are in .htb
     // (zipped) form
-    if ( !frame->GetAdvancedHtmlHelpController().Initialize(wxT("doc")) )
+    if ( !frame->GetAdvancedHtmlHelpController().Initialize("doc") )
     {
-        wxLogError(wxT("Cannot initialize the advanced HTML help system, aborting."));
+        wxLogError("Cannot initialize the advanced HTML help system, aborting.");
 
         return false;
     }
@@ -363,7 +363,7 @@ bool MyApp::OnInit()
 #if 0
     // defined(__WXMSW__) && wxUSE_MS_HTML_HELP
     wxString path(wxGetCwd());
-    if ( !frame->GetMSHtmlHelpController().Initialize(path + wxT("\\doc.chm")) )
+    if ( !frame->GetMSHtmlHelpController().Initialize(path + "\\doc.chm") )
     {
         wxLogError("Cannot initialize the MS HTML help system, aborting.");
 
@@ -399,51 +399,51 @@ MyFrame::MyFrame(const wxString& title, const wxPoint& pos, const wxSize& size)
     // create a menu bar
     wxMenu *menuFile = new wxMenu;
 
-    menuFile->Append(HelpDemo_Help_Index, wxT("&Help Index..."));
-    menuFile->Append(HelpDemo_Help_Classes, wxT("&Help on Classes..."));
-    menuFile->Append(HelpDemo_Help_Functions, wxT("&Help on Functions..."));
-    menuFile->Append(HelpDemo_Help_ContextHelp, wxT("&Context Help..."));
-    menuFile->Append(HelpDemo_Help_DialogContextHelp, wxT("&Dialog Context Help...\tCtrl-H"));
-    menuFile->Append(HelpDemo_Help_Help, wxT("&About Help Demo..."));
-    menuFile->Append(HelpDemo_Help_Search, wxT("&Search help..."));
+    menuFile->Append(HelpDemo_Help_Index, "&Help Index...");
+    menuFile->Append(HelpDemo_Help_Classes, "&Help on Classes...");
+    menuFile->Append(HelpDemo_Help_Functions, "&Help on Functions...");
+    menuFile->Append(HelpDemo_Help_ContextHelp, "&Context Help...");
+    menuFile->Append(HelpDemo_Help_DialogContextHelp, "&Dialog Context Help...\tCtrl-H");
+    menuFile->Append(HelpDemo_Help_Help, "&About Help Demo...");
+    menuFile->Append(HelpDemo_Help_Search, "&Search help...");
 #if USE_HTML_HELP
     menuFile->AppendSeparator();
-    menuFile->Append(HelpDemo_Advanced_Html_Help_Index, wxT("Advanced HTML &Help Index..."));
-    menuFile->Append(HelpDemo_Advanced_Html_Help_Classes, wxT("Advanced HTML &Help on Classes..."));
-    menuFile->Append(HelpDemo_Advanced_Html_Help_Functions, wxT("Advanced HTML &Help on Functions..."));
-    menuFile->Append(HelpDemo_Advanced_Html_Help_Help, wxT("Advanced HTML &About Help Demo..."));
-    menuFile->Append(HelpDemo_Advanced_Html_Help_Search, wxT("Advanced HTML &Search help..."));
-    menuFile->Append(HelpDemo_Advanced_Html_Help_Modal, wxT("Advanced HTML Help &Modal Dialog..."));
+    menuFile->Append(HelpDemo_Advanced_Html_Help_Index, "Advanced HTML &Help Index...");
+    menuFile->Append(HelpDemo_Advanced_Html_Help_Classes, "Advanced HTML &Help on Classes...");
+    menuFile->Append(HelpDemo_Advanced_Html_Help_Functions, "Advanced HTML &Help on Functions...");
+    menuFile->Append(HelpDemo_Advanced_Html_Help_Help, "Advanced HTML &About Help Demo...");
+    menuFile->Append(HelpDemo_Advanced_Html_Help_Search, "Advanced HTML &Search help...");
+    menuFile->Append(HelpDemo_Advanced_Html_Help_Modal, "Advanced HTML Help &Modal Dialog...");
 #endif
 
 #if wxUSE_MS_HTML_HELP && !defined(__WXUNIVERSAL__)
     menuFile->AppendSeparator();
-    menuFile->Append(HelpDemo_MS_Html_Help_Index, wxT("MS HTML &Help Index..."));
-    menuFile->Append(HelpDemo_MS_Html_Help_Classes, wxT("MS HTML &Help on Classes..."));
-    menuFile->Append(HelpDemo_MS_Html_Help_Functions, wxT("MS HTML &Help on Functions..."));
-    menuFile->Append(HelpDemo_MS_Html_Help_Help, wxT("MS HTML &About Help Demo..."));
-    menuFile->Append(HelpDemo_MS_Html_Help_Search, wxT("MS HTML &Search help..."));
+    menuFile->Append(HelpDemo_MS_Html_Help_Index, "MS HTML &Help Index...");
+    menuFile->Append(HelpDemo_MS_Html_Help_Classes, "MS HTML &Help on Classes...");
+    menuFile->Append(HelpDemo_MS_Html_Help_Functions, "MS HTML &Help on Functions...");
+    menuFile->Append(HelpDemo_MS_Html_Help_Help, "MS HTML &About Help Demo...");
+    menuFile->Append(HelpDemo_MS_Html_Help_Search, "MS HTML &Search help...");
 #endif
 
 #if wxUSE_MS_HTML_HELP && wxUSE_WXHTML_HELP && !defined(__WXUNIVERSAL__)
     menuFile->AppendSeparator();
-    menuFile->Append(HelpDemo_Best_Help_Index, wxT("Best &Help Index..."));
+    menuFile->Append(HelpDemo_Best_Help_Index, "Best &Help Index...");
 #endif
 
 #ifndef __WXMSW__
 #if !wxUSE_HTML
     menuFile->AppendSeparator();
-    menuFile->Append(HelpDemo_Help_KDE, wxT("Use &KDE"));
-    menuFile->Append(HelpDemo_Help_GNOME, wxT("Use &GNOME"));
-    menuFile->Append(HelpDemo_Help_Netscape, wxT("Use &Netscape"));
+    menuFile->Append(HelpDemo_Help_KDE, "Use &KDE");
+    menuFile->Append(HelpDemo_Help_GNOME, "Use &GNOME");
+    menuFile->Append(HelpDemo_Help_Netscape, "Use &Netscape");
 #endif
 #endif
     menuFile->AppendSeparator();
-    menuFile->Append(HelpDemo_Quit, wxT("E&xit"));
+    menuFile->Append(HelpDemo_Quit, "E&xit");
 
     // now append the freshly created menu to the menu bar...
     wxMenuBar *menuBar = new wxMenuBar;
-    menuBar->Append(menuFile, wxT("&File"));
+    menuBar->Append(menuFile, "&File");
 
     // ... and attach this menu bar to the frame
     SetMenuBar(menuBar);
@@ -451,7 +451,7 @@ MyFrame::MyFrame(const wxString& title, const wxPoint& pos, const wxSize& size)
 #if wxUSE_STATUSBAR
     // create a status bar just for fun (by default with 1 pane only)
     CreateStatusBar();
-    SetStatusText(wxT("Welcome to wxWidgets!"));
+    SetStatusText("Welcome to wxWidgets!");
 #endif // wxUSE_STATUSBAR
 
 #if USE_HTML_HELP
@@ -463,8 +463,8 @@ MyFrame::MyFrame(const wxString& title, const wxPoint& pos, const wxSize& size)
     m_embeddedHelpWindow->Create(this,
         wxID_ANY, wxDefaultPosition, GetClientSize(), wxTAB_TRAVERSAL|wxNO_BORDER, wxHF_DEFAULT_STYLE);
 
-    m_embeddedHtmlHelp.AddBook(wxFileName(wxT("doc.zip")));
-    m_embeddedHtmlHelp.Display(wxT("Introduction"));
+    m_embeddedHtmlHelp.AddBook(wxFileName("doc.zip"));
+    m_embeddedHtmlHelp.Display("Introduction");
 #else
     // now create some controls
 
@@ -475,7 +475,7 @@ MyFrame::MyFrame(const wxString& title, const wxPoint& pos, const wxSize& size)
     //panel->SetHelpText(wxContextId(300));
 
     // and a static control whose parent is the panel
-    wxStaticText* staticText = new wxStaticText(panel, 302, wxT("Hello, world!"), wxPoint(10, 10));
+    wxStaticText* staticText = new wxStaticText(panel, 302, "Hello, world!", wxPoint(10, 10));
     staticText->SetHelpText(_("This static text control isn't doing a lot right now."));
 #endif
 }
@@ -531,7 +531,7 @@ void MyFrame::OnBestHelp(wxCommandEvent& event)
 #if USE_HTML_HELP
 void MyFrame::OnModalHtmlHelp(wxCommandEvent& WXUNUSED(event))
 {
-    wxHtmlModalHelp(this, wxT("doc.zip"), wxT("Introduction"));
+    wxHtmlModalHelp(this, "doc.zip", "Introduction");
 }
 #endif
 
@@ -638,8 +638,8 @@ void MyFrame::ShowHelp(int commandId, wxHelpControllerBase& helpController)
        case HelpDemo_MS_Html_Help_Search:
        case HelpDemo_Best_Help_Search:
        {
-          wxString key = wxGetTextFromUser(wxT("Search for?"),
-                                           wxT("Search help for keyword"),
+          wxString key = wxGetTextFromUser("Search for?",
+                                           "Search help for keyword",
                                            wxEmptyString,
                                            this);
           if(! key.IsEmpty())
@@ -658,13 +658,13 @@ void MyFrame::ShowHelp(int commandId, wxHelpControllerBase& helpController)
        // These three calls are only used by wxExtHelpController
 
        case HelpDemo_Help_KDE:
-          helpController.SetViewer(wxT("kdehelp"));
+          helpController.SetViewer("kdehelp");
           break;
        case HelpDemo_Help_GNOME:
-          helpController.SetViewer(wxT("gnome-help-browser"));
+          helpController.SetViewer("gnome-help-browser");
           break;
        case HelpDemo_Help_Netscape:
-          helpController.SetViewer(wxT("netscape"), wxHELP_NETSCAPE);
+          helpController.SetViewer("netscape", wxHELP_NETSCAPE);
           break;
    }
 }
@@ -678,7 +678,7 @@ wxBEGIN_EVENT_TABLE(MyModalDialog, wxDialog)
 wxEND_EVENT_TABLE()
 
 MyModalDialog::MyModalDialog(wxWindow *parent)
-             : wxDialog(parent, wxID_ANY, wxString(wxT("Modal dialog")))
+             : wxDialog(parent, wxID_ANY, wxString("Modal dialog"))
 {
     // Add the context-sensitive help button on the caption for the platforms
     // which support it (currently MSW only)
@@ -688,10 +688,10 @@ MyModalDialog::MyModalDialog(wxWindow *parent)
     wxBoxSizer *sizerTop = new wxBoxSizer(wxVERTICAL);
     wxBoxSizer *sizerRow = new wxBoxSizer(wxHORIZONTAL);
 
-    wxButton* btnOK = new wxButton(this, wxID_OK, wxT("&OK"));
+    wxButton* btnOK = new wxButton(this, wxID_OK, "&OK");
     btnOK->SetHelpText(_("The OK button confirms the dialog choices."));
 
-    wxButton* btnCancel = new wxButton(this, wxID_CANCEL, wxT("&Cancel"));
+    wxButton* btnCancel = new wxButton(this, wxID_CANCEL, "&Cancel");
     btnCancel->SetHelpText(_("The Cancel button cancels the dialog."));
 
     sizerRow->Add(btnOK, 0, wxALIGN_CENTER | wxALL, 5);
@@ -702,7 +702,7 @@ MyModalDialog::MyModalDialog(wxWindow *parent)
     sizerRow->Add(new wxContextHelpButton(this), 0, wxALIGN_CENTER | wxALL, 5);
 #endif
 
-    wxTextCtrl *text = new wxTextCtrl(this, wxID_ANY, wxT("A demo text control"),
+    wxTextCtrl *text = new wxTextCtrl(this, wxID_ANY, "A demo text control",
                                       wxDefaultPosition, wxSize(300, 100),
                                       wxTE_MULTILINE);
     text->SetHelpText(_("Type text here if you have got nothing more interesting to do"));
