@@ -91,7 +91,6 @@ class StaticWidgetsPage : public WidgetsPage
 {
 public:
     StaticWidgetsPage(WidgetsBookCtrl *book, wxImageList *imaglist);
-    virtual ~StaticWidgetsPage(){};
 
     virtual wxWindow *GetWidget() const wxOVERRIDE { return m_statText; }
     virtual Widgets GetWidgets() const wxOVERRIDE
@@ -201,7 +200,7 @@ wxEND_EVENT_TABLE()
 // implementation
 // ============================================================================
 
-IMPLEMENT_WIDGETS_PAGE(StaticWidgetsPage, wxT("Static"),
+IMPLEMENT_WIDGETS_PAGE(StaticWidgetsPage, "Static",
                        (int)wxPlatform(GENERIC_CTRLS).If(wxOS_WINDOWS,NATIVE_CTRLS)
                        );
 
@@ -261,22 +260,22 @@ void StaticWidgetsPage::CreateContent()
 
     static const wxString halign[] =
     {
-        wxT("left"),
-        wxT("centre"),
-        wxT("right"),
+        "left",
+        "centre",
+        "right",
     };
 
     static const wxString valign[] =
     {
-        wxT("top"),
-        wxT("centre"),
-        wxT("bottom"),
+        "top",
+        "centre",
+        "bottom",
     };
 
-    m_radioHAlign = new wxRadioBox(this, wxID_ANY, wxT("&Horz alignment"),
+    m_radioHAlign = new wxRadioBox(this, wxID_ANY, "&Horz alignment",
                                    wxDefaultPosition, wxDefaultSize,
                                    WXSIZEOF(halign), halign, 3);
-    m_radioVAlign = new wxRadioBox(this, wxID_ANY, wxT("&Vert alignment"),
+    m_radioVAlign = new wxRadioBox(this, wxID_ANY, "&Vert alignment",
                                    wxDefaultPosition, wxDefaultSize,
                                    WXSIZEOF(valign), valign, 3);
 
@@ -286,23 +285,23 @@ void StaticWidgetsPage::CreateContent()
 
     sizerLeft->Add(5, 5, 0, wxGROW | wxALL, 5); // spacer
 
-    m_chkEllipsize = CreateCheckBoxAndAddToSizer(sizerLeft, wxT("&Ellipsize"));
+    m_chkEllipsize = CreateCheckBoxAndAddToSizer(sizerLeft, "&Ellipsize");
 
     static const wxString ellipsizeMode[] =
     {
-        wxT("&start"),
-        wxT("&middle"),
-        wxT("&end"),
+        "&start",
+        "&middle",
+        "&end",
     };
 
-    m_radioEllipsize = new wxRadioBox(this, wxID_ANY, wxT("&Ellipsize mode"),
+    m_radioEllipsize = new wxRadioBox(this, wxID_ANY, "&Ellipsize mode",
                                       wxDefaultPosition, wxDefaultSize,
                                       WXSIZEOF(ellipsizeMode), ellipsizeMode,
                                       3);
 
     sizerLeft->Add(m_radioEllipsize, 0, wxGROW | wxALL, 5);
 
-    wxButton *btn = new wxButton(this, StaticPage_Reset, wxT("&Reset"));
+    wxButton *btn = new wxButton(this, StaticPage_Reset, "&Reset");
     sizerLeft->Add(btn, 0, wxALIGN_CENTRE_HORIZONTAL | wxALL, 15);
 
     // middle pane
@@ -311,9 +310,7 @@ void StaticWidgetsPage::CreateContent()
 
     m_textBox = new wxTextCtrl(this, wxID_ANY, wxEmptyString);
     wxButton *b1 = new wxButton(this, wxID_ANY, "Change &box label");
-    b1->Connect(wxEVT_BUTTON,
-                wxCommandEventHandler(StaticWidgetsPage::OnButtonBoxText),
-                NULL, this);
+    b1->Bind(wxEVT_BUTTON, &StaticWidgetsPage::OnButtonBoxText, this);
     sizerMiddle->Add(m_textBox, 0, wxEXPAND|wxALL, 5);
     sizerMiddle->Add(b1, 0, wxLEFT|wxBOTTOM, 5);
 
@@ -321,9 +318,7 @@ void StaticWidgetsPage::CreateContent()
                                  wxDefaultPosition, wxDefaultSize,
                                  wxTE_MULTILINE|wxHSCROLL);
     wxButton *b2 = new wxButton(this, wxID_ANY, "Change &text label");
-    b2->Connect(wxEVT_BUTTON,
-                wxCommandEventHandler(StaticWidgetsPage::OnButtonLabelText),
-                NULL, this);
+    b2->Bind(wxEVT_BUTTON, &StaticWidgetsPage::OnButtonLabelText, this);
     sizerMiddle->Add(m_textLabel, 0, wxEXPAND|wxALL, 5);
     sizerMiddle->Add(b2, 0, wxLEFT|wxBOTTOM, 5);
 
@@ -333,9 +328,7 @@ void StaticWidgetsPage::CreateContent()
                                            wxTE_MULTILINE|wxHSCROLL);
 
     wxButton *b3 = new wxButton(this, wxID_ANY, "Change decorated text label");
-    b3->Connect(wxEVT_BUTTON,
-                wxCommandEventHandler(StaticWidgetsPage::OnButtonLabelWithMarkupText),
-                NULL, this);
+    b3->Bind(wxEVT_BUTTON, &StaticWidgetsPage::OnButtonLabelWithMarkupText, this);
     sizerMiddle->Add(m_textLabelWithMarkup, 0, wxEXPAND|wxALL, 5);
     sizerMiddle->Add(b3, 0, wxLEFT|wxBOTTOM, 5);
 
@@ -347,14 +340,14 @@ void StaticWidgetsPage::CreateContent()
     // NB: must be done _before_ calling CreateStatic()
     Reset();
 
-    m_textBox->SetValue(wxT("This is a &box"));
-    m_textLabel->SetValue(wxT("And this is a\n\tlabel inside the box with a &mnemonic.\n")
-                          wxT("Only this text is affected by the ellipsize settings."));
+    m_textBox->SetValue("This is a &box");
+    m_textLabel->SetValue("And this is a\n\tlabel inside the box with a &mnemonic.\n"
+                          "Only this text is affected by the ellipsize settings.");
 #if wxUSE_MARKUP
-    m_textLabelWithMarkup->SetValue(wxT("Another label, this time <b>decorated</b> ")
-                                    wxT("with <u>markup</u>; here you need entities ")
-                                    wxT("for the symbols: &lt; &gt; &amp; &apos; &quot; ")
-                                    wxT(" but you can still place &mnemonics..."));
+    m_textLabelWithMarkup->SetValue("Another label, this time <b>decorated</b> "
+                                    "with <u>markup</u>; here you need entities "
+                                    "for the symbols: &lt; &gt; &amp; &apos; &quot; "
+                                    " but you can still place &mnemonics...");
 #endif // wxUSE_MARKUP
 
     // right pane
@@ -423,8 +416,8 @@ void StaticWidgetsPage::CreateStatic()
     switch ( m_radioHAlign->GetSelection() )
     {
         default:
-            wxFAIL_MSG(wxT("unexpected radiobox selection"));
-            // fall through
+            wxFAIL_MSG("unexpected radiobox selection");
+            wxFALLTHROUGH;
 
         case StaticHAlign_Left:
             align |= wxALIGN_LEFT;
@@ -442,8 +435,8 @@ void StaticWidgetsPage::CreateStatic()
     switch ( m_radioVAlign->GetSelection() )
     {
         default:
-            wxFAIL_MSG(wxT("unexpected radiobox selection"));
-            // fall through
+            wxFAIL_MSG("unexpected radiobox selection");
+            wxFALLTHROUGH;
 
         case StaticVAlign_Top:
             align |= wxALIGN_TOP;
@@ -463,8 +456,8 @@ void StaticWidgetsPage::CreateStatic()
         switch ( m_radioEllipsize->GetSelection() )
         {
             default:
-                wxFAIL_MSG(wxT("unexpected radiobox selection"));
-                // fall through
+                wxFAIL_MSG("unexpected radiobox selection");
+                wxFALLTHROUGH;
 
             case StaticEllipsize_Start:
                 flagsDummyText |= wxST_ELLIPSIZE_START;
@@ -563,12 +556,8 @@ void StaticWidgetsPage::CreateStatic()
 
     m_sizerStatic->Layout();
 
-    m_statText->Connect(wxEVT_LEFT_UP,
-                        wxMouseEventHandler(StaticWidgetsPage::OnMouseEvent),
-                        NULL, this);
-    staticBox->Connect(wxEVT_LEFT_UP,
-                       wxMouseEventHandler(StaticWidgetsPage::OnMouseEvent),
-                       NULL, this);
+    m_statText->Bind(wxEVT_LEFT_UP, &StaticWidgetsPage::OnMouseEvent, this);
+    staticBox->Bind(wxEVT_LEFT_UP, &StaticWidgetsPage::OnMouseEvent, this);
 
     SetUpWidget();
 }
@@ -614,9 +603,9 @@ void StaticWidgetsPage::OnButtonLabelText(wxCommandEvent& WXUNUSED(event))
     // test GetLabel() and GetLabelText(); the first should return the
     // label as it is written in the relative text control; the second should
     // return the label as it's shown in the wxStaticText
-    wxLogMessage(wxT("The original label should be '%s'"),
+    wxLogMessage("The original label should be '%s'",
                  m_statText->GetLabel());
-    wxLogMessage(wxT("The label text is '%s'"),
+    wxLogMessage("The label text is '%s'",
                  m_statText->GetLabelText());
 }
 
@@ -628,9 +617,9 @@ void StaticWidgetsPage::OnButtonLabelWithMarkupText(wxCommandEvent& WXUNUSED(eve
     // test GetLabel() and GetLabelText(); the first should return the
     // label as it is written in the relative text control; the second should
     // return the label as it's shown in the wxStaticText
-    wxLogMessage(wxT("The original label should be '%s'"),
+    wxLogMessage("The original label should be '%s'",
                  m_statMarkup->GetLabel());
-    wxLogMessage(wxT("The label text is '%s'"),
+    wxLogMessage("The label text is '%s'",
                  m_statMarkup->GetLabelText());
 }
 #endif // wxUSE_MARKUP
