@@ -1094,15 +1094,10 @@ wxComboCtrlBase::CreateTextCtrl(int style)
                        style);
 
         // Connecting the events is currently the most reliable way
-        wxWindowID id = m_text->GetId();
-        m_text->Connect(id, wxEVT_TEXT,
-                        wxCommandEventHandler(wxComboCtrlBase::OnTextCtrlEvent),
-                        NULL, this);
+        m_text->Bind(wxEVT_TEXT, &wxComboCtrlBase::OnTextCtrlEvent, this);
         if ( style & wxTE_PROCESS_ENTER )
         {
-            m_text->Connect(id, wxEVT_TEXT_ENTER,
-                            wxCommandEventHandler(wxComboCtrlBase::OnTextCtrlEvent),
-                            NULL, this);
+            m_text->Bind(wxEVT_TEXT_ENTER, &wxComboCtrlBase::OnTextCtrlEvent, this);
         }
 
         m_text->SetHint(m_hintText);
@@ -2078,7 +2073,7 @@ void wxComboCtrlBase::OnFocusEvent( wxFocusEvent& event )
 {
     // On Mac, setting focus here led to infinite recursion so
     // m_resetFocus is used as a guard
-    
+
     if ( event.GetEventType() == wxEVT_SET_FOCUS )
     {
         if ( !m_resetFocus && GetTextCtrl() && !GetTextCtrl()->HasFocus() )
@@ -2088,7 +2083,7 @@ void wxComboCtrlBase::OnFocusEvent( wxFocusEvent& event )
             m_resetFocus = false;
         }
     }
-    
+
     Refresh();
 }
 
@@ -2282,7 +2277,7 @@ void wxComboCtrlBase::ShowPopup()
     wxSize ctrlSz = GetSize();
 
     screenHeight = wxSystemSettings::GetMetric( wxSYS_SCREEN_Y );
-    scrPos = GetParent()->ClientToScreen(GetPosition());
+    scrPos = GetScreenPosition();
 
     spaceAbove = scrPos.y;
     spaceBelow = screenHeight - spaceAbove - ctrlSz.y;
@@ -2576,8 +2571,9 @@ void wxComboCtrlBase::OnPopupDismiss(bool generateEvent)
 void wxComboCtrlBase::HidePopup(bool generateEvent)
 {
     // Should be able to call this without popup interface
-    if ( IsPopupWindowState(Hidden) )
+    if ( IsPopupWindowState(Hidden) || IsPopupWindowState(Closing) )
         return;
+    m_popupWinState = Closing; // To prevent from reentering
 
     // transfer value and show it in textctrl, if any
     if ( !IsPopupWindowState(Animating) )
