@@ -4,7 +4,6 @@
 // Author:      Francesco Montorsi
 // Modified by:
 // Created:     15/04/2006
-// RCS-ID:      $Id$
 // Copyright:   (c) Francesco Montorsi
 // Licence:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
@@ -39,19 +38,23 @@
 // implementation
 // ============================================================================
 
+#if defined(__WXGTK20__) && !defined(__WXUNIVERSAL__)
+    #define SetMinMaxPointSize(min, max)
+#else
+    #define SetMinMaxPointSize(min, max)  GetPickerWidget()->GetFontData()->SetRange((min), (max))
+#endif
+
 const char wxFontPickerCtrlNameStr[] = "fontpicker";
 const char wxFontPickerWidgetNameStr[] = "fontpickerwidget";
 
-wxDEFINE_EVENT(wxEVT_COMMAND_FONTPICKER_CHANGED, wxFontPickerEvent);
-IMPLEMENT_DYNAMIC_CLASS(wxFontPickerCtrl, wxPickerBase)
-IMPLEMENT_DYNAMIC_CLASS(wxFontPickerEvent, wxCommandEvent)
+wxDEFINE_EVENT(wxEVT_FONTPICKER_CHANGED, wxFontPickerEvent);
+wxIMPLEMENT_DYNAMIC_CLASS(wxFontPickerCtrl, wxPickerBase);
+wxIMPLEMENT_DYNAMIC_CLASS(wxFontPickerEvent, wxCommandEvent);
 
 // ----------------------------------------------------------------------------
 // wxFontPickerCtrl
 // ----------------------------------------------------------------------------
 
-#define M_PICKER     ((wxFontPickerWidget*)m_picker)
-
 bool wxFontPickerCtrl::Create( wxWindow *parent, wxWindowID id,
                         const wxFont &initial,
                         const wxPoint &pos, const wxSize &size,
@@ -71,9 +74,7 @@ bool wxFontPickerCtrl::Create( wxWindow *parent, wxWindowID id,
     // complete sizer creation
     wxPickerBase::PostCreation();
 
-    m_picker->Connect(wxEVT_COMMAND_FONTPICKER_CHANGED,
-            wxFontPickerEventHandler(wxFontPickerCtrl::OnFontChange),
-            NULL, this);
+    m_picker->Bind(wxEVT_FONTPICKER_CHANGED, &wxFontPickerCtrl::OnFontChange, this);
 
     return true;
 }
@@ -117,7 +118,7 @@ wxFont wxFontPickerCtrl::String2Font(const wxString &s)
 
 void wxFontPickerCtrl::SetSelectedFont(const wxFont &f)
 {
-    M_PICKER->SetSelectedFont(f);
+    GetPickerWidget()->SetSelectedFont(f);
     UpdateTextCtrlFromPicker();
 }
 
@@ -125,24 +126,17 @@ void wxFontPickerCtrl::UpdatePickerFromTextCtrl()
 {
     wxASSERT(m_text);
 
-    if (m_bIgnoreNextTextCtrlUpdate)
-    {
-        // ignore this update
-        m_bIgnoreNextTextCtrlUpdate = false;
-        return;
-    }
-
     // NB: we don't use the wxFont::wxFont(const wxString &) constructor
     //     since that constructor expects the native font description
     //     string returned by wxFont::GetNativeFontInfoDesc() and not
     //     the user-friendly one returned by wxFont::GetNativeFontInfoUserDesc()
     wxFont f = String2Font(m_text->GetValue());
-    if (!f.Ok())
+    if (!f.IsOk())
         return;     // invalid user input
 
-    if (M_PICKER->GetSelectedFont() != f)
+    if (GetPickerWidget()->GetSelectedFont() != f)
     {
-        M_PICKER->SetSelectedFont(f);
+        GetPickerWidget()->SetSelectedFont(f);
 
         // fire an event
         wxFontPickerEvent event(this, GetId(), f);
@@ -155,14 +149,22 @@ void wxFontPickerCtrl::UpdateTextCtrlFromPicker()
     if (!m_text)
         return;     // no textctrl to update
 
-    // NOTE: this SetValue() will generate an unwanted wxEVT_COMMAND_TEXT_UPDATED
-    //       which will trigger a unneeded UpdateFromTextCtrl(); thus before using
-    //       SetValue() we set the m_bIgnoreNextTextCtrlUpdate flag...
-    m_bIgnoreNextTextCtrlUpdate = true;
-    m_text->SetValue(Font2String(M_PICKER->GetSelectedFont()));
+    // Take care to use ChangeValue() here and not SetValue() to avoid
+    // infinite recursion.
+    m_text->ChangeValue(Font2String(GetPickerWidget()->GetSelectedFont()));
 }
 
+void wxFontPickerCtrl::SetMinPointSize(unsigned int min)
+{
+    m_nMinPointSize = min;
+    SetMinMaxPointSize(m_nMinPointSize, m_nMaxPointSize);
+}
 
+void wxFontPickerCtrl::SetMaxPointSize(unsigned int max)
+{
+    m_nMaxPointSize = max;
+    SetMinMaxPointSize(m_nMinPointSize, m_nMaxPointSize);
+}
 
 // ----------------------------------------------------------------------------
 // wxFontPickerCtrl - event handlers
