@@ -3,7 +3,6 @@
 // Purpose:     wxGIFDecoder, GIF reader for wxImage and wxAnimation
 // Author:      Guillermo Rodriguez Garcia <guille@iies.es>
 // Version:     3.04
-// RCS-ID:      $Id$
 // Copyright:   (c) Guillermo Rodriguez Garcia
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -26,10 +25,22 @@
 #include <stdlib.h>
 #include <string.h>
 #include "wx/gifdecod.h"
+#include "wx/scopedarray.h"
 #include "wx/scopedptr.h"
 #include "wx/scopeguard.h"
 
+enum
+{
+    GIF_MARKER_EXT       = '!', // 0x21
+    GIF_MARKER_SEP       = ',', // 0x2C
+    GIF_MARKER_ENDOFDATA = ';', // 0x3B
 
+    GIF_MARKER_EXT_GRAPHICS_CONTROL = 0xF9,
+    GIF_MARKER_EXT_COMMENT          = 0xFE,
+    GIF_MARKER_EXT_APP              = 0xFF
+};
+
+#define GetFrame(n)     ((GIFImage*)m_frames[n])
 
 //---------------------------------------------------------------------------
 // GIFImage
@@ -52,6 +63,7 @@ public:
     unsigned char *p;               // bitmap
     unsigned char *pal;             // palette
     unsigned int ncolours;          // number of colours
+    wxString comment;
 
     wxDECLARE_NO_COPY_CLASS(GIFImage);
 };
@@ -118,15 +130,17 @@ bool wxGIFDecoder::ConvertToImage(unsigned int frame, wxImage *image) const
     unsigned long i;
     int      transparent;
 
-    // just in case...
-    image->Destroy();
+    // Store the original value of the transparency option, before it is reset
+    // by Create().
+    const wxString&
+        transparency = image->GetOption(wxIMAGE_OPTION_GIF_TRANSPARENCY);
 
     // create the image
     wxSize sz = GetFrameSize(frame);
     image->Create(sz.GetWidth(), sz.GetHeight());
     image->SetType(wxBITMAP_TYPE_GIF);
 
-    if (!image->Ok())
+    if (!image->IsOk())
         return false;
 
     pal = GetPalette(frame);
@@ -137,24 +151,58 @@ bool wxGIFDecoder::ConvertToImage(unsigned int frame, wxImage *image) const
     // set transparent colour mask
     if (transparent != -1)
     {
-        for (i = 0; i < GetNcolours(frame); i++)
+        if ( transparency.empty() ||
+                transparency == wxIMAGE_OPTION_GIF_TRANSPARENCY_HIGHLIGHT )
         {
-            if ((pal[3 * i + 0] == 255) &&
-                (pal[3 * i + 1] == 0) &&
-                (pal[3 * i + 2] == 255))
+            // By default, we assign bright pink to transparent pixels to make
+            // them perfectly noticeable if someone accidentally draws the
+            // image without taking transparency into account. Due to this use
+            // of pink, we need to change any existing image pixels with this
+            // colour to use something different.
+            for (i = 0; i < GetNcolours(frame); i++)
             {
-                pal[3 * i + 2] = 254;
+                if ((pal[3 * i + 0] == 255) &&
+                    (pal[3 * i + 1] == 0) &&
+                    (pal[3 * i + 2] == 255))
+                {
+                    pal[3 * i + 2] = 254;
+                }
             }
+
+            pal[3 * transparent + 0] = 255;
+            pal[3 * transparent + 1] = 0;
+            pal[3 * transparent + 2] = 255;
+
+            image->SetMaskColour(255, 0, 255);
         }
+        else if ( transparency == wxIMAGE_OPTION_GIF_TRANSPARENCY_UNCHANGED )
+        {
+            // Leave the GIF exactly as it was, just adjust (in the least
+            // noticeable way, by just flipping a single bit) non-transparent
+            // pixels colour,
+            for (i = 0; i < GetNcolours(frame); i++)
+            {
+                if ((pal[3 * i + 0] == pal[3 * transparent + 0]) &&
+                    (pal[3 * i + 1] == pal[3 * transparent + 1]) &&
+                    (pal[3 * i + 2] == pal[3 * transparent + 2]))
+                {
+                    pal[3 * i + 2] ^= 1;
+                }
+            }
 
-        pal[3 * transparent + 0] = 255,
-        pal[3 * transparent + 1] = 0,
-        pal[3 * transparent + 2] = 255;
-
-        image->SetMaskColour(255, 0, 255);
+            image->SetMaskColour(pal[3 * transparent + 0],
+                                 pal[3 * transparent + 1],
+                                 pal[3 * transparent + 2]);
+        }
+        else
+        {
+            wxFAIL_MSG( wxS("Unknown wxIMAGE_OPTION_GIF_TRANSPARENCY value") );
+        }
     }
     else
+    {
         image->SetMask(false);
+    }
 
 #if wxUSE_PALETTE
     unsigned char r[256];
@@ -180,6 +228,12 @@ bool wxGIFDecoder::ConvertToImage(unsigned int frame, wxImage *image) const
         *(dst++) = pal[3 * (*src) + 2];
     }
 
+    wxString comment = GetFrame(frame)->comment;
+    if ( !comment.empty() )
+    {
+        image->SetOption(wxIMAGE_OPTION_GIF_COMMENT, comment);
+    }
+
     return true;
 }
 
@@ -188,9 +242,6 @@ bool wxGIFDecoder::ConvertToImage(unsigned int frame, wxImage *image) const
 // Data accessors
 //---------------------------------------------------------------------------
 
-#define GetFrame(n)     ((GIFImage*)m_frames[n])
-
-
 // Get data for current frame
 
 wxSize wxGIFDecoder::GetFrameSize(unsigned int frame) const
@@ -254,7 +305,7 @@ int wxGIFDecoder::getcode(wxInputStream& stream, int bits, int ab_fin)
         // if no bytes left in this block, read the next block
         if (m_restbyte == 0)
         {
-            m_restbyte = (unsigned char)stream.GetC();
+            m_restbyte = stream.GetC();
 
             /* Some encoders are a bit broken: instead of issuing
              * an end-of-image symbol (ab_fin) they come up with
@@ -303,26 +354,18 @@ wxGIFErrorCode
 wxGIFDecoder::dgif(wxInputStream& stream, GIFImage *img, int interl, int bits)
 {
     static const int allocSize = 4096 + 1;
-    int *ab_prefix = new int[allocSize]; // alphabet (prefixes)
-    if (ab_prefix == NULL)
-    {
-        return wxGIF_MEMERR;
-    }
 
-    int *ab_tail = new int[allocSize];   // alphabet (tails)
-    if (ab_tail == NULL)
-    {
-        delete[] ab_prefix;
+    wxScopedArray<int> ab_prefix(allocSize); // alphabet (prefixes)
+    if ( !ab_prefix )
         return wxGIF_MEMERR;
-    }
 
-    int *stack = new int[allocSize];     // decompression stack
-    if (stack == NULL)
-    {
-        delete[] ab_prefix;
-        delete[] ab_tail;
+    wxScopedArray<int> ab_tail(allocSize);   // alphabet (tails)
+    if ( !ab_tail )
+        return wxGIF_MEMERR;
+
+    wxScopedArray<int> stack(allocSize);     // decompression stack
+    if ( !stack )
         return wxGIF_MEMERR;
-    }
 
     int ab_clr;                     // clear code
     int ab_fin;                     // end of info code
@@ -339,7 +382,7 @@ wxGIFDecoder::dgif(wxInputStream& stream, GIFImage *img, int interl, int bits)
     ab_clr = (1 << bits);
     ab_fin = (1 << bits) + 1;
 
-    // these will change through the decompression proccess
+    // these will change through the decompression process
     ab_bits  = bits + 1;
     ab_free  = (1 << bits) + 2;
     ab_max   = (1 << ab_bits) - 1;
@@ -392,21 +435,11 @@ wxGIFDecoder::dgif(wxInputStream& stream, GIFImage *img, int interl, int bits)
             // GIF files, the allocSize of 4096+1 is enough. This
             // will only happen with badly formed GIFs.
             if (pos >= allocSize)
-            {
-                delete[] ab_prefix;
-                delete[] ab_tail;
-                delete[] stack;
                 return wxGIF_INVFORMAT;
-            }
         }
 
         if (pos >= allocSize)
-        {
-            delete[] ab_prefix;
-            delete[] ab_tail;
-            delete[] stack;
             return wxGIF_INVFORMAT;
-        }
 
         stack[pos] = code;              // push last code into the stack
         abcabca    = code;              // save for special case
@@ -419,12 +452,7 @@ wxGIFDecoder::dgif(wxInputStream& stream, GIFImage *img, int interl, int bits)
             // to reset it. This checks whether we really got it, otherwise
             // the GIF is damaged.
             if (ab_free > ab_max)
-            {
-                delete[] ab_prefix;
-                delete[] ab_tail;
-                delete[] stack;
                 return wxGIF_INVFORMAT;
-            }
 
             // This assert seems unnecessary since the condition above
             // eliminates the only case in which it went false. But I really
@@ -564,10 +592,6 @@ as an End of Information itself)
     }
     while (code != ab_fin);
 
-    delete [] ab_prefix ;
-    delete [] ab_tail ;
-    delete [] stack ;
-
     return wxGIF_OK;
 }
 
@@ -663,15 +687,16 @@ wxGIFErrorCode wxGIFDecoder::LoadGIF(wxInputStream& stream)
     int transparent = -1;
     disposal = wxANIM_UNSPECIFIED;
     delay = -1;
+    wxString comment;
 
     bool done = false;
     while (!done)
     {
-        type = (unsigned char)stream.GetC();
+        type = stream.GetC();
 
         /*
         If the end of file has been reached (or an error) and a ";"
-        (0x3B) hasn't been encountered yet, exit the loop. (Without this
+        (GIF_MARKER_ENDOFDATA) hasn't been encountered yet, exit the loop. (Without this
         check the while loop would loop endlessly.) Later on, in the next while
         loop, the file will be treated as being truncated (But still
         be decoded as far as possible). returning wxGIF_TRUNCATED is not
@@ -686,150 +711,179 @@ wxGIFErrorCode wxGIFDecoder::LoadGIF(wxInputStream& stream)
             break; // Alternative : "return wxGIF_INVFORMAT;"
         }
 
-        // end of data?
-        if (type == 0x3B)
+        switch (type)
         {
-            done = true;
-        }
-        else
-        // extension block?
-        if (type == 0x21)
-        {
-            if (((unsigned char)stream.GetC()) == 0xF9)
-            // graphics control extension, parse it
-            {
-                static const unsigned int gceSize = 6;
-                stream.Read(buf, gceSize);
-                if (stream.LastRead() != gceSize)
+            case GIF_MARKER_ENDOFDATA:
+                done = true;
+                break;
+            case GIF_MARKER_EXT:
+                switch (stream.GetC())
                 {
-                    Destroy();
-                    return wxGIF_INVFORMAT;
-                }
-
-                // read delay and convert from 1/100 of a second to ms
-                delay = 10 * (buf[2] + 256 * buf[3]);
-
-                // read transparent colour index, if used
-                transparent = buf[1] & 0x01 ? buf[4] : -1;
-
-                // read disposal method
-                disposal = (wxAnimationDisposal)(((buf[1] & 0x1C) >> 2) - 1);
-            }
-            else
-            // other extension, skip
-            {
-                while ((i = (unsigned char)stream.GetC()) != 0)
-                {
-                    if (stream.Eof() || (stream.LastRead() == 0) ||
-                        stream.SeekI(i, wxFromCurrent) == wxInvalidOffset)
+                    case GIF_MARKER_EXT_GRAPHICS_CONTROL:
                     {
-                        done = true;
+                        // graphics control extension, parse it
+
+                        static const unsigned int gceSize = 6;
+                        stream.Read(buf, gceSize);
+                        if (stream.LastRead() != gceSize)
+                        {
+                            Destroy();
+                            return wxGIF_INVFORMAT;
+                        }
+
+                        // read delay and convert from 1/100 of a second to ms
+                        delay = 10 * (buf[2] + 256 * buf[3]);
+
+                        // read transparent colour index, if used
+                        transparent = buf[1] & 0x01 ? buf[4] : -1;
+
+                        // read disposal method
+                        disposal = (wxAnimationDisposal)(((buf[1] & 0x1C) >> 2) - 1);
                         break;
                     }
-                }
-            }
-        }
-        else
-        // image descriptor block?
-        if (type == 0x2C)
-        {
-            // allocate memory for IMAGEN struct
-            GIFImagePtr pimg(new GIFImage());
-
-            wxScopeGuard guardDestroy = wxMakeObjGuard(*this, &wxGIFDecoder::Destroy);
-
-            if ( !pimg.get() )
-                return wxGIF_MEMERR;
-
-            // fill in the data
-            static const unsigned int idbSize = (2 + 2 + 2 + 2 + 1);
-            stream.Read(buf, idbSize);
-            if (stream.LastRead() != idbSize)
-                return wxGIF_INVFORMAT;
-
-            pimg->left = buf[0] + 256 * buf[1];
-            pimg->top = buf[2] + 256 * buf[3];
-/*
-            pimg->left = buf[4] + 256 * buf[5];
-            pimg->top = buf[4] + 256 * buf[5];
-*/
-            pimg->w = buf[4] + 256 * buf[5];
-            pimg->h = buf[6] + 256 * buf[7];
-
-            if ( anim )
-            {
-                // some GIF images specify incorrect animation size but we can
-                // still open them if we fix up the animation size, see #9465
-                if ( m_nFrames == 0 )
-                {
-                    if ( pimg->w > (unsigned)m_szAnimation.x )
-                        m_szAnimation.x = pimg->w;
-                    if ( pimg->h > (unsigned)m_szAnimation.y )
-                        m_szAnimation.y = pimg->h;
-                }
-                else // subsequent frames
-                {
-                    // check that we have valid size
-                    if ( (!pimg->w || pimg->w > (unsigned)m_szAnimation.x) ||
-                            (!pimg->h || pimg->h > (unsigned)m_szAnimation.y) )
+                    case GIF_MARKER_EXT_COMMENT:
                     {
-                        wxLogError(_("Incorrect GIF frame size (%u, %d) for "
-                                     "the frame #%u"),
-                                   pimg->w, pimg->h, m_nFrames);
-                        return wxGIF_INVFORMAT;
+                        int len = stream.GetC();
+                        while (len)
+                        {
+                            if ( stream.Eof() )
+                            {
+                                done = true;
+                                break;
+                            }
+
+                            wxCharBuffer charbuf(len);
+                            stream.Read(charbuf.data(), len);
+                            if ( (int) stream.LastRead() != len )
+                            {
+                                done = true;
+                                break;
+                            }
+
+                            comment += wxConvertMB2WX(charbuf.data());
+
+                            len = stream.GetC();
+                        }
+
+                        break;
+                    }
+                    default:
+                        // other extension, skip
+                        while ((i = stream.GetC()) != 0)
+                        {
+                            if (stream.Eof() || (stream.LastRead() == 0) ||
+                                stream.SeekI(i, wxFromCurrent) == wxInvalidOffset)
+                            {
+                                done = true;
+                                break;
+                            }
+                        }
+                        break;
+                }
+                break;
+            case GIF_MARKER_SEP:
+            {
+                // allocate memory for IMAGEN struct
+                GIFImagePtr pimg(new GIFImage());
+
+                wxScopeGuard guardDestroy = wxMakeObjGuard(*this, &wxGIFDecoder::Destroy);
+
+                if ( !pimg.get() )
+                    return wxGIF_MEMERR;
+
+                // fill in the data
+                static const unsigned int idbSize = (2 + 2 + 2 + 2 + 1);
+                stream.Read(buf, idbSize);
+                if (stream.LastRead() != idbSize)
+                    return wxGIF_INVFORMAT;
+
+                pimg->comment = comment;
+                comment.clear();
+                pimg->left = buf[0] + 256 * buf[1];
+                pimg->top = buf[2] + 256 * buf[3];
+    /*
+                pimg->left = buf[4] + 256 * buf[5];
+                pimg->top = buf[4] + 256 * buf[5];
+    */
+                pimg->w = buf[4] + 256 * buf[5];
+                pimg->h = buf[6] + 256 * buf[7];
+
+                if ( anim )
+                {
+                    // some GIF images specify incorrect animation size but we can
+                    // still open them if we fix up the animation size, see #9465
+                    if ( m_nFrames == 0 )
+                    {
+                        if ( pimg->w > (unsigned)m_szAnimation.x )
+                            m_szAnimation.x = pimg->w;
+                        if ( pimg->h > (unsigned)m_szAnimation.y )
+                            m_szAnimation.y = pimg->h;
+                    }
+                    else // subsequent frames
+                    {
+                        // check that we have valid size
+                        if ( (!pimg->w || pimg->w > (unsigned)m_szAnimation.x) ||
+                                (!pimg->h || pimg->h > (unsigned)m_szAnimation.y) )
+                        {
+                            wxLogError(_("Incorrect GIF frame size (%u, %d) for "
+                                         "the frame #%u"),
+                                       pimg->w, pimg->h, m_nFrames);
+                            return wxGIF_INVFORMAT;
+                        }
                     }
                 }
-            }
 
-            interl = ((buf[8] & 0x40)? 1 : 0);
-            size = pimg->w * pimg->h;
+                interl = ((buf[8] & 0x40)? 1 : 0);
+                size = pimg->w * pimg->h;
 
-            pimg->transparent = transparent;
-            pimg->disposal = disposal;
-            pimg->delay = delay;
+                pimg->transparent = transparent;
+                pimg->disposal = disposal;
+                pimg->delay = delay;
 
-            // allocate memory for image and palette
-            pimg->p   = (unsigned char *) malloc((unsigned int)size);
-            pimg->pal = (unsigned char *) malloc(768);
+                // allocate memory for image and palette
+                pimg->p   = (unsigned char *) malloc((unsigned int)size);
+                pimg->pal = (unsigned char *) malloc(768);
 
-            if ((!pimg->p) || (!pimg->pal))
-                return wxGIF_MEMERR;
+                if ((!pimg->p) || (!pimg->pal))
+                    return wxGIF_MEMERR;
 
-            // load local color map if available, else use global map
-            if ((buf[8] & 0x80) == 0x80)
-            {
-                unsigned int local_ncolors = 2 << (buf[8] & 0x07);
-                unsigned int numBytes = 3 * local_ncolors;
-                stream.Read(pimg->pal, numBytes);
-                pimg->ncolours = local_ncolors;
-                if (stream.LastRead() != numBytes)
+                // load local color map if available, else use global map
+                if ((buf[8] & 0x80) == 0x80)
+                {
+                    unsigned int local_ncolors = 2 << (buf[8] & 0x07);
+                    unsigned int numBytes = 3 * local_ncolors;
+                    stream.Read(pimg->pal, numBytes);
+                    pimg->ncolours = local_ncolors;
+                    if (stream.LastRead() != numBytes)
+                        return wxGIF_INVFORMAT;
+                }
+                else
+                {
+                    memcpy(pimg->pal, pal, 768);
+                    pimg->ncolours = global_ncolors;
+                }
+
+                // get initial code size from first byte in raster data
+                bits = stream.GetC();
+                if (bits == 0)
                     return wxGIF_INVFORMAT;
+
+                // decode image
+                wxGIFErrorCode result = dgif(stream, pimg.get(), interl, bits);
+                if (result != wxGIF_OK)
+                    return result;
+
+                guardDestroy.Dismiss();
+
+                // add the image to our frame array
+                m_frames.Add(pimg.release());
+                m_nFrames++;
+
+                // if this is not an animated GIF, exit after first image
+                if (!anim)
+                    done = true;
+                break;
             }
-            else
-            {
-                memcpy(pimg->pal, pal, 768);
-                pimg->ncolours = global_ncolors;
-            }
-
-            // get initial code size from first byte in raster data
-            bits = (unsigned char)stream.GetC();
-            if (bits == 0)
-                return wxGIF_INVFORMAT;
-
-            // decode image
-            wxGIFErrorCode result = dgif(stream, pimg.get(), interl, bits);
-            if (result != wxGIF_OK)
-                return result;
-
-            guardDestroy.Dismiss();
-
-            // add the image to our frame array
-            m_frames.Add(pimg.release());
-            m_nFrames++;
-
-            // if this is not an animated GIF, exit after first image
-            if (!anim)
-                done = true;
         }
     }
 
@@ -840,75 +894,80 @@ wxGIFErrorCode wxGIFDecoder::LoadGIF(wxInputStream& stream)
     }
 
     // try to read to the end of the stream
-    while (type != 0x3B)
+    while (type != GIF_MARKER_ENDOFDATA)
     {
         if (!stream.IsOk())
             return wxGIF_TRUNCATED;
 
-        type = (unsigned char)stream.GetC();
+        type = stream.GetC();
 
-        if (type == 0x21)
+        switch (type)
         {
-            // extension type
-            (void) stream.GetC();
+            case GIF_MARKER_EXT:
+                // extension type
+                (void) stream.GetC();
 
-            // skip all data
-            while ((i = (unsigned char)stream.GetC()) != 0)
+                // skip all data
+                while ((i = stream.GetC()) != 0)
+                {
+                    if (stream.Eof() || (stream.LastRead() == 0) ||
+                        stream.SeekI(i, wxFromCurrent) == wxInvalidOffset)
+                    {
+                        Destroy();
+                        return wxGIF_INVFORMAT;
+                    }
+                }
+                break;
+            case GIF_MARKER_SEP:
             {
-                if (stream.Eof() || (stream.LastRead() == 0) ||
-                    stream.SeekI(i, wxFromCurrent) == wxInvalidOffset)
+                // image descriptor block
+                static const unsigned int idbSize = (2 + 2 + 2 + 2 + 1);
+                stream.Read(buf, idbSize);
+                if (stream.LastRead() != idbSize)
                 {
                     Destroy();
                     return wxGIF_INVFORMAT;
                 }
-            }
-        }
-        else if (type == 0x2C)
-        {
-            // image descriptor block
-            static const unsigned int idbSize = (2 + 2 + 2 + 2 + 1);
-            stream.Read(buf, idbSize);
-            if (stream.LastRead() != idbSize)
-            {
-                Destroy();
-                return wxGIF_INVFORMAT;
-            }
 
-            // local color map
-            if ((buf[8] & 0x80) == 0x80)
-            {
-                unsigned int local_ncolors = 2 << (buf[8] & 0x07);
-                wxFileOffset numBytes = 3 * local_ncolors;
-                if (stream.SeekI(numBytes, wxFromCurrent) == wxInvalidOffset)
+                // local color map
+                if ((buf[8] & 0x80) == 0x80)
+                {
+                    unsigned int local_ncolors = 2 << (buf[8] & 0x07);
+                    wxFileOffset numBytes = 3 * local_ncolors;
+                    if (stream.SeekI(numBytes, wxFromCurrent) == wxInvalidOffset)
+                    {
+                        Destroy();
+                        return wxGIF_INVFORMAT;
+                    }
+                }
+
+                // initial code size
+                (void) stream.GetC();
+                if (stream.Eof() || (stream.LastRead() == 0))
                 {
                     Destroy();
                     return wxGIF_INVFORMAT;
                 }
-            }
 
-            // initial code size
-            (void) stream.GetC();
-            if (stream.Eof() || (stream.LastRead() == 0))
-            {
-                Destroy();
-                return wxGIF_INVFORMAT;
-            }
-
-            // skip all data
-            while ((i = (unsigned char)stream.GetC()) != 0)
-            {
-                if (stream.Eof() || (stream.LastRead() == 0) ||
-                    stream.SeekI(i, wxFromCurrent) == wxInvalidOffset)
+                // skip all data
+                while ((i = stream.GetC()) != 0)
                 {
-                    Destroy();
-                    return wxGIF_INVFORMAT;
+                    if (stream.Eof() || (stream.LastRead() == 0) ||
+                        stream.SeekI(i, wxFromCurrent) == wxInvalidOffset)
+                    {
+                        Destroy();
+                        return wxGIF_INVFORMAT;
+                    }
                 }
+                break;
             }
-        }
-        else if ((type != 0x3B) && (type != 00)) // testing
-        {
-            // images are OK, but couldn't read to the end of the stream
-            return wxGIF_TRUNCATED;
+            default:
+                if ((type != GIF_MARKER_ENDOFDATA) && (type != 00)) // testing
+                {
+                    // images are OK, but couldn't read to the end of the stream
+                    return wxGIF_TRUNCATED;
+                }
+                break;
         }
     }
 
