@@ -1,8 +1,7 @@
 /////////////////////////////////////////////////////////////////////////////
-// Name:        xpmdecod.cpp
+// Name:        src/common/xpmdecod.cpp
 // Purpose:     wxXPMDecoder
 // Author:      John Cristy, Vaclav Slavik
-// RCS-ID:      $Id: xpmdecod.cpp,v 1.43.2.2 2006/01/18 16:32:46 JS Exp $
 // Copyright:   (c) John Cristy, Vaclav Slavik
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -90,35 +89,30 @@ license is as follows:
  * in this Software without prior written authorization from GROUPE BULL.
  */
 
-#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
-#pragma implementation "xpmdecod.h"
-#endif
-
 // For compilers that support precompilation, includes "wx.h".
 #include "wx/wxprec.h"
 
 #ifdef __BORLANDC__
-#  pragma hdrstop
-#endif
-
-#ifndef WX_PRECOMP
-#  include "wx/defs.h"
+    #pragma hdrstop
 #endif
 
 #if wxUSE_IMAGE && wxUSE_XPM
 
-#include "wx/stream.h"
-#include "wx/image.h"
-#include "wx/utils.h"
-#include "wx/log.h"
-#include "wx/hashmap.h"
-#include "wx/intl.h"
-#include <string.h>
-
-#include <ctype.h>
-
 #include "wx/xpmdecod.h"
 
+#ifndef WX_PRECOMP
+    #include "wx/intl.h"
+    #include "wx/log.h"
+    #include "wx/utils.h"
+    #include "wx/hashmap.h"
+    #include "wx/stream.h"
+    #include "wx/image.h"
+    #include "wx/palette.h"
+#endif
+
+#include <string.h>
+#include <ctype.h>
+
 #if wxUSE_STREAMS
 bool wxXPMDecoder::CanRead(wxInputStream& stream)
 {
@@ -127,8 +121,6 @@ bool wxXPMDecoder::CanRead(wxInputStream& stream)
     if ( !stream.Read(buf, WXSIZEOF(buf)) )
         return false;
 
-    stream.SeekI(-(wxFileOffset)WXSIZEOF(buf), wxFromCurrent);
-
     return memcmp(buf, "/* XPM */", WXSIZEOF(buf)) == 0;
 }
 
@@ -221,7 +213,7 @@ wxImage wxXPMDecoder::ReadFile(wxInputStream& stream)
         return wxNullImage;
     }
 
-    xpm_lines = new const char*[lines_cnt];
+    xpm_lines = new const char*[lines_cnt + 1];
     xpm_lines[0] = xpm_buffer;
     line = 1;
     for (p = xpm_buffer; (*p != '\0') && (line < lines_cnt); p++)
@@ -234,6 +226,8 @@ wxImage wxXPMDecoder::ReadFile(wxInputStream& stream)
         }
     }
 
+    xpm_lines[lines_cnt] = NULL;
+
     /*
      *  Read the image:
      */
@@ -252,7 +246,7 @@ wxImage wxXPMDecoder::ReadFile(wxInputStream& stream)
 * A hard coded rgb.txt. To keep it short I removed all colornames with        *
 * trailing numbers, Blue3 etc, except the GrayXX. Sorry Grey-lovers I prefer  *
 * Gray ;-). But Grey is recognized on lookups, only on save Gray will be      *
-* used, maybe you want to do some substitue there too.                        *
+* used, maybe you want to do some substitute there too.                        *
 *                                                                             *
 * To save memory the RGBs are coded in one long value, as done by the RGB     *
 * macro.                                                                      *
@@ -269,7 +263,7 @@ typedef struct
 
 #define myRGB(r,g,b)   ((wxUint32)r<<16|(wxUint32)g<<8|(wxUint32)b)
 
-static rgbRecord theRGBRecords[] =
+static const rgbRecord theRGBRecords[] =
 {
     {"aliceblue", myRGB(240, 248, 255)},
     {"antiquewhite", myRGB(250, 235, 215)},
@@ -508,7 +502,7 @@ static rgbRecord theRGBRecords[] =
     {"yellowgreen", myRGB(50, 216, 56)},
     {NULL, myRGB(0, 0, 0)}
 };
-static int numTheRGBRecords = 235;
+static const int numTheRGBRecords = 235;
 
 static unsigned char ParseHexadecimal(char digit1, char digit2)
 {
@@ -623,7 +617,7 @@ static bool GetRGBFromName(const char *inname, bool *isNone,
 
 static const char *ParseColor(const char *data)
 {
-    static const char *targets[] =
+    static const char *const targets[] =
                         {"c ", "g ", "g4 ", "m ", "b ", "s ", NULL};
 
     const char *p, *r;
@@ -655,22 +649,25 @@ static const char *ParseColor(const char *data)
 
 struct wxXPMColourMapData
 {
+    wxXPMColourMapData() { R = G = B = 0; }
     unsigned char R,G,B;
 };
 WX_DECLARE_STRING_HASH_MAP(wxXPMColourMapData, wxXPMColourMap);
 
-wxImage wxXPMDecoder::ReadData(const char **xpm_data)
+wxImage wxXPMDecoder::ReadData(const char* const* xpm_data)
 {
+    wxCHECK_MSG(xpm_data, wxNullImage, wxT("NULL XPM data") );
+
     wxImage img;
     int count;
     unsigned width, height, colors_cnt, chars_per_pixel;
     size_t i, j, i_key;
-    wxChar key[64];
+    char key[64];
     const char *clr_def;
-    bool hasMask;
     wxXPMColourMap clr_tbl;
     wxXPMColourMap::iterator it;
     wxString maskKey;
+    wxString keyString;
 
     /*
      *  Read hints and initialize structures:
@@ -680,7 +677,7 @@ wxImage wxXPMDecoder::ReadData(const char **xpm_data)
                    &width, &height, &colors_cnt, &chars_per_pixel);
     if ( count != 4 || width * height * colors_cnt == 0 )
     {
-        wxLogError(_T("XPM: Not XPM data!"));
+        wxLogError(_("XPM: incorrect header format!"));
         return wxNullImage;
     }
 
@@ -689,68 +686,82 @@ wxImage wxXPMDecoder::ReadData(const char **xpm_data)
     //     8bit RGB...
     wxCHECK_MSG(chars_per_pixel < 64, wxNullImage, wxT("XPM colormaps this large not supported."));
 
-    img.Create(width, height);
-    if ( !img.Ok() ) return img;
+    if (!img.Create(width, height, false))
+        return wxNullImage;
 
-    img.SetMask(false);
-    key[chars_per_pixel] = wxT('\0');
-    hasMask = false;
+    key[chars_per_pixel] = '\0';
 
     /*
      *  Create colour map:
      */
+    wxXPMColourMapData clr_data;
     for (i = 0; i < colors_cnt; i++)
     {
-        wxXPMColourMapData clr_data = {255,0,255};
+        const char *xmpColLine = xpm_data[1 + i];
+
+        // we must have at least " x y" after the colour index, hence +5
+        if ( !xmpColLine || strlen(xmpColLine) < chars_per_pixel + 5 )
+        {
+            wxLogError(_("XPM: incorrect colour description in line %d"),
+                       (int)(1 + i));
+            return wxNullImage;
+        }
 
         for (i_key = 0; i_key < chars_per_pixel; i_key++)
-            key[i_key] = (wxChar)xpm_data[1 + i][i_key];
-        clr_def = ParseColor(xpm_data[1 + i] + chars_per_pixel);
+            key[i_key] = xmpColLine[i_key];
+        clr_def = ParseColor(xmpColLine + chars_per_pixel);
 
         if ( clr_def == NULL )
         {
-            wxLogError(_("XPM: malformed colour definition '%s'!"),
-                       xpm_data[1+i]);
+            wxLogError(_("XPM: malformed colour definition '%s' at line %d!"),
+                       xmpColLine, (int)(1 + i));
+            return wxNullImage;
         }
-        else
+
+        bool isNone = false;
+        if ( !GetRGBFromName(clr_def, &isNone,
+                             &clr_data.R, &clr_data.G, &clr_data.B) )
         {
-            bool isNone = false;
-            if ( !GetRGBFromName(clr_def, &isNone,
-                                 &clr_data.R, &clr_data.G, &clr_data.B) )
-            {
-                wxLogError(_("XPM: malformed colour definition '%s'!"),
-                           xpm_data[1+i]);
-            }
-            else
-            {
-                if ( isNone )
-                {
-                    img.SetMask(true);
-                    img.SetMaskColour(255, 0, 255);
-                    hasMask = true;
-                    maskKey = key;
-                }
-            }
+            wxLogError(_("XPM: malformed colour definition '%s' at line %d!"),
+                       xmpColLine, (int)(1 + i));
+            return wxNullImage;
         }
-        clr_tbl[key] = clr_data;
+
+        keyString = key;
+        if ( isNone )
+            maskKey = keyString;
+
+        clr_tbl[keyString] = clr_data;
     }
 
-    /*
-     *  Modify colour entries with RGB = (255,0,255) to (255,0,254) if
-     *  mask colour is present (so that existing pixels with (255,0,255)
-     *  magenta colour are not incorrectly made transparent):
-     */
-    if (hasMask)
+    // deal with the mask: we must replace pseudo-colour "None" with the mask
+    // colour (which can be any colour not otherwise used in the image)
+    if (!maskKey.empty())
     {
-        for (it = clr_tbl.begin(); it != clr_tbl.end(); ++it)
+        wxLongToLongHashMap rgb_table;
+        long rgb;
+        const size_t n = clr_tbl.size();
+        wxXPMColourMap::const_iterator iter = clr_tbl.begin();
+        for (i = 0; i < n; ++i, ++iter)
         {
-            if (it->second.R == 255 && it->second.G == 0 &&
-                it->second.B == 255 &&
-                it->first != maskKey)
-            {
-                it->second.B = 254;
-            }
+            const wxXPMColourMapData& data = iter->second;
+            rgb = (data.R << 16) + (data.G << 8) + data.B;
+            rgb_table[rgb];
         }
+        for (rgb = 0; rgb <= 0xffffff && rgb_table.count(rgb); ++rgb)
+            ;
+        if (rgb > 0xffffff)
+        {
+            wxLogError(_("XPM: no colors left to use for mask!"));
+            return wxNullImage;
+        }
+
+        wxXPMColourMapData& maskData = clr_tbl[maskKey];
+        maskData.R = wxByte(rgb >> 16);
+        maskData.G = wxByte(rgb >> 8);
+        maskData.B = wxByte(rgb);
+
+        img.SetMaskColour(maskData.R, maskData.G, maskData.B);
     }
 
     /*
@@ -765,10 +776,21 @@ wxImage wxXPMDecoder::ReadData(const char **xpm_data)
     {
         for (i = 0; i < width; i++, img_data += 3)
         {
+            const char *xpmImgLine = xpm_data[1 + colors_cnt + j];
+            if ( !xpmImgLine || strlen(xpmImgLine) < width*chars_per_pixel )
+            {
+                wxLogError(_("XPM: truncated image data at line %d!"),
+                           (int)(1 + colors_cnt + j));
+                return wxNullImage;
+            }
+
             for (i_key = 0; i_key < chars_per_pixel; i_key++)
-                key[i_key] = (wxChar)xpm_data[1 + colors_cnt + j]
-                                             [chars_per_pixel * i + i_key];
-            entry = clr_tbl.find(key);
+            {
+                key[i_key] = xpmImgLine[chars_per_pixel * i + i_key];
+            }
+
+            keyString = key;
+            entry = clr_tbl.find(keyString);
             if ( entry == end )
             {
                 wxLogError(_("XPM: Malformed pixel data!"));
@@ -779,15 +801,29 @@ wxImage wxXPMDecoder::ReadData(const char **xpm_data)
                 // each remaining pixel if we don't bail out
                 return wxNullImage;
             }
-            else
-            {
-                img_data[0] = entry->second.R;
-                img_data[1] = entry->second.G;
-                img_data[2] = entry->second.B;
-            }
+
+            img_data[0] = entry->second.R;
+            img_data[1] = entry->second.G;
+            img_data[2] = entry->second.B;
         }
     }
+#if wxUSE_PALETTE
+    unsigned char* r = new unsigned char[colors_cnt];
+    unsigned char* g = new unsigned char[colors_cnt];
+    unsigned char* b = new unsigned char[colors_cnt];
 
+    for (it = clr_tbl.begin(), i = 0; it != clr_tbl.end(); it++, i++)
+    {
+        r[i] = it->second.R;
+        g[i] = it->second.G;
+        b[i] = it->second.B;
+    }
+    wxASSERT(i == colors_cnt);
+    img.SetPalette(wxPalette(colors_cnt, r, g, b));
+    delete[] r;
+    delete[] g;
+    delete[] b;
+#endif // wxUSE_PALETTE
     return img;
 }
 
