@@ -5,7 +5,6 @@
 // Author:      Vadim Zeitlin
 // Modified by:
 // Created:     20.09.99 (extracted from src/common/log.cpp)
-// RCS-ID:      $Id$
 // Copyright:   (c) 1998 Vadim Zeitlin <zeitlin@dptmaths.ens-cachan.fr>
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -52,6 +51,7 @@
 #include "wx/collpane.h"
 #include "wx/arrstr.h"
 #include "wx/msgout.h"
+#include "wx/scopeguard.h"
 
 #ifdef  __WXMSW__
     // for OutputDebugString()
@@ -59,24 +59,13 @@
 #endif // Windows
 
 
-#ifdef  __WXPM__
-    #include <time.h>
-#endif
-
 #if wxUSE_LOG_DIALOG
     #include "wx/listctrl.h"
     #include "wx/imaglist.h"
     #include "wx/image.h"
 #endif // wxUSE_LOG_DIALOG/!wxUSE_LOG_DIALOG
 
-#if defined(__MWERKS__) && wxUSE_UNICODE
-    #include <wtime.h>
-#endif
-
-#include "wx/datetime.h"
-
-// the suffix we add to the button to show that the dialog can be expanded
-#define EXPAND_SUFFIX wxT(" >>")
+#include "wx/time.h"
 
 #define CAN_SAVE_FILES (wxUSE_FILE && wxUSE_FILEDLG)
 
@@ -90,7 +79,6 @@
 // allows to exclude the usage of wxDateTime
 static wxString TimeStamp(const wxString& format, time_t t)
 {
-#if wxUSE_DATETIME
     wxChar buf[4096];
     struct tm tm;
     if ( !wxStrftime(buf, WXSIZEOF(buf), format, wxLocaltime_r(&t, &tm)) )
@@ -99,9 +87,6 @@ static wxString TimeStamp(const wxString& format, time_t t)
         wxFAIL_MSG(wxT("strftime() failed"));
     }
     return wxString(buf);
-#else // !wxUSE_DATETIME
-    return wxEmptyString;
-#endif // wxUSE_DATETIME/!wxUSE_DATETIME
 }
 
 
@@ -166,11 +151,11 @@ private:
     // the maximum length of the log message
     static size_t ms_maxLength;
 
-    DECLARE_EVENT_TABLE()
+    wxDECLARE_EVENT_TABLE();
     wxDECLARE_NO_COPY_CLASS(wxLogDialog);
 };
 
-BEGIN_EVENT_TABLE(wxLogDialog, wxDialog)
+wxBEGIN_EVENT_TABLE(wxLogDialog, wxDialog)
     EVT_BUTTON(wxID_OK, wxLogDialog::OnOk)
 #if wxUSE_CLIPBOARD
     EVT_BUTTON(wxID_COPY,   wxLogDialog::OnCopy)
@@ -179,7 +164,7 @@ BEGIN_EVENT_TABLE(wxLogDialog, wxDialog)
     EVT_BUTTON(wxID_SAVE,   wxLogDialog::OnSave)
 #endif // CAN_SAVE_FILES
     EVT_LIST_ITEM_ACTIVATED(wxID_ANY, wxLogDialog::OnListItemActivated)
-END_EVENT_TABLE()
+wxEND_EVENT_TABLE()
 
 #endif // wxUSE_LOG_DIALOG
 
@@ -245,7 +230,7 @@ wxString wxLogGui::GetTitle() const
 
         default:
             wxFAIL_MSG( "unexpected icon severity" );
-            // fall through
+            wxFALLTHROUGH;
 
         case wxICON_INFORMATION:
             titleFormat = _("%s Information");
@@ -320,6 +305,10 @@ void wxLogGui::Flush()
     // showing right now: nested modal dialogs make for really bad UI!
     Suspend();
 
+    // and ensure that we allow showing the log again afterwards, even if an
+    // exception is thrown
+    wxON_BLOCK_EXIT0(wxLog::Resume);
+
     if ( nMsgCount == 1 )
     {
         // make a copy before calling Clear()
@@ -342,9 +331,6 @@ void wxLogGui::Flush()
 
         DoShowMultipleLogMessages(messages, severities, times, title, style);
     }
-
-    // allow flushing the logs again
-    Resume();
 }
 
 // log all kinds of messages
@@ -355,7 +341,6 @@ void wxLogGui::DoLogRecord(wxLogLevel level,
     switch ( level )
     {
         case wxLOG_Info:
-            if ( GetVerbose() )
         case wxLOG_Message:
             {
                 m_aMessages.Add(msg);
@@ -380,7 +365,7 @@ void wxLogGui::DoLogRecord(wxLogLevel level,
                 // find the top window and set it's status text if it has any
                 if ( pFrame == NULL ) {
                     wxWindow *pWin = wxTheApp->GetTopWindow();
-                    if ( pWin != NULL && pWin->IsKindOf(CLASSINFO(wxFrame)) ) {
+                    if ( wxDynamicCast(pWin, wxFrame) ) {
                         pFrame = (wxFrame *)pWin;
                     }
                 }
@@ -403,7 +388,7 @@ void wxLogGui::DoLogRecord(wxLogLevel level,
 #endif // wxUSE_LOG_DIALOG
                 m_bErrors = true;
             }
-            // fall through
+            wxFALLTHROUGH;
 
         case wxLOG_Warning:
             if ( !m_bErrors ) {
@@ -456,6 +441,9 @@ public:
     wxLogFrame(wxWindow *pParent, wxLogWindow *log, const wxString& szTitle);
     virtual ~wxLogFrame();
 
+    // Don't prevent the application from exiting if just this frame remains.
+    virtual bool ShouldPreventAppExit() const wxOVERRIDE { return false; }
+
     // menu callbacks
     void OnClose(wxCommandEvent& event);
     void OnCloseWindow(wxCloseEvent& event);
@@ -485,11 +473,11 @@ private:
     wxTextCtrl  *m_pTextCtrl;
     wxLogWindow *m_log;
 
-    DECLARE_EVENT_TABLE()
+    wxDECLARE_EVENT_TABLE();
     wxDECLARE_NO_COPY_CLASS(wxLogFrame);
 };
 
-BEGIN_EVENT_TABLE(wxLogFrame, wxFrame)
+wxBEGIN_EVENT_TABLE(wxLogFrame, wxFrame)
     // wxLogWindow menu events
     EVT_MENU(Menu_Close, wxLogFrame::OnClose)
 #if CAN_SAVE_FILES
@@ -498,7 +486,7 @@ BEGIN_EVENT_TABLE(wxLogFrame, wxFrame)
     EVT_MENU(Menu_Clear, wxLogFrame::OnClear)
 
     EVT_CLOSE(wxLogFrame::OnCloseWindow)
-END_EVENT_TABLE()
+wxEND_EVENT_TABLE()
 
 wxLogFrame::wxLogFrame(wxWindow *pParent, wxLogWindow *log, const wxString& szTitle)
           : wxFrame(pParent, wxID_ANY, szTitle)
@@ -521,7 +509,7 @@ wxLogFrame::wxLogFrame(wxWindow *pParent, wxLogWindow *log, const wxString& szTi
     wxMenuBar *pMenuBar = new wxMenuBar;
     wxMenu *pMenu = new wxMenu;
 #if CAN_SAVE_FILES
-    pMenu->Append(Menu_Save,  _("&Save..."), _("Save log contents to file"));
+    pMenu->Append(Menu_Save,  _("Save &As..."), _("Save log contents to file"));
 #endif // CAN_SAVE_FILES
     pMenu->Append(Menu_Clear, _("C&lear"), _("Clear the log contents"));
     pMenu->AppendSeparator();
@@ -534,8 +522,6 @@ wxLogFrame::wxLogFrame(wxWindow *pParent, wxLogWindow *log, const wxString& szTi
     // status bar for menu prompts
     CreateStatusBar();
 #endif // wxUSE_STATUSBAR
-
-    m_log->OnFrameCreate(this);
 }
 
 void wxLogFrame::DoClose()
@@ -610,6 +596,11 @@ wxLogWindow::wxLogWindow(wxWindow *pParent,
                          bool bShow,
                          bool bDoPass)
 {
+    // Initialize it to NULL to ensure that we don't crash if any log messages
+    // are generated before the frame is fully created (while this doesn't
+    // happen normally, it might, in principle).
+    m_pLogFrame = NULL;
+
     PassMessages(bDoPass);
 
     m_pLogFrame = new wxLogFrame(pParent, this, szTitle);
@@ -644,10 +635,6 @@ wxFrame *wxLogWindow::GetFrame() const
     return m_pLogFrame;
 }
 
-void wxLogWindow::OnFrameCreate(wxFrame * WXUNUSED(frame))
-{
-}
-
 bool wxLogWindow::OnFrameClose(wxFrame * WXUNUSED(frame))
 {
     // allow to close
@@ -694,9 +681,6 @@ wxLogDialog::wxLogDialog(wxWindow *parent,
         // happens to pop up a Log message while translating this :-)
         ms_details = wxTRANSLATE("&Details");
         ms_details = wxGetTranslation(ms_details);
-#ifdef __SMARTPHONE__
-        ms_details = wxStripMenuCodes(ms_details);
-#endif
     }
 
     if ( ms_maxLength == 0 )
@@ -721,7 +705,7 @@ wxLogDialog::wxLogDialog(wxWindow *parent,
     bool isPda = (wxSystemSettings::GetScreenType() <= wxSYS_SCREEN_PDA);
 
     // create the controls which are always shown and layout them: we use
-    // sizers even though our window is not resizeable to calculate the size of
+    // sizers even though our window is not resizable to calculate the size of
     // the dialog properly
     wxBoxSizer *sizerTop = new wxBoxSizer(wxVERTICAL);
     wxBoxSizer *sizerAll = new wxBoxSizer(isPda ? wxVERTICAL : wxHORIZONTAL);
@@ -751,12 +735,16 @@ wxLogDialog::wxLogDialog(wxWindow *parent,
 
 
     // add the details pane
-#ifndef __SMARTPHONE__
+#if wxUSE_COLLPANE
     wxCollapsiblePane * const
         collpane = new wxCollapsiblePane(this, wxID_ANY, ms_details);
     sizerTop->Add(collpane, wxSizerFlags(1).Expand().Border());
 
     wxWindow *win = collpane->GetPane();
+#else
+    wxPanel* win = new wxPanel(this, wxID_ANY, wxDefaultPosition, wxDefaultSize,
+                               wxBORDER_NONE);
+#endif
     wxSizer * const paneSz = new wxBoxSizer(wxVERTICAL);
 
     CreateDetailsControls(win);
@@ -777,15 +765,11 @@ wxLogDialog::wxLogDialog(wxWindow *parent,
     btnSizer->Add(new wxButton(win, wxID_SAVE), flagsBtn);
 #endif // CAN_SAVE_FILES
 
-    paneSz->Add(btnSizer, wxSizerFlags().Right().Border(wxTOP));
+    paneSz->Add(btnSizer, wxSizerFlags().Right().Border(wxTOP|wxBOTTOM));
 #endif // wxUSE_CLIPBOARD || CAN_SAVE_FILES
 
     win->SetSizer(paneSz);
     paneSz->SetSizeHints(win);
-#else // __SMARTPHONE__
-    SetLeftMenu(wxID_OK);
-    SetRightMenu(wxID_MORE, ms_details + EXPAND_SUFFIX);
-#endif // __SMARTPHONE__/!__SMARTPHONE__
 
     SetSizerAndFit(sizerTop);
 
@@ -811,11 +795,6 @@ void wxLogDialog::CreateDetailsControls(wxWindow *parent)
                                 wxLC_REPORT |
                                 wxLC_NO_HEADER |
                                 wxLC_SINGLE_SEL);
-#ifdef __WXWINCE__
-    // This makes a big aesthetic difference on WinCE but I
-    // don't want to risk problems on other platforms
-    m_listctrl->Hide();
-#endif
 
     // no need to translate these strings as they're not shown to the
     // user anyhow (we use wxLC_NO_HEADER style)
@@ -829,7 +808,7 @@ void wxLogDialog::CreateDetailsControls(wxWindow *parent)
     wxImageList *imageList = new wxImageList(ICON_SIZE, ICON_SIZE);
 
     // order should be the same as in the switch below!
-    static const wxChar* const icons[] =
+    static const char* const icons[] =
     {
         wxART_ERROR,
         wxART_WARNING,
@@ -845,7 +824,7 @@ void wxLogDialog::CreateDetailsControls(wxWindow *parent)
 
         // This may very well fail if there are insufficient colours available.
         // Degrade gracefully.
-        if ( !bmp.Ok() )
+        if ( !bmp.IsOk() )
         {
             loadedIcons = false;
 
