@@ -2,7 +2,6 @@
 // Name:        src/gtk/radiobox.cpp
 // Purpose:
 // Author:      Robert Roebling
-// Id:          $Id: radiobox.cpp,v 1.130 2006/12/07 07:16:14 PC Exp $
 // Copyright:   (c) 1998 Robert Roebling
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -14,20 +13,11 @@
 
 #include "wx/radiobox.h"
 
-#ifndef WX_PRECOMP
-    #include "wx/log.h"
-    #include "wx/frame.h"
-    #include "wx/dialog.h"
-#endif
-
 #if wxUSE_TOOLTIPS
     #include "wx/tooltip.h"
 #endif
 
 #include "wx/gtk/private.h"
-#include <gdk/gdkkeysyms.h>
-
-#include "wx/gtk/win_gtk.h"
 
 //-----------------------------------------------------------------------------
 // wxGTKRadioButtonInfo
@@ -49,7 +39,7 @@ public:
 //-----------------------------------------------------------------------------
 
 #include "wx/listimpl.cpp"
-WX_DEFINE_LIST( wxRadioBoxButtonsInfoList );
+WX_DEFINE_LIST( wxRadioBoxButtonsInfoList )
 
 extern bool          g_blockEventsOnDrag;
 
@@ -60,18 +50,15 @@ extern bool          g_blockEventsOnDrag;
 extern "C" {
 static void gtk_radiobutton_clicked_callback( GtkToggleButton *button, wxRadioBox *rb )
 {
-    if (g_isIdle) wxapp_install_idle_handler();
-
-    if (!rb->m_hasVMT) return;
     if (g_blockEventsOnDrag) return;
 
-    if (!button->active) return;
+    if (!gtk_toggle_button_get_active(button)) return;
 
-    wxCommandEvent event( wxEVT_COMMAND_RADIOBOX_SELECTED, rb->GetId() );
+    wxCommandEvent event( wxEVT_RADIOBOX, rb->GetId() );
     event.SetInt( rb->GetSelection() );
     event.SetString( rb->GetStringSelection() );
     event.SetEventObject( rb );
-    rb->GetEventHandler()->ProcessEvent(event);
+    rb->HandleWindowEvent(event);
 }
 }
 
@@ -82,29 +69,26 @@ static void gtk_radiobutton_clicked_callback( GtkToggleButton *button, wxRadioBo
 extern "C" {
 static gint gtk_radiobox_keypress_callback( GtkWidget *widget, GdkEventKey *gdk_event, wxRadioBox *rb )
 {
-    // don't need to install idle handler, its done from "event" signal
-
-    if (!rb->m_hasVMT) return FALSE;
     if (g_blockEventsOnDrag) return FALSE;
 
-    if ( ((gdk_event->keyval == GDK_Tab) ||
-          (gdk_event->keyval == GDK_ISO_Left_Tab)) &&
+    if ( ((gdk_event->keyval == GDK_KEY_Tab) ||
+          (gdk_event->keyval == GDK_KEY_ISO_Left_Tab)) &&
          rb->GetParent() && (rb->GetParent()->HasFlag( wxTAB_TRAVERSAL)) )
     {
         wxNavigationKeyEvent new_event;
         new_event.SetEventObject( rb->GetParent() );
         // GDK reports GDK_ISO_Left_Tab for SHIFT-TAB
-        new_event.SetDirection( (gdk_event->keyval == GDK_Tab) );
+        new_event.SetDirection( (gdk_event->keyval == GDK_KEY_Tab) );
         // CTRL-TAB changes the (parent) window, i.e. switch notebook page
-        new_event.SetWindowChange( (gdk_event->state & GDK_CONTROL_MASK) );
+        new_event.SetWindowChange( (gdk_event->state & GDK_CONTROL_MASK) != 0 );
         new_event.SetCurrentFocus( rb );
-        return rb->GetParent()->GetEventHandler()->ProcessEvent( new_event );
+        return rb->GetParent()->HandleWindowEvent(new_event);
     }
 
-    if ((gdk_event->keyval != GDK_Up) &&
-        (gdk_event->keyval != GDK_Down) &&
-        (gdk_event->keyval != GDK_Left) &&
-        (gdk_event->keyval != GDK_Right))
+    if ((gdk_event->keyval != GDK_KEY_Up) &&
+        (gdk_event->keyval != GDK_KEY_Down) &&
+        (gdk_event->keyval != GDK_KEY_Left) &&
+        (gdk_event->keyval != GDK_KEY_Right))
     {
         return FALSE;
     }
@@ -119,8 +103,8 @@ static gint gtk_radiobox_keypress_callback( GtkWidget *widget, GdkEventKey *gdk_
         return FALSE;
     }
 
-    if ((gdk_event->keyval == GDK_Up) ||
-        (gdk_event->keyval == GDK_Left))
+    if ((gdk_event->keyval == GDK_KEY_Up) ||
+        (gdk_event->keyval == GDK_KEY_Left))
     {
         if (node == rb->m_buttonsInfo.GetFirst())
             node = rb->m_buttonsInfo.GetLast();
@@ -144,46 +128,33 @@ static gint gtk_radiobox_keypress_callback( GtkWidget *widget, GdkEventKey *gdk_
 }
 
 extern "C" {
-static gint gtk_radiobutton_focus_in( GtkWidget *widget,
-                                      GdkEvent *WXUNUSED(event),
-                                      wxRadioBox *win )
+static gint gtk_radiobutton_focus_out( GtkWidget * WXUNUSED(widget),
+                                       GdkEventFocus *WXUNUSED(event),
+                                       wxRadioBox *win )
 {
-    if ( win->m_lostFocus )
-    {
-        // no, we didn't really lose it
-        win->m_lostFocus = FALSE;
-    }
-    else if ( !win->m_hasFocus )
-    {
-        win->m_hasFocus = true;
-
-        wxFocusEvent event( wxEVT_SET_FOCUS, win->GetId() );
-        event.SetEventObject( win );
-
-        // never stop the signal emission, it seems to break the kbd handling
-        // inside the radiobox
-        (void)win->GetEventHandler()->ProcessEvent( event );
-    }
+    // NB: This control is composed of several GtkRadioButton widgets and
+    //     when focus changes from one of them to another in the same
+    //     wxRadioBox, we get a focus-out event followed by focus-in for
+    //     another GtkRadioButton owned by the same control. We don't want
+    //     to generate two spurious wxEVT_SET_FOCUS events in this case,
+    //     so we defer sending wx events until idle time.
+    win->GTKHandleFocusOut();
 
+    // never stop the signal emission, it seems to break the kbd handling
+    // inside the radiobox
     return FALSE;
 }
 }
 
 extern "C" {
-static gint gtk_radiobutton_focus_out( GtkWidget *widget,
-                                       GdkEvent *WXUNUSED(event),
-                                       wxRadioBox *win )
+static gint gtk_radiobutton_focus_in( GtkWidget * WXUNUSED(widget),
+                                      GdkEventFocus *WXUNUSED(event),
+                                      wxRadioBox *win )
 {
-  //    wxASSERT_MSG( win->m_hasFocus, _T("got focus out without any focus in?") );
-  // Replace with a warning, else we dump core a lot!
-  //  if (!win->m_hasFocus)
-  //      wxLogWarning(_T("Radiobox got focus out without any focus in.") );
-
-    // we might have lost the focus, but may be not - it may have just gone to
-    // another button in the same radiobox, so we'll check for it in the next
-    // idle iteration (leave m_hasFocus == true for now)
-    win->m_lostFocus = true;
+    win->GTKHandleFocusIn();
 
+    // never stop the signal emission, it seems to break the kbd handling
+    // inside the radiobox
     return FALSE;
 }
 }
@@ -209,21 +180,23 @@ static void gtk_radiobutton_size_allocate( GtkWidget *widget,
 }
 }
 
+#ifndef __WXGTK3__
+extern "C" {
+static gboolean expose_event(GtkWidget* widget, GdkEventExpose*, wxWindow*)
+{
+    const GtkAllocation& a = widget->allocation;
+    gtk_paint_flat_box(gtk_widget_get_style(widget), gtk_widget_get_window(widget),
+        GTK_STATE_NORMAL, GTK_SHADOW_NONE, NULL, widget, "", a.x, a.y, a.width, a.height);
+    return false;
+}
+}
+#endif
 
 //-----------------------------------------------------------------------------
 // wxRadioBox
 //-----------------------------------------------------------------------------
 
-IMPLEMENT_DYNAMIC_CLASS(wxRadioBox,wxControl)
-
-void wxRadioBox::Init()
-{
-    m_needParent = true;
-    m_acceptsFocus = true;
-
-    m_hasFocus =
-    m_lostFocus = false;
-}
+wxIMPLEMENT_DYNAMIC_CLASS(wxRadioBox, wxControl);
 
 bool wxRadioBox::Create( wxWindow *parent, wxWindowID id,
                          const wxString& title,
@@ -252,7 +225,14 @@ bool wxRadioBox::Create( wxWindow *parent, wxWindowID id, const wxString& title,
     }
 
     m_widget = GTKCreateFrame(title);
+    g_object_ref(m_widget);
     wxControl::SetLabel(title);
+    if ( HasFlag(wxNO_BORDER) )
+    {
+        // If we don't do this here, the wxNO_BORDER style is ignored in Show()
+        gtk_frame_set_shadow_type(GTK_FRAME(m_widget), GTK_SHADOW_NONE);
+    }
+
 
     // majorDim may be 0 if all trailing parameters were omitted, so don't
     // assert here but just use the correct value for it
@@ -262,29 +242,72 @@ bool wxRadioBox::Create( wxWindow *parent, wxWindowID id, const wxString& title,
     unsigned int num_of_cols = GetColumnCount();
     unsigned int num_of_rows = GetRowCount();
 
-    GtkRadioButton *rbtn = (GtkRadioButton*) NULL;
+    GtkRadioButton *rbtn = NULL;
 
+#ifdef __WXGTK3__
+    GtkWidget* grid = gtk_grid_new();
+    gtk_widget_show(grid);
+    gtk_container_add(GTK_CONTAINER(m_widget), grid);
+#else
     GtkWidget *table = gtk_table_new( num_of_rows, num_of_cols, FALSE );
     gtk_table_set_col_spacings( GTK_TABLE(table), 1 );
     gtk_table_set_row_spacings( GTK_TABLE(table), 1 );
     gtk_widget_show( table );
     gtk_container_add( GTK_CONTAINER(m_widget), table );
+#endif
 
-    wxString label;
-    GSList *radio_button_group = (GSList *) NULL;
+    GSList *radio_button_group = NULL;
     for (unsigned int i = 0; i < (unsigned int)n; i++)
     {
         if ( i != 0 )
             radio_button_group = gtk_radio_button_get_group( GTK_RADIO_BUTTON(rbtn) );
 
-        label.Empty();
-        for ( const wxChar *pc = choices[i]; *pc; pc++ )
+        // Process mnemonic in the label
+        wxString label;
+        bool hasMnemonic = false;
+        for ( wxString::const_iterator pc = choices[i].begin();
+              pc != choices[i].end(); ++pc )
         {
-            if ( *pc != wxT('&') )
-                label += *pc;
+            if ( *pc == wxS('_') )
+            {
+                // If we have a literal underscore character in the label
+                // containing mnemonic, two underscores should be used.
+                if ( hasMnemonic )
+                    label += wxS('_');
+            }
+            else if ( *pc == wxS('&') )
+            {
+                ++pc; // skip it
+                 if ( pc == choices[i].end() )
+                 {
+                     break;
+                 }
+                 else if ( *pc != wxS('&') )
+                 {
+                     if ( !hasMnemonic )
+                     {
+                         hasMnemonic = true;
+                         // So far we assumed that label doesn't contain mnemonic
+                         // and therefore single underscore characters were not
+                         // replaced by two underscores. Now we have to double
+                         // all existing underscore characters.
+                         label.Replace(wxS("_"), wxS("__"));
+                         label += wxS('_');
+                     }
+                     else
+                     {
+                         wxFAIL_MSG(wxT("duplicate mnemonic char in radio button label"));
+                     }
+                 }
+            }
+
+            label += *pc;
         }
+        if ( hasMnemonic )
+            rbtn = GTK_RADIO_BUTTON( gtk_radio_button_new_with_mnemonic( radio_button_group, wxGTK_CONV( label ) ) );
+        else
+            rbtn = GTK_RADIO_BUTTON( gtk_radio_button_new_with_label( radio_button_group, wxGTK_CONV( label ) ) );
 
-        rbtn = GTK_RADIO_BUTTON( gtk_radio_button_new_with_label( radio_button_group, wxGTK_CONV( label ) ) );
         gtk_widget_show( GTK_WIDGET(rbtn) );
 
         g_signal_connect (rbtn, "key_press_event",
@@ -292,6 +315,20 @@ bool wxRadioBox::Create( wxWindow *parent, wxWindowID id, const wxString& title,
 
         m_buttonsInfo.Append( new wxGTKRadioButtonInfo( rbtn, wxRect() ) );
 
+#ifdef __WXGTK3__
+        int left, top;
+        if (HasFlag(wxRA_SPECIFY_COLS))
+        {
+            left = i % num_of_cols;
+            top = i / num_of_cols;
+        }
+        else
+        {
+            left = i / num_of_rows;
+            top = i % num_of_rows;
+        }
+        gtk_grid_attach(GTK_GRID(grid), GTK_WIDGET(rbtn), left, top, 1, 1);
+#else
         if (HasFlag(wxRA_SPECIFY_COLS))
         {
             int left = i%num_of_cols;
@@ -310,6 +347,7 @@ bool wxRadioBox::Create( wxWindow *parent, wxWindowID id, const wxString& title,
             gtk_table_attach( GTK_TABLE(table), GTK_WIDGET(rbtn), left, right, top, bottom,
                   GTK_FILL, GTK_FILL, 1, 1 );
         }
+#endif
 
         ConnectWidget( GTK_WIDGET(rbtn) );
 
@@ -339,6 +377,7 @@ wxRadioBox::~wxRadioBox()
     while (node)
     {
         GtkWidget *button = GTK_WIDGET( node->GetData()->button );
+        GTKDisconnect(button);
         gtk_widget_destroy( button );
         node = node->GetNext();
     }
@@ -374,25 +413,6 @@ bool wxRadioBox::Show( bool show )
     return true;
 }
 
-void wxRadioBox::SetFocus()
-{
-    wxCHECK_RET( m_widget != NULL, wxT("invalid radiobox") );
-
-    if (m_buttonsInfo.GetCount() == 0) return;
-
-    wxRadioBoxButtonsInfoList::compatibility_iterator node = m_buttonsInfo.GetFirst();
-    while (node)
-    {
-        GtkToggleButton *button = GTK_TOGGLE_BUTTON( node->GetData()->button );
-        if (button->active)
-        {
-            gtk_widget_grab_focus( GTK_WIDGET(button) );
-            return;
-        }
-        node = node->GetNext();
-    }
-}
-
 void wxRadioBox::SetSelection( int n )
 {
     wxCHECK_RET( m_widget != NULL, wxT("invalid radiobox") );
@@ -420,7 +440,7 @@ int wxRadioBox::GetSelection(void) const
     while (node)
     {
         GtkToggleButton *button = GTK_TOGGLE_BUTTON( node->GetData()->button );
-        if (button->active) return count;
+        if (gtk_toggle_button_get_active(button)) return count;
         count++;
         node = node->GetNext();
     }
@@ -438,7 +458,7 @@ wxString wxRadioBox::GetString(unsigned int n) const
 
     wxCHECK_MSG( node, wxEmptyString, wxT("radiobox wrong index") );
 
-    GtkLabel *label = GTK_LABEL(GTK_BIN(node->GetData()->button)->child);
+    GtkLabel* label = GTK_LABEL(gtk_bin_get_child(GTK_BIN(node->GetData()->button)));
 
     wxString str( wxGTK_CONV_BACK( gtk_label_get_text(label) ) );
 
@@ -460,7 +480,7 @@ void wxRadioBox::SetString(unsigned int item, const wxString& label)
 
     wxCHECK_RET( node, wxT("radiobox wrong index") );
 
-    GtkLabel *g_label = GTK_LABEL(GTK_BIN(node->GetData()->button)->child);
+    GtkLabel* g_label = GTK_LABEL(gtk_bin_get_child(GTK_BIN(node->GetData()->button)));
 
     gtk_label_set_text( g_label, wxGTK_CONV( label ) );
 }
@@ -474,13 +494,16 @@ bool wxRadioBox::Enable( bool enable )
     while (node)
     {
         GtkButton *button = GTK_BUTTON( node->GetData()->button );
-        GtkLabel *label = GTK_LABEL(GTK_BIN(button)->child);
+        GtkLabel *label = GTK_LABEL(gtk_bin_get_child(GTK_BIN(button)));
 
         gtk_widget_set_sensitive( GTK_WIDGET(button), enable );
         gtk_widget_set_sensitive( GTK_WIDGET(label), enable );
         node = node->GetNext();
     }
 
+    if (enable)
+        GTKFixSensitivity();
+
     return true;
 }
 
@@ -493,7 +516,7 @@ bool wxRadioBox::Enable(unsigned int item, bool enable)
     wxCHECK_MSG( node, false, wxT("radiobox wrong index") );
 
     GtkButton *button = GTK_BUTTON( node->GetData()->button );
-    GtkLabel *label = GTK_LABEL(GTK_BIN(button)->child);
+    GtkLabel *label = GTK_LABEL(gtk_bin_get_child(GTK_BIN(button)));
 
     gtk_widget_set_sensitive( GTK_WIDGET(button), enable );
     gtk_widget_set_sensitive( GTK_WIDGET(label), enable );
@@ -513,7 +536,7 @@ bool wxRadioBox::IsItemEnabled(unsigned int item) const
 
     // don't use GTK_WIDGET_IS_SENSITIVE() here, we want to return true even if
     // the parent radiobox is disabled
-    return GTK_WIDGET_SENSITIVE(GTK_WIDGET(button));
+    return gtk_widget_get_sensitive(GTK_WIDGET(button)) != 0;
 }
 
 bool wxRadioBox::Show(unsigned int item, bool show)
@@ -544,7 +567,7 @@ bool wxRadioBox::IsItemShown(unsigned int item) const
 
     GtkButton *button = GTK_BUTTON( node->GetData()->button );
 
-    return GTK_WIDGET_VISIBLE(GTK_WIDGET(button));
+    return gtk_widget_get_visible(GTK_WIDGET(button)) != 0;
 }
 
 unsigned int wxRadioBox::GetCount() const
@@ -557,9 +580,8 @@ void wxRadioBox::GtkDisableEvents()
     wxRadioBoxButtonsInfoList::compatibility_iterator node = m_buttonsInfo.GetFirst();
     while (node)
     {
-        g_signal_handlers_disconnect_by_func (node->GetData()->button,
-                                              (gpointer) gtk_radiobutton_clicked_callback,
-                                              this);
+        g_signal_handlers_block_by_func(node->GetData()->button,
+            (gpointer)gtk_radiobutton_clicked_callback, this);
 
         node = node->GetNext();
     }
@@ -570,8 +592,8 @@ void wxRadioBox::GtkEnableEvents()
     wxRadioBoxButtonsInfoList::compatibility_iterator node = m_buttonsInfo.GetFirst();
     while (node)
     {
-        g_signal_connect (node->GetData()->button, "clicked",
-                          G_CALLBACK (gtk_radiobutton_clicked_callback), this);
+        g_signal_handlers_unblock_by_func(node->GetData()->button,
+            (gpointer)gtk_radiobutton_clicked_callback, this);
 
         node = node->GetNext();
     }
@@ -586,11 +608,17 @@ void wxRadioBox::DoApplyWidgetStyle(GtkRcStyle *style)
     {
         GtkWidget *widget = GTK_WIDGET( node->GetData()->button );
 
-        gtk_widget_modify_style( widget, style );
-        gtk_widget_modify_style(GTK_BIN(widget)->child, style);
+        GTKApplyStyle(widget, style);
+        GTKApplyStyle(gtk_bin_get_child(GTK_BIN(widget)), style);
 
         node = node->GetNext();
     }
+
+#ifndef __WXGTK3__
+    g_signal_handlers_disconnect_by_func(m_widget, (void*)expose_event, this);
+    if (m_backgroundColour.IsOk())
+        g_signal_connect(m_widget, "expose-event", G_CALLBACK(expose_event), this);
+#endif
 }
 
 bool wxRadioBox::GTKWidgetNeedsMnemonic() const
@@ -604,7 +632,7 @@ void wxRadioBox::GTKWidgetDoSetMnemonic(GtkWidget* w)
 }
 
 #if wxUSE_TOOLTIPS
-void wxRadioBox::ApplyToolTip(GtkTooltips * WXUNUSED(tips), const wxChar *tip)
+void wxRadioBox::GTKApplyToolTip(const char* tip)
 {
     // set this tooltip for all radiobuttons which don't have their own tips
     unsigned n = 0;
@@ -614,8 +642,7 @@ void wxRadioBox::ApplyToolTip(GtkTooltips * WXUNUSED(tips), const wxChar *tip)
     {
         if ( !GetItemToolTip(n) )
         {
-            wxToolTip::Apply(GTK_WIDGET(node->GetData()->button),
-                             wxConvCurrent->cWX2MB(tip));
+            wxToolTip::GTKApply(GTK_WIDGET(node->GetData()->button), tip);
         }
     }
 }
@@ -628,21 +655,23 @@ void wxRadioBox::DoSetItemToolTip(unsigned int n, wxToolTip *tooltip)
     if ( tooltip )
         buf = wxGTK_CONV(tooltip->GetTip());
 
-    wxToolTip::Apply(GTK_WIDGET(m_buttonsInfo[n]->button), buf);
+    wxToolTip::GTKApply(GTK_WIDGET(m_buttonsInfo[n]->button), buf);
 }
 
 #endif // wxUSE_TOOLTIPS
 
 GdkWindow *wxRadioBox::GTKGetWindow(wxArrayGdkWindows& windows) const
 {
-    windows.push_back(m_widget->window);
+    windows.push_back(gtk_widget_get_window(m_widget));
 
     wxRadioBoxButtonsInfoList::compatibility_iterator node = m_buttonsInfo.GetFirst();
     while (node)
     {
         GtkWidget *button = GTK_WIDGET( node->GetData()->button );
 
-        windows.push_back(button->window);
+        // don't put NULL pointers in the 'windows' array!
+        if (gtk_widget_get_window(button))
+            windows.push_back(gtk_widget_get_window(button));
 
         node = node->GetNext();
     }
@@ -650,34 +679,11 @@ GdkWindow *wxRadioBox::GTKGetWindow(wxArrayGdkWindows& windows) const
     return NULL;
 }
 
-void wxRadioBox::OnInternalIdle()
-{
-    wxControl::OnInternalIdle();
-
-    if ( m_lostFocus )
-    {
-        m_hasFocus = false;
-        m_lostFocus = false;
-
-        wxFocusEvent event( wxEVT_KILL_FOCUS, GetId() );
-        event.SetEventObject( this );
-
-        (void)GetEventHandler()->ProcessEvent( event );
-    }
-}
-
 // static
 wxVisualAttributes
 wxRadioBox::GetClassDefaultAttributes(wxWindowVariant WXUNUSED(variant))
 {
-    wxVisualAttributes attr;
-    // NB: we need toplevel window so that GTK+ can find the right style
-    GtkWidget *wnd = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-    GtkWidget* widget = gtk_radio_button_new_with_label(NULL, "");
-    gtk_container_add(GTK_CONTAINER(wnd), widget);
-    attr = GetDefaultAttributesFromGTKWidget(widget);
-    gtk_widget_destroy(wnd);
-    return attr;
+    return GetDefaultAttributesFromGTKWidget(gtk_radio_button_new_with_label(NULL, ""));
 }
 
 int wxRadioBox::GetItemFromPoint(const wxPoint& point) const
