@@ -29,9 +29,8 @@
 #include <sys/stat.h>
 #include <ctype.h>
 
-#include <gtk/gtk.h>
 #include "wx/gtk/private.h"
-#include "wx/gtk/private/gtk2-compat.h"
+#include "wx/gtk/private/gtk3-compat.h"
 
 // ----------------------------------------------------------------------------
 // helpers
@@ -181,12 +180,12 @@ static void wxGtkTextApplyTagsFromAttr(GtkWidget *text,
 #elif GTK_CHECK_VERSION(2,11,0)
 // gtk+ doesn't support justify before gtk+-2.11.0 with pango-1.17 being available
 // (but if new enough pango isn't available it's a mere gtk warning)
-                if (!gtk_check_version(2,11,0))
+                if (wx_is_at_least_gtk2(11))
                 {
                     align = GTK_JUSTIFY_FILL;
                     break;
                 }
-                // fallthrough
+                wxFALLTHROUGH;
 #endif
             default:
                 align = GTK_JUSTIFY_LEFT;
@@ -216,9 +215,19 @@ static void wxGtkTextApplyTagsFromAttr(GtkWidget *text,
         wxGtkTextRemoveTagsWithPrefix(text_buffer, "WXINDENT", &para_start, &para_end);
 
         // Convert indent from 1/10th of a mm into pixels
+#ifdef __WXGTK4__
+        GdkMonitor* monitor = gdk_display_get_monitor_at_window(
+            gtk_widget_get_display(text), gtk_widget_get_window(text));
+        GdkRectangle rect;
+        gdk_monitor_get_geometry(monitor, &rect);
+        float factor = float(rect.width) / gdk_monitor_get_width_mm(monitor);
+#else
+        wxGCC_WARNING_SUPPRESS(deprecated-declarations)
         float factor =
             (float)gdk_screen_get_width(gtk_widget_get_screen(text)) /
                       gdk_screen_get_width_mm(gtk_widget_get_screen(text)) / 10;
+        wxGCC_WARNING_RESTORE()
+#endif
 
         const int indent = (int)(factor * attr.GetLeftIndent());
         const int subIndent = (int)(factor * attr.GetLeftSubIndent());
@@ -273,10 +282,19 @@ static void wxGtkTextApplyTagsFromAttr(GtkWidget *text,
         if (!tag)
         {
             // Factor to convert from 1/10th of a mm into pixels
+#ifdef __WXGTK4__
+            GdkMonitor* monitor = gdk_display_get_monitor_at_window(
+                gtk_widget_get_display(text), gtk_widget_get_window(text));
+            GdkRectangle rect;
+            gdk_monitor_get_geometry(monitor, &rect);
+            float factor = float(rect.width) / gdk_monitor_get_width_mm(monitor);
+#else
+            wxGCC_WARNING_SUPPRESS(deprecated-declarations)
             float factor =
                 (float)gdk_screen_get_width(gtk_widget_get_screen(text)) /
                           gdk_screen_get_width_mm(gtk_widget_get_screen(text)) / 10;
-
+            wxGCC_WARNING_RESTORE()
+#endif
             PangoTabArray* tabArray = pango_tab_array_new(tabs.GetCount(), TRUE);
             for (size_t i = 0; i < tabs.GetCount(); i++)
                 pango_tab_array_set_tab(tabArray, i, PANGO_TAB_LEFT, (gint)(tabs[i] * factor));
@@ -288,25 +306,6 @@ static void wxGtkTextApplyTagsFromAttr(GtkWidget *text,
     }
 }
 
-static void wxGtkTextInsert(GtkWidget *text,
-                            GtkTextBuffer *text_buffer,
-                            const wxTextAttr& attr,
-                            const wxCharBuffer& buffer)
-
-{
-    gint start_offset;
-    GtkTextIter iter, start;
-
-    gtk_text_buffer_get_iter_at_mark( text_buffer, &iter,
-                                     gtk_text_buffer_get_insert (text_buffer) );
-    start_offset = gtk_text_iter_get_offset (&iter);
-    gtk_text_buffer_insert( text_buffer, &iter, buffer, strlen(buffer) );
-
-    gtk_text_buffer_get_iter_at_offset (text_buffer, &start, start_offset);
-
-    wxGtkTextApplyTagsFromAttr(text, text_buffer, attr, &start, &iter);
-}
-
 // Implementation of wxTE_AUTO_URL for wxGTK2 by Mart Raudsepp,
 
 extern "C" {
@@ -322,6 +321,14 @@ au_apply_tag_callback(GtkTextBuffer *buffer,
 }
 }
 
+// Check if the style contains wxTE_PROCESS_TAB and update the given
+// GtkTextView accordingly.
+static void wxGtkSetAcceptsTab(GtkWidget* text, long style)
+{
+    gtk_text_view_set_accepts_tab(GTK_TEXT_VIEW(text),
+                                  (style & wxTE_PROCESS_TAB) != 0);
+}
+
 //-----------------------------------------------------------------------------
 //  GtkTextCharPredicates for gtk_text_iter_*_find_char
 //-----------------------------------------------------------------------------
@@ -463,7 +470,7 @@ extern "C" {
 
 // Normal version used for detecting IME input and generating appropriate
 // events for it.
-void
+static void
 wx_insert_text_callback(GtkTextBuffer* buffer,
                         GtkTextIter* WXUNUSED(end),
                         gchar *text,
@@ -480,18 +487,24 @@ wx_insert_text_callback(GtkTextBuffer* buffer,
 
 // And an "after" version used for detecting URLs in the text.
 static void
-au_insert_text_callback(GtkTextBuffer * WXUNUSED(buffer),
+au_insert_text_callback(GtkTextBuffer *buffer,
                         GtkTextIter *end,
                         gchar *text,
                         gint len,
                         wxTextCtrl *win)
 {
-    if (!len || !(win->GetWindowStyleFlag() & wxTE_AUTO_URL) )
-        return;
-
     GtkTextIter start = *end;
     gtk_text_iter_backward_chars(&start, g_utf8_strlen(text, len));
 
+    if ( !win->GetDefaultStyle().IsDefault() )
+    {
+        wxGtkTextApplyTagsFromAttr(win->GetHandle(), buffer, win->GetDefaultStyle(),
+                                   &start, end);
+    }
+
+    if ( !len || !(win->GetWindowStyleFlag() & wxTE_AUTO_URL) )
+        return;
+
     GtkTextIter line_start = start;
     GtkTextIter line_end = *end;
     GtkTextIter words_start = start;
@@ -600,7 +613,7 @@ static void state_flags_changed(GtkWidget*, GtkStateFlags, wxTextCtrl* win)
 //  wxTextCtrl
 //-----------------------------------------------------------------------------
 
-BEGIN_EVENT_TABLE(wxTextCtrl, wxTextCtrlBase)
+wxBEGIN_EVENT_TABLE(wxTextCtrl, wxTextCtrlBase)
     EVT_CHAR(wxTextCtrl::OnChar)
 
     EVT_MENU(wxID_CUT, wxTextCtrl::OnCut)
@@ -624,7 +637,7 @@ BEGIN_EVENT_TABLE(wxTextCtrl, wxTextCtrlBase)
     EVT_RIGHT_DOWN  (wxTextCtrl::OnUrlMouseEvent)
     EVT_RIGHT_UP    (wxTextCtrl::OnUrlMouseEvent)
     EVT_RIGHT_DCLICK(wxTextCtrl::OnUrlMouseEvent)
-END_EVENT_TABLE()
+wxEND_EVENT_TABLE()
 
 void wxTextCtrl::Init()
 {
@@ -737,11 +750,13 @@ bool wxTextCtrl::Create( wxWindow *parent,
         gtk_entry_set_width_chars((GtkEntry*)m_text, 1);
 
         // work around probable bug in GTK+ 2.18 when calling WriteText on a
-        // new, empty control, see http://trac.wxwidgets.org/ticket/11409
+        // new, empty control, see https://trac.wxwidgets.org/ticket/11409
         gtk_entry_get_text((GtkEntry*)m_text);
 
+#ifndef __WXGTK3__
         if (style & wxNO_BORDER)
-            g_object_set (m_text, "has-frame", FALSE, NULL);
+            gtk_entry_set_has_frame((GtkEntry*)m_text, FALSE);
+#endif
 
     }
     g_object_ref(m_widget);
@@ -791,6 +806,8 @@ bool wxTextCtrl::Create( wxWindow *parent,
 
     if (multi_line)
     {
+        wxGtkSetAcceptsTab(m_text, style);
+
         // Handle URLs on multi-line controls with wxTE_AUTO_URL style
         if (style & wxTE_AUTO_URL)
         {
@@ -804,9 +821,6 @@ bool wxTextCtrl::Create( wxWindow *parent,
                                        "underline", PANGO_UNDERLINE_SINGLE,
                                        NULL);
 
-            // Check for URLs after each text change
-            g_signal_connect_after (m_buffer, "insert_text",
-                                    G_CALLBACK (au_insert_text_callback), this);
             g_signal_connect_after (m_buffer, "delete_range",
                                     G_CALLBACK (au_delete_range_callback), this);
 
@@ -829,6 +843,10 @@ bool wxTextCtrl::Create( wxWindow *parent,
         // the IME-generated input.
         g_signal_connect(m_buffer, "insert_text",
                          G_CALLBACK(wx_insert_text_callback), this);
+
+        // Needed for wxTE_AUTO_URL and applying custom styles
+        g_signal_connect_after(m_buffer, "insert_text",
+                               G_CALLBACK(au_insert_text_callback), this);
     }
     else // single line
     {
@@ -857,7 +875,10 @@ GtkEditable *wxTextCtrl::GetEditable() const
 
 GtkEntry *wxTextCtrl::GetEntry() const
 {
-    return GTK_ENTRY(m_text);
+    if (GTK_IS_ENTRY(m_text))
+        return (GtkEntry*)m_text;
+
+    return NULL;
 }
 
 int wxTextCtrl::GTKIMFilterKeypress(GdkEventKey* event) const
@@ -865,18 +886,14 @@ int wxTextCtrl::GTKIMFilterKeypress(GdkEventKey* event) const
     if (IsSingleLine())
         return wxTextEntry::GTKIMFilterKeypress(event);
 
-    int result;
+    int result = false;
 #if GTK_CHECK_VERSION(2, 22, 0)
-#ifndef __WXGTK3__
-    result = false;
-    if (gtk_check_version(2,22,0) == NULL)
-#endif
+    if (wx_is_at_least_gtk2(22))
     {
         result = gtk_text_view_im_context_filter_keypress(GTK_TEXT_VIEW(m_text), event);
     }
 #else // GTK+ < 2.22
     wxUnusedVar(event);
-    result = false;
 #endif // GTK+ 2.22+
 
     return result;
@@ -975,6 +992,15 @@ void wxTextCtrl::SetWindowStyleFlag(long style)
     if ( (style & wxTE_PROCESS_ENTER) != (styleOld & wxTE_PROCESS_ENTER) )
         GTKSetActivatesDefault();
 
+    if ( IsMultiLine() )
+    {
+        wxGtkSetAcceptsTab(m_text, style);
+    }
+    //else: there doesn't seem to be any way to do it for entries and while we
+    //      could emulate wxTE_PROCESS_TAB for them by handling Tab key events
+    //      explicitly, it doesn't seem to be worth doing it, this style is
+    //      pretty useless with single-line controls.
+
     static const long flagsWrap = wxTE_WORDWRAP | wxTE_CHARWRAP | wxTE_DONTWRAP;
     if ( (style & flagsWrap) != (styleOld & flagsWrap) )
         GTKSetWrapMode();
@@ -992,6 +1018,11 @@ wxString wxTextCtrl::GetValue() const
 {
     wxCHECK_MSG( m_text != NULL, wxEmptyString, wxT("invalid text ctrl") );
 
+    return wxTextEntry::GetValue();
+}
+
+wxString wxTextCtrl::DoGetValue() const
+{
     if ( IsMultiLine() )
     {
         GtkTextIter start;
@@ -1004,7 +1035,7 @@ wxString wxTextCtrl::GetValue() const
     }
     else // single line
     {
-        return wxTextEntry::GetValue();
+        return wxTextEntry::DoGetValue();
     }
 }
 
@@ -1034,79 +1065,28 @@ bool wxTextCtrl::IsEmpty() const
     return wxTextEntry::IsEmpty();
 }
 
-void wxTextCtrl::DoSetValue( const wxString &value, int flags )
-{
-    wxCHECK_RET( m_text != NULL, wxT("invalid text ctrl") );
-
-    m_modified = false;
-
-    if ( !IsMultiLine() )
-    {
-        wxTextEntry::DoSetValue(value, flags);
-        return;
-    }
-
-    if (value.IsEmpty())
-    {
-        if ( !(flags & SetValue_SendEvent) )
-            EnableTextChangedEvents(false);
-
-        gtk_text_buffer_set_text( m_buffer, "", 0 );
-
-        if ( !(flags & SetValue_SendEvent) )
-            EnableTextChangedEvents(true);
-
-        return;
-    }
-
-#if wxUSE_UNICODE
-    const wxCharBuffer buffer(value.utf8_str());
-#else
-    wxFontEncoding enc = m_defaultStyle.HasFont()
-                            ? m_defaultStyle.GetFont().GetEncoding()
-                            : wxFONTENCODING_SYSTEM;
-    if ( enc == wxFONTENCODING_SYSTEM )
-        enc = GetTextEncoding();
-
-    const wxCharBuffer buffer(wxGTK_CONV_ENC(value, enc));
-    if ( !buffer )
-    {
-        // see comment in WriteText() as to why we must warn the user about
-        // this
-        wxLogWarning(_("Failed to set text in the text control."));
-        return;
-    }
-#endif
-
-    if ( !(flags & SetValue_SendEvent) )
-    {
-        EnableTextChangedEvents(false);
-    }
-
-    gtk_text_buffer_set_text( m_buffer, buffer, strlen(buffer) );
-
-    if ( !m_defaultStyle.IsDefault() )
-    {
-        GtkTextIter start, end;
-        gtk_text_buffer_get_bounds( m_buffer, &start, &end );
-        wxGtkTextApplyTagsFromAttr(m_widget, m_buffer, m_defaultStyle,
-                                   &start, &end);
-    }
-
-    if ( !(flags & SetValue_SendEvent) )
-    {
-        EnableTextChangedEvents(true);
-    }
-}
-
 void wxTextCtrl::WriteText( const wxString &text )
 {
     wxCHECK_RET( m_text != NULL, wxT("invalid text ctrl") );
 
+    if ( text.empty() )
+    {
+        // We don't need to actually do anything, but we still need to generate
+        // an event expected from this call.
+        SendTextUpdatedEvent(this);
+        return;
+    }
+
     // we're changing the text programmatically
     DontMarkDirtyOnNextChange();
+    // make sure marking is re-enabled even if events are suppressed
+    wxON_BLOCK_EXIT_SET(m_dontMarkDirty, false);
 
-    // avoid generating wxEVT_CHAR when called from wxEVT_CHAR handler
+    // Inserting new text into the control below will emit insert-text signal
+    // which assumes that if m_imKeyEvent is set, it is called in response to
+    // this key press -- which is not the case here (but m_imKeyEvent might
+    // still be set e.g. because we're called from a menu event handler
+    // triggered by a keyboard accelerator), so reset m_imKeyEvent temporarily.
     GdkEventKey* const imKeyEvent_save = m_imKeyEvent;
     m_imKeyEvent = NULL;
     wxON_BLOCK_EXIT_SET(m_imKeyEvent, imKeyEvent_save);
@@ -1118,7 +1098,7 @@ void wxTextCtrl::WriteText( const wxString &text )
     }
 
 #if wxUSE_UNICODE
-    const wxCharBuffer buffer(text.utf8_str());
+    const wxScopedCharBuffer buffer(text.utf8_str());
 #else
     // check if we have a specific style for the current position
     wxFontEncoding enc = wxFONTENCODING_SYSTEM;
@@ -1131,7 +1111,7 @@ void wxTextCtrl::WriteText( const wxString &text )
     if ( enc == wxFONTENCODING_SYSTEM )
         enc = GetTextEncoding();
 
-    const wxCharBuffer buffer(wxGTK_CONV_ENC(text, enc));
+    const wxScopedCharBuffer buffer(wxGTK_CONV_ENC(text, enc));
     if ( !buffer )
     {
         // we must log an error here as losing the text like this can be a
@@ -1146,7 +1126,11 @@ void wxTextCtrl::WriteText( const wxString &text )
     gtk_text_buffer_delete_selection(m_buffer, false, true);
 
     // Insert the text
-    wxGtkTextInsert( m_text, m_buffer, m_defaultStyle, buffer );
+    GtkTextIter iter;
+    gtk_text_buffer_get_iter_at_mark( m_buffer, &iter,
+                                      gtk_text_buffer_get_insert (m_buffer) );
+
+    gtk_text_buffer_insert( m_buffer, &iter, buffer, buffer.length() );
 
     // Scroll to cursor, but only if scrollbar thumb is at the very bottom
     // won't work when frozen, text view is not using m_buffer then
@@ -1213,7 +1197,7 @@ bool wxTextCtrl::PositionToXY(long pos, long *x, long *y ) const
     }
     else // single line control
     {
-        if (pos <= gtk_entry_get_text_length(GTK_ENTRY(m_text)))
+        if (pos <= GTKGetEntryTextLength(GTK_ENTRY(m_text)))
         {
             if ( y )
                 *y = 0;
@@ -1233,15 +1217,36 @@ bool wxTextCtrl::PositionToXY(long pos, long *x, long *y ) const
 long wxTextCtrl::XYToPosition(long x, long y ) const
 {
     if ( IsSingleLine() )
-        return 0;
+    {
+        if ( y != 0 || x > GTKGetEntryTextLength(GTK_ENTRY(m_text)) )
+            return -1;
+
+        return x;
+    }
+
+    const gint numLines = gtk_text_buffer_get_line_count (m_buffer);
 
     GtkTextIter iter;
-    if (y >= gtk_text_buffer_get_line_count (m_buffer))
+    if (y >= numLines)
         return -1;
 
     gtk_text_buffer_get_iter_at_line(m_buffer, &iter, y);
-    if (x >= gtk_text_iter_get_chars_in_line (&iter))
+
+    const gint lineLength = gtk_text_iter_get_chars_in_line (&iter);
+    if (x > lineLength)
+    {
+        // This coordinate is always invalid.
         return -1;
+    }
+
+    if (x == lineLength)
+    {
+        // In this case the coordinate is considered to be valid by wx if this
+        // is the last line, as it corresponds to the last position beyond the
+        // last character of the text, and invalid otherwise.
+        if (y != numLines - 1)
+            return -1;
+    }
 
     return gtk_text_iter_get_offset(&iter) + x;
 }
@@ -1448,6 +1453,11 @@ void wxTextCtrl::ShowPosition( long pos )
         else
             gtk_text_view_scroll_mark_onscreen(GTK_TEXT_VIEW(m_text), mark);
     }
+    else // single line
+    {   // This function not only shows character at required position
+        // but also places the cursor at this position.
+        gtk_editable_set_position(GTK_EDITABLE(m_text), pos);
+    }
 }
 
 wxTextCtrlHitTestResult
@@ -1455,8 +1465,53 @@ wxTextCtrl::HitTest(const wxPoint& pt, long *pos) const
 {
     if ( !IsMultiLine() )
     {
-        // not supported
-        return wxTE_HT_UNKNOWN;
+        // These variables will contain the position inside PangoLayout.
+        int x = pt.x,
+            y = pt.y;
+
+        // Get the offsets of PangoLayout inside the control.
+        //
+        // Note that contrary to what GTK+ documentation implies, the
+        // horizontal offset already accounts for scrolling, i.e. it will be
+        // negative if text is scrolled.
+        gint ofsX = 0,
+             ofsY = 0;
+        gtk_entry_get_layout_offsets(GTK_ENTRY(m_text), &ofsX, &ofsY);
+
+        x -= ofsX;
+        y -= ofsY;
+
+        // And scale the coordinates for Pango.
+        x *= PANGO_SCALE;
+        y *= PANGO_SCALE;
+
+        PangoLayout* const layout = gtk_entry_get_layout(GTK_ENTRY(m_text));
+
+        int idx = -1,
+            ofs = 0;
+        if ( !pango_layout_xy_to_index(layout, x, y, &idx, &ofs) )
+        {
+            // Try to guess why did it fail.
+            if ( x < 0 || y < 0 )
+            {
+                if ( pos )
+                    *pos = 0;
+
+                return wxTE_HT_BEFORE;
+            }
+            else
+            {
+                if ( pos )
+                    *pos = wxTextEntry::GetLastPosition();
+
+                return wxTE_HT_BEYOND;
+            }
+        }
+
+        if ( pos )
+            *pos = idx;
+
+        return wxTE_HT_ON_TEXT;
     }
 
     int x, y;
@@ -1661,8 +1716,7 @@ GdkWindow *wxTextCtrl::GTKGetWindow(wxArrayGdkWindows& WXUNUSED(windows)) const
     else
     {
 #ifdef __WXGTK3__
-        GdkWindow* wxGTKFindWindow(GtkWidget* widget);
-        return wxGTKFindWindow(m_text);
+        return GTKFindWindow(m_text);
 #else
         return gtk_entry_get_text_window(GTK_ENTRY(m_text));
 #endif
@@ -1794,8 +1848,13 @@ bool wxTextCtrl::GetStyle(long position, wxTextAttr& style)
     }
     else // have custom attributes
     {
+#ifdef __WXGTK3__
+        style.SetBackgroundColour(*pattr->appearance.rgba[0]);
+        style.SetTextColour(*pattr->appearance.rgba[1]);
+#else
         style.SetBackgroundColour(pattr->appearance.bg_color);
         style.SetTextColour(pattr->appearance.fg_color);
+#endif
 
         const wxGtkString
             pangoFontString(pango_font_description_to_string(pattr->font));
@@ -1804,6 +1863,11 @@ bool wxTextCtrl::GetStyle(long position, wxTextAttr& style)
         if ( font.SetNativeFontInfo(wxString(pangoFontString)) )
             style.SetFont(font);
 
+        if ( pattr->appearance.underline != PANGO_UNDERLINE_NONE )
+            style.SetFontUnderlined(true);
+        if ( pattr->appearance.strikethrough )
+            style.SetFontStrikethrough(true);
+
         // TODO: set alignment, tabs and indents
     }
 
@@ -1812,37 +1876,6 @@ bool wxTextCtrl::GetStyle(long position, wxTextAttr& style)
 
 void wxTextCtrl::DoApplyWidgetStyle(GtkRcStyle *style)
 {
-#ifdef __WXGTK3__
-    // Preserve selection colors, otherwise the GTK_STATE_FLAG_NORMAL override
-    // will be used, and the selection is invisible
-    const GtkStateFlags selectedFocused =
-        GtkStateFlags(GTK_STATE_FLAG_SELECTED | GTK_STATE_FLAG_FOCUSED);
-    // remove any previous override
-    gtk_widget_override_color(m_text, GTK_STATE_FLAG_NORMAL, NULL);
-    gtk_widget_override_color(m_text, selectedFocused, NULL);
-    gtk_widget_override_background_color(m_text, GTK_STATE_FLAG_NORMAL, NULL);
-    gtk_widget_override_background_color(m_text, selectedFocused, NULL);
-    const bool fg_ok = m_foregroundColour.IsOk();
-    const bool bg_ok = m_backgroundColour.IsOk();
-    if (fg_ok || bg_ok)
-    {
-        GdkRGBA fg_orig, bg_orig;
-        GtkStyleContext* context = gtk_widget_get_style_context(m_text);
-        gtk_style_context_save(context);
-        if (IsMultiLine())
-            gtk_style_context_add_class(context, GTK_STYLE_CLASS_VIEW);
-        gtk_style_context_set_state(context, selectedFocused);
-        gtk_style_context_get_color(context, selectedFocused, &fg_orig);
-        gtk_style_context_get_background_color(context, selectedFocused, &bg_orig);
-        gtk_style_context_restore(context);
-
-        if (fg_ok)
-            gtk_widget_override_color(m_text, selectedFocused, &fg_orig);
-        if (bg_ok)
-            gtk_widget_override_background_color(m_text, selectedFocused, &bg_orig);
-    }
-#endif // __WXGTK3__
-
     GTKApplyStyle(m_text, style);
 }
 
@@ -2055,7 +2088,7 @@ void wxTextCtrl::OnUrlMouseEvent(wxMouseEvent& event)
     SetCursor(wxCursor(wxCURSOR_HAND));
 
     start = end;
-    if(!gtk_text_iter_begins_tag(&start, tag))
+    if (!gtk_text_iter_starts_tag(&start, tag))
         gtk_text_iter_backward_to_tag_toggle(&start, tag);
     if(!gtk_text_iter_ends_tag(&end, tag))
         gtk_text_iter_forward_to_tag_toggle(&end, tag);
