@@ -2,7 +2,6 @@
 // Name:        src/gtk1/notebook.cpp
 // Purpose:
 // Author:      Robert Roebling
-// Id:          $Id: notebook.cpp,v 1.136 2006/10/09 17:14:31 VZ Exp $
 // Copyright:   (c) 1998 Robert Roebling, Vadim Zeitlin
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -31,13 +30,6 @@
 
 #include <gdk/gdkkeysyms.h>
 
-// ----------------------------------------------------------------------------
-// events
-// ----------------------------------------------------------------------------
-
-DEFINE_EVENT_TYPE(wxEVT_COMMAND_NOTEBOOK_PAGE_CHANGED)
-DEFINE_EVENT_TYPE(wxEVT_COMMAND_NOTEBOOK_PAGE_CHANGING)
-
 //-----------------------------------------------------------------------------
 // idle system
 //-----------------------------------------------------------------------------
@@ -68,8 +60,8 @@ public:
     wxGtkNotebookPage()
     {
         m_image = -1;
-        m_page = (GtkNotebookPage *) NULL;
-        m_box = (GtkWidget *) NULL;
+        m_page = NULL;
+        m_box = NULL;
     }
 
     wxString           m_text;
@@ -97,7 +89,7 @@ static void gtk_notebook_page_change_callback(GtkNotebook *WXUNUSED(widget),
     // are you trying to call SetSelection() from a notebook event handler?
     // you shouldn't!
     wxCHECK_RET( !notebook->m_inSwitchPage,
-                 _T("gtk_notebook_page_change_callback reentered") );
+                 wxT("gtk_notebook_page_change_callback reentered") );
 
     notebook->m_inSwitchPage = true;
     if (g_isIdle)
@@ -107,14 +99,14 @@ static void gtk_notebook_page_change_callback(GtkNotebook *WXUNUSED(widget),
 
     if (notebook->m_skipNextPageChangeEvent)
     {
-        // this event was programatically generated by ChangeSelection() and thus must
+        // this event was programmatically generated by ChangeSelection() and thus must
         // be skipped
         notebook->m_skipNextPageChangeEvent = false;
 
         // make wxNotebook::GetSelection() return the correct (i.e. consistent
-        // with wxNotebookEvent::GetSelection()) value even though the page is
+        // with wxBookCtrlEvent::GetSelection()) value even though the page is
         // not really changed in GTK+
-        notebook->m_selection = page;
+        notebook->SetSelection(page);
     }
     else
     {
@@ -126,9 +118,9 @@ static void gtk_notebook_page_change_callback(GtkNotebook *WXUNUSED(widget),
         else // change allowed
         {
             // make wxNotebook::GetSelection() return the correct (i.e. consistent
-            // with wxNotebookEvent::GetSelection()) value even though the page is
+            // with wxBookCtrlEvent::GetSelection()) value even though the page is
             // not really changed in GTK+
-            notebook->m_selection = page;
+            notebook->SetSelection(page);
 
             notebook->SendPageChangedEvent(old);
         }
@@ -234,10 +226,10 @@ static gint gtk_notebook_key_press_callback( GtkWidget *widget, GdkEventKey *gdk
     if ((gdk_event->keyval == GDK_Tab) || (gdk_event->keyval == GDK_ISO_Left_Tab))
     {
         int sel = notebook->GetSelection();
-        if (sel == -1)
+        if (sel == wxNOT_FOUND)
             return TRUE;
         wxGtkNotebookPage *nb_page = notebook->GetNotebookPage(sel);
-        wxCHECK_MSG( nb_page, FALSE, _T("invalid selection in wxNotebook") );
+        wxCHECK_MSG( nb_page, FALSE, wxT("invalid selection in wxNotebook") );
 
         wxNavigationKeyEvent event;
         event.SetEventObject( notebook );
@@ -249,7 +241,7 @@ static gint gtk_notebook_key_press_callback( GtkWidget *widget, GdkEventKey *gdk
         event.SetCurrentFocus( notebook );
 
         wxNotebookPage *client = notebook->GetPage(sel);
-        if ( !client->GetEventHandler()->ProcessEvent( event ) )
+        if ( !client->HandleWindowEvent( event ) )
         {
              client->SetFocus();
         }
@@ -287,19 +279,15 @@ static void wxInsertChildInNotebook( wxNotebook* parent, wxWindow* child )
 // wxNotebook
 //-----------------------------------------------------------------------------
 
-IMPLEMENT_DYNAMIC_CLASS(wxNotebook,wxControl)
-
-BEGIN_EVENT_TABLE(wxNotebook, wxControl)
+wxBEGIN_EVENT_TABLE(wxNotebook, wxBookCtrlBase)
     EVT_NAVIGATION_KEY(wxNotebook::OnNavigationKey)
-END_EVENT_TABLE()
+wxEND_EVENT_TABLE()
 
 void wxNotebook::Init()
 {
     m_padding = 0;
     m_inSwitchPage = false;
 
-    m_imageList = (wxImageList *) NULL;
-    m_selection = -1;
     m_themeEnabled = true;
 }
 
@@ -369,9 +357,9 @@ bool wxNotebook::Create(wxWindow *parent, wxWindowID id,
 
 int wxNotebook::GetSelection() const
 {
-    wxCHECK_MSG( m_widget != NULL, -1, wxT("invalid notebook") );
+    wxCHECK_MSG( m_widget != NULL, wxNOT_FOUND, wxT("invalid notebook") );
 
-    if ( m_selection == -1 )
+    if ( m_selection == wxNOT_FOUND )
     {
         GList *nb_pages = GTK_NOTEBOOK(m_widget)->children;
 
@@ -382,8 +370,8 @@ int wxNotebook::GetSelection() const
             gpointer cur = notebook->cur_page;
             if ( cur != NULL )
             {
-                wxConstCast(this, wxNotebook)->m_selection =
-                    g_list_index( nb_pages, cur );
+                const_cast<wxNotebook *>(this)->
+                    SetSelection(g_list_index( nb_pages, cur ));
             }
         }
     }
@@ -415,16 +403,16 @@ int wxNotebook::GetPageImage( size_t page ) const
 
 wxGtkNotebookPage* wxNotebook::GetNotebookPage( int page ) const
 {
-    wxCHECK_MSG( m_widget != NULL, (wxGtkNotebookPage*) NULL, wxT("invalid notebook") );
+    wxCHECK_MSG( m_widget != NULL, NULL, wxT("invalid notebook") );
 
-    wxCHECK_MSG( page < (int)m_pagesData.GetCount(), (wxGtkNotebookPage*) NULL, wxT("invalid notebook index") );
+    wxCHECK_MSG( page < (int)m_pagesData.GetCount(), NULL, wxT("invalid notebook index") );
 
     return m_pagesData.Item(page)->GetData();
 }
 
 int wxNotebook::DoSetSelection( size_t page, int flags )
 {
-    wxCHECK_MSG( m_widget != NULL, -1, wxT("invalid notebook") );
+    wxCHECK_MSG( m_widget != NULL, wxNOT_FOUND, wxT("invalid notebook") );
 
     wxCHECK_MSG( page < m_pagesData.GetCount(), -1, wxT("invalid notebook index") );
 
@@ -437,15 +425,13 @@ int wxNotebook::DoSetSelection( size_t page, int flags )
     m_selection = page;
     gtk_notebook_set_page( GTK_NOTEBOOK(m_widget), page );
 
-#ifdef __WXDEBUG__
-    if ( !(flags & SetSelection_SendEvent) )
-    {
-            // gtk_notebook_set_current_page will emit the switch-page signal which will be
-            // caught by our gtk_notebook_page_change_callback which should have reset the
-            // flag to false:
-        wxASSERT(!m_skipNextPageChangeEvent);
-    }
-#endif // __WXDEBUG__
+    // gtk_notebook_set_current_page is supposed to emit the switch-page signal
+    // which should be caught by our gtk_notebook_page_change_callback which
+    // should have reset the flag to false, check it:
+    wxASSERT_LEVEL_2_MSG(
+        (flags & SetSelection_SendEvent) || !m_skipNextPageChangeEvent,
+        "internal error in selection events generation"
+    );
 
     wxNotebookPage *client = GetPage(page);
     if ( client )
@@ -492,7 +478,7 @@ bool wxNotebook::SetPageImage( size_t page, int image )
     if (image == -1 && nb_page->m_image == -1)
         return true; /* Case 1): Nothing to do. */
 
-    GtkWidget *pixmapwid = (GtkWidget*) NULL;
+    GtkWidget *pixmapwid = NULL;
 
     if (nb_page->m_image != -1)
     {
@@ -523,15 +509,15 @@ bool wxNotebook::SetPageImage( size_t page, int image )
     }
 
     /* Only cases 3) and 4) left */
-    wxASSERT( m_imageList != NULL ); /* Just in case */
+    wxASSERT( HasImageList() ); /* Just in case */
 
     /* Construct the new pixmap */
-    const wxBitmap *bmp = m_imageList->GetBitmapPtr(image);
-    GdkPixmap *pixmap = bmp->GetPixmap();
-    GdkBitmap *mask = (GdkBitmap*) NULL;
-    if ( bmp->GetMask() )
+    const wxBitmap bmp = GetImageList()->GetBitmap(image);
+    GdkPixmap *pixmap = bmp.GetPixmap();
+    GdkBitmap *mask = NULL;
+    if ( bmp.GetMask() )
     {
-        mask = bmp->GetMask()->GetBitmap();
+        mask = bmp.GetMask()->GetBitmap();
     }
 
     if (pixmapwid == NULL)
@@ -598,18 +584,17 @@ bool wxNotebook::DeleteAllPages()
     while (m_pagesData.GetCount() > 0)
         DeletePage( m_pagesData.GetCount()-1 );
 
-    wxASSERT_MSG( GetPageCount() == 0, _T("all pages must have been deleted") );
+    wxASSERT_MSG( GetPageCount() == 0, wxT("all pages must have been deleted") );
 
-    InvalidateBestSize();
     return wxNotebookBase::DeleteAllPages();
 }
 
 wxNotebookPage *wxNotebook::DoRemovePage( size_t page )
 {
-    if ( m_selection != -1 && (size_t)m_selection >= page )
+    if ( m_selection != wxNOT_FOUND && (size_t)m_selection >= page )
     {
         // the index will become invalid after the page is deleted
-        m_selection = -1;
+        m_selection = wxNOT_FOUND;
     }
 
     wxNotebookPage *client = wxNotebookBase::DoRemovePage(page);
@@ -650,7 +635,7 @@ bool wxNotebook::InsertPage( size_t position,
                wxT("Can't add a page whose parent is not the notebook!") );
 
     wxCHECK_MSG( position <= GetPageCount(), FALSE,
-                 _T("invalid page index in wxNotebookPage::InsertPage()") );
+                 wxT("invalid page index in wxNotebookPage::InsertPage()") );
 
     // Hack Alert! (Part II): See above in wxInsertChildInNotebook callback
     // why this has to be done.  NOTE: using gtk_widget_unparent here does not
@@ -675,7 +660,7 @@ bool wxNotebook::InsertPage( size_t position,
     else
         m_pagesData.Insert( position, nb_page );
 
-    m_pages.Insert(win, position);
+    m_pages.insert(m_pages.begin() + position, win);
 
     nb_page->m_box = gtk_hbox_new( FALSE, 1 );
     gtk_container_border_width( GTK_CONTAINER(nb_page->m_box), 2 );
@@ -692,14 +677,14 @@ bool wxNotebook::InsertPage( size_t position,
 
     if (imageId != -1)
     {
-        wxASSERT( m_imageList != NULL );
+        wxASSERT( HasImageList() );
 
-        const wxBitmap *bmp = m_imageList->GetBitmapPtr(imageId);
-        GdkPixmap *pixmap = bmp->GetPixmap();
-        GdkBitmap *mask = (GdkBitmap*) NULL;
-        if ( bmp->GetMask() )
+        const wxBitmap bmp = GetImageList()->GetBitmap(imageId);
+        GdkPixmap *pixmap = bmp.GetPixmap();
+        GdkBitmap *mask = NULL;
+        if ( bmp.GetMask() )
         {
-            mask = bmp->GetMask()->GetBitmap();
+            mask = bmp.GetMask()->GetBitmap();
         }
 
         GtkWidget *pixmapwid = gtk_pixmap_new (pixmap, mask );
@@ -865,10 +850,4 @@ wxNotebook::GetClassDefaultAttributes(wxWindowVariant WXUNUSED(variant))
     return GetDefaultAttributesFromGTKWidget(gtk_notebook_new);
 }
 
-//-----------------------------------------------------------------------------
-// wxNotebookEvent
-//-----------------------------------------------------------------------------
-
-IMPLEMENT_DYNAMIC_CLASS(wxNotebookEvent, wxNotifyEvent)
-
 #endif
