@@ -1,10 +1,8 @@
 /////////////////////////////////////////////////////////////////////////////
-// Name:        msw/debughlp.cpp
+// Name:        src/msw/debughlp.cpp
 // Purpose:     various Win32 debug helpers
-// Author:      Vadim Zeitlin
-// Modified by:
+// Author:      Vadim Zeitlin, Suzumizaki-kimitaka
 // Created:     2005-01-08 (extracted from crashrpt.cpp)
-// RCS-ID:      $Id: debughlp.cpp,v 1.9 2005/07/20 18:32:57 VZ Exp $
 // Copyright:   (c) 2003-2005 Vadim Zeitlin <vadim@wxwindows.org>
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -23,9 +21,15 @@
     #pragma hdrstop
 #endif
 
+#ifndef WX_PRECOMP
+    #include "wx/wxcrt.h"
+#endif // WX_PRECOMP
+
 #include "wx/msw/debughlp.h"
 
-#if wxUSE_DBGHELP
+#if wxUSE_DBGHELP && wxUSE_DYNLIB_CLASS
+
+#include "wx/scopedarray.h"
 
 // ----------------------------------------------------------------------------
 // constants
@@ -35,6 +39,59 @@
 // ourselves to that many levels of embedded fields inside structs
 static const unsigned MAX_DUMP_DEPTH = 20;
 
+// ----------------------------------------------------------------------------
+// local helpers
+// ----------------------------------------------------------------------------
+
+namespace
+{
+
+// Automatically initialize SizeOfStruct field of debug help structs to the
+// correct value.
+template <class T>
+struct SizedStruct : public T
+{
+    SizedStruct()
+    {
+        ::ZeroMemory(this, sizeof(T));
+        this->SizeOfStruct = sizeof(T);
+    }
+};
+
+// This is used for working with variable-sized SYMBOL_INFO[W] structs that
+// have a variable length array as their last field.
+template <class T>
+class VarSizedStruct
+{
+public:
+    VarSizedStruct()
+    {
+        ::ZeroMemory(m_buffer, sizeof(T) + MAX_NAME_LEN*sizeof(TCHAR));
+
+        (*this)->SizeOfStruct = sizeof(T);
+        (*this)->MaxNameLen = MAX_NAME_LEN;
+    }
+
+    operator T*()
+    {
+        return static_cast<T*>(static_cast<void*>(m_buffer));
+    }
+
+    T* operator->()
+    {
+        return *this;
+    }
+
+private:
+    // Currently this is just a constant, we could make it a template parameter
+    // if we wanted.
+    enum { MAX_NAME_LEN = 1024 };
+
+    BYTE m_buffer[sizeof(T) + MAX_NAME_LEN*sizeof(TCHAR)];
+};
+
+} // anonymous namespace
+
 // ----------------------------------------------------------------------------
 // globals
 // ----------------------------------------------------------------------------
@@ -50,7 +107,8 @@ static wxString gs_errMsg;
 // static members
 // ----------------------------------------------------------------------------
 
-#define DEFINE_SYM_FUNCTION(func) wxDbgHelpDLL::func ## _t wxDbgHelpDLL::func = 0
+#define DEFINE_SYM_FUNCTION(func, name) \
+    wxDbgHelpDLL::func ## _t wxDbgHelpDLL::func = 0
 
 wxDO_FOR_ALL_SYM_FUNCS(DEFINE_SYM_FUNCTION);
 
@@ -62,28 +120,41 @@ wxDO_FOR_ALL_SYM_FUNCS(DEFINE_SYM_FUNCTION);
 
 // load all function we need from the DLL
 
-static bool BindDbgHelpFunctions(const wxDynamicLibrary& dllDbgHelp)
+/* static */
+bool wxDbgHelpDLL::BindDbgHelpFunctions(const wxDynamicLibrary& dllDbgHelp)
 {
-    #define LOAD_SYM_FUNCTION(name)                                           \
-        wxDbgHelpDLL::name = (wxDbgHelpDLL::name ## _t)                       \
-                                dllDbgHelp.GetSymbol(_T(#name));              \
-        if ( !wxDbgHelpDLL::name )                                            \
+    #define LOAD_SYM_FUNCTION(func, name)                                     \
+        wxDbgHelpDLL::func = (wxDbgHelpDLL::func ## _t)                       \
+            dllDbgHelp.GetSymbol(wxT(#name));                                 \
+        if ( !wxDbgHelpDLL::func )                                            \
         {                                                                     \
-            gs_errMsg += _T("Function ") _T(#name) _T("() not found.\n");     \
+            gs_errMsg += wxT("Function ") wxT(#name) wxT("() not found.\n");  \
             return false;                                                     \
         }
 
-    wxDO_FOR_ALL_SYM_FUNCS(LOAD_SYM_FUNCTION);
+    wxDO_FOR_ALL_SYM_FUNCS_REQUIRED(LOAD_SYM_FUNCTION);
 
     #undef LOAD_SYM_FUNCTION
 
+    #define LOAD_SYM_FUNCTION_OPTIONAL(func, name)                            \
+        if ( dllDbgHelp.HasSymbol(wxT(#name)) )                               \
+        {                                                                     \
+            wxDbgHelpDLL::func = (wxDbgHelpDLL::func ## _t)                   \
+                dllDbgHelp.GetSymbol(wxT(#name));                             \
+        }
+
+    wxDO_FOR_ALL_SYM_FUNCS_OPTIONAL(LOAD_SYM_FUNCTION_OPTIONAL);
+
+    #undef LOAD_SYM_FUNCTION_CAN_FAIL
+
     return true;
 }
 
 // called by Init() if we hadn't done this before
-static bool DoInit()
+/* static */
+bool wxDbgHelpDLL::DoInit()
 {
-    wxDynamicLibrary dllDbgHelp(_T("dbghelp.dll"), wxDL_VERBATIM);
+    wxDynamicLibrary dllDbgHelp(wxT("dbghelp.dll"), wxDL_VERBATIM);
     if ( dllDbgHelp.IsLoaded() )
     {
         if ( BindDbgHelpFunctions(dllDbgHelp) )
@@ -99,19 +170,19 @@ static bool DoInit()
             return true;
         }
 
-        gs_errMsg += _T("\nPlease update your dbghelp.dll version, ")
-                     _T("at least version 5.1 is needed!\n")
-                     _T("(if you already have a new version, please ")
-                     _T("put it in the same directory where the program is.)\n");
+        gs_errMsg += wxT("\nPlease update your dbghelp.dll version, ")
+                     wxT("at least version 5.1 is needed!\n")
+                     wxT("(if you already have a new version, please ")
+                     wxT("put it in the same directory where the program is.)\n");
     }
     else // failed to load dbghelp.dll
     {
-        gs_errMsg += _T("Please install dbghelp.dll available free of charge ")
-                     _T("from Microsoft to get more detailed crash information!");
+        gs_errMsg += wxT("Please install dbghelp.dll available free of charge ")
+                     wxT("from Microsoft to get more detailed crash information!");
     }
 
-    gs_errMsg += _T("\nLatest dbghelp.dll is available at ")
-                 _T("http://www.microsoft.com/whdc/ddk/debugging/\n");
+    gs_errMsg += wxT("\nLatest dbghelp.dll is available at ")
+                 wxT("http://www.microsoft.com/whdc/ddk/debugging/\n");
 
     return false;
 }
@@ -144,8 +215,8 @@ const wxString& wxDbgHelpDLL::GetErrorMessage()
 /* static */
 void wxDbgHelpDLL::LogError(const wxChar *func)
 {
-    ::OutputDebugString(wxString::Format(_T("dbghelp: %s() failed: %s\r\n"),
-                        func, wxSysErrorMsg(::GetLastError())));
+    ::OutputDebugString(wxString::Format(wxT("dbghelp: %s() failed: %s\r\n"),
+                        func, wxSysErrorMsgStr(::GetLastError())).t_str());
 }
 
 // ----------------------------------------------------------------------------
@@ -170,13 +241,13 @@ DoGetTypeInfo(DWORD64 base, ULONG ti, IMAGEHLP_SYMBOL_TYPE_INFO type, void *rc)
 
 static inline
 bool
-DoGetTypeInfo(PSYMBOL_INFO pSym, IMAGEHLP_SYMBOL_TYPE_INFO type, void *rc)
+DoGetTypeInfo(wxPSYMBOL_INFO pSym, IMAGEHLP_SYMBOL_TYPE_INFO type, void *rc)
 {
     return DoGetTypeInfo(pSym->ModBase, pSym->TypeIndex, type, rc);
 }
 
 static inline
-wxDbgHelpDLL::BasicType GetBasicType(PSYMBOL_INFO pSym)
+wxDbgHelpDLL::BasicType GetBasicType(wxPSYMBOL_INFO pSym)
 {
     wxDbgHelpDLL::BasicType bt;
     return DoGetTypeInfo(pSym, TI_GET_BASETYPE, &bt)
@@ -185,7 +256,7 @@ wxDbgHelpDLL::BasicType GetBasicType(PSYMBOL_INFO pSym)
 }
 
 /* static */
-wxString wxDbgHelpDLL::GetSymbolName(PSYMBOL_INFO pSym)
+wxString wxDbgHelpDLL::GetSymbolName(wxPSYMBOL_INFO pSym)
 {
     wxString s;
 
@@ -212,12 +283,12 @@ wxDbgHelpDLL::DumpBaseType(BasicType bt, DWORD64 length, PVOID pAddress)
 {
     if ( !pAddress )
     {
-        return _T("null");
+        return wxT("null");
     }
 
     if ( ::IsBadReadPtr(pAddress, length) != 0 )
     {
-        return _T("BAD");
+        return wxT("BAD");
     }
 
 
@@ -229,13 +300,13 @@ wxDbgHelpDLL::DumpBaseType(BasicType bt, DWORD64 length, PVOID pAddress)
         const BYTE b = *(PBYTE)pAddress;
 
         if ( bt == BASICTYPE_BOOL )
-            s = b ? _T("true") : _T("false");
+            s = b ? wxT("true") : wxT("false");
         else
-            s.Printf(_T("%#04x"), b);
+            s.Printf(wxT("%#04x"), b);
     }
     else if ( length == 2 )
     {
-        s.Printf(bt == BASICTYPE_UINT ? _T("%#06x") : _T("%d"),
+        s.Printf(bt == BASICTYPE_UINT ? wxT("%#06x") : wxT("%d"),
                  *(PWORD)pAddress);
     }
     else if ( length == 4 )
@@ -244,7 +315,7 @@ wxDbgHelpDLL::DumpBaseType(BasicType bt, DWORD64 length, PVOID pAddress)
 
         if ( bt == BASICTYPE_FLOAT )
         {
-            s.Printf(_T("%f"), *(PFLOAT)pAddress);
+            s.Printf(wxT("%f"), *(PFLOAT)pAddress);
 
             handled = true;
         }
@@ -256,12 +327,12 @@ wxDbgHelpDLL::DumpBaseType(BasicType bt, DWORD64 length, PVOID pAddress)
             const char *pc = *(PSTR *)pAddress;
             if ( ::IsBadStringPtrA(pc, NUM_CHARS) == 0 )
             {
-                s += _T('"');
+                s += wxT('"');
                 for ( size_t n = 0; n < NUM_CHARS && *pc; n++, pc++ )
                 {
                     s += *pc;
                 }
-                s += _T('"');
+                s += wxT('"');
 
                 handled = true;
             }
@@ -270,18 +341,18 @@ wxDbgHelpDLL::DumpBaseType(BasicType bt, DWORD64 length, PVOID pAddress)
         if ( !handled )
         {
             // treat just as an opaque DWORD
-            s.Printf(_T("%#x"), *(PDWORD)pAddress);
+            s.Printf(wxT("%#x"), *(PDWORD)pAddress);
         }
     }
     else if ( length == 8 )
     {
         if ( bt == BASICTYPE_FLOAT )
         {
-            s.Printf(_T("%lf"), *(double *)pAddress);
+            s.Printf(wxT("%lf"), *(double *)pAddress);
         }
         else // opaque 64 bit value
         {
-            s.Printf(_T("%#" wxLongLongFmtSpec _T("x")), *(PDWORD *)pAddress);
+            s.Printf("%#" wxLongLongFmtSpec "x", *(wxLongLong_t *)pAddress);
         }
     }
 
@@ -289,7 +360,7 @@ wxDbgHelpDLL::DumpBaseType(BasicType bt, DWORD64 length, PVOID pAddress)
 }
 
 wxString
-wxDbgHelpDLL::DumpField(PSYMBOL_INFO pSym, void *pVariable, unsigned level)
+wxDbgHelpDLL::DumpField(wxPSYMBOL_INFO pSym, void *pVariable, unsigned level)
 {
     wxString s;
 
@@ -315,7 +386,7 @@ wxDbgHelpDLL::DumpField(PSYMBOL_INFO pSym, void *pVariable, unsigned level)
         case SYMBOL_TAG_DATA:
             if ( !pVariable )
             {
-                s = _T("NULL");
+                s = wxT("NULL");
             }
             else // valid location
             {
@@ -336,7 +407,7 @@ wxDbgHelpDLL::DumpField(PSYMBOL_INFO pSym, void *pVariable, unsigned level)
 
 
                 // now pass to the type representing the type of this member
-                SYMBOL_INFO sym = *pSym;
+                wxSYMBOL_INFO sym = *pSym;
                 if ( !DoGetTypeInfo(pSym, TI_GET_TYPEID, &sym.TypeIndex) )
                     break;
 
@@ -359,26 +430,35 @@ wxDbgHelpDLL::DumpField(PSYMBOL_INFO pSym, void *pVariable, unsigned level)
                     case SYMBOL_TAG_BASE_CLASS:
                         s = DumpUDT(&sym, pVariable, level);
                         break;
+
+                    default:
+                        // Suppress gcc warnings about unhandled enum values.
+                        break;
                 }
             }
 
             if ( !s.empty() )
             {
-                s = GetSymbolName(pSym) + _T(" = ") + s;
+                s = GetSymbolName(pSym) + wxT(" = ") + s;
             }
             break;
+
+        default:
+            // Suppress gcc warnings about unhandled enum values, don't assert
+            // to avoid problems during fatal crash generation.
+            break;
     }
 
     if ( !s.empty() )
     {
-        s = wxString(_T('\t'), level + 1) + s + _T('\n');
+        s = wxString(wxT('\t'), level + 1) + s + wxT('\n');
     }
 
     return s;
 }
 
 /* static */ wxString
-wxDbgHelpDLL::DumpUDT(PSYMBOL_INFO pSym, void *pVariable, unsigned level)
+wxDbgHelpDLL::DumpUDT(wxPSYMBOL_INFO pSym, void *pVariable, unsigned level)
 {
     wxString s;
 
@@ -391,11 +471,11 @@ wxDbgHelpDLL::DumpUDT(PSYMBOL_INFO pSym, void *pVariable, unsigned level)
     s.reserve(512);
     s = GetSymbolName(pSym);
 
-#if !wxUSE_STL
+#if !wxUSE_STD_STRING
     // special handling for ubiquitous wxString: although the code below works
     // for it as well, it shows the wxStringBase class and takes 4 lines
     // instead of only one as this branch
-    if ( s == _T("wxString") )
+    if ( s == wxT("wxString") )
     {
         wxString *ps = (wxString *)pVariable;
 
@@ -416,10 +496,10 @@ wxDbgHelpDLL::DumpUDT(PSYMBOL_INFO pSym, void *pVariable, unsigned level)
             }
         }
 
-        s << _T("(\"") << (p ? p : _T("???")) << _T(")\"");
+        s << wxT("(\"") << (p ? p : wxT("???")) << wxT(")\"");
     }
     else // any other UDT
-#endif // !wxUSE_STL
+#endif // !wxUSE_STD_STRING
     {
         // Determine how many children this type has.
         DWORD dwChildrenCount = 0;
@@ -442,10 +522,10 @@ wxDbgHelpDLL::DumpUDT(PSYMBOL_INFO pSym, void *pVariable, unsigned level)
             return s;
         }
 
-        s << _T(" {\n");
+        s << wxT(" {\n");
 
         // Iterate through all children
-        SYMBOL_INFO sym;
+        wxSYMBOL_INFO sym;
         wxZeroMemory(sym);
         sym.ModBase = pSym->ModBase;
         for ( unsigned i = 0; i < dwChildrenCount; i++ )
@@ -469,7 +549,7 @@ wxDbgHelpDLL::DumpUDT(PSYMBOL_INFO pSym, void *pVariable, unsigned level)
 
         free(children);
 
-        s << wxString(_T('\t'), level + 1) << _T('}');
+        s << wxString(wxT('\t'), level + 1) << wxT('}');
     }
 
     return s;
@@ -477,7 +557,7 @@ wxDbgHelpDLL::DumpUDT(PSYMBOL_INFO pSym, void *pVariable, unsigned level)
 
 /* static */
 wxDbgHelpDLL::SymbolTag
-wxDbgHelpDLL::DereferenceSymbol(PSYMBOL_INFO pSym, void **ppData)
+wxDbgHelpDLL::DereferenceSymbol(wxPSYMBOL_INFO pSym, void **ppData)
 {
     SymbolTag tag = SYMBOL_TAG_NULL;
     for ( ;; )
@@ -514,12 +594,17 @@ wxDbgHelpDLL::DereferenceSymbol(PSYMBOL_INFO pSym, void **ppData)
 }
 
 /* static */ wxString
-wxDbgHelpDLL::DumpSymbol(PSYMBOL_INFO pSym, void *pVariable)
+wxDbgHelpDLL::DumpSymbol(wxPSYMBOL_INFO pSym, void *pVariable)
 {
     wxString s;
-    SYMBOL_INFO symDeref = *pSym;
+    wxSYMBOL_INFO symDeref = *pSym;
     switch ( DereferenceSymbol(&symDeref, &pVariable) )
     {
+        default:
+            // Suppress gcc warnings about unhandled enum values, don't assert
+            // to avoid problems during fatal crash generation.
+            break;
+
         case SYMBOL_TAG_UDT:
             // show UDT recursively
             s = DumpUDT(&symDeref, pVariable);
@@ -538,6 +623,322 @@ wxDbgHelpDLL::DumpSymbol(PSYMBOL_INFO pSym, void *pVariable)
     return s;
 }
 
+// ----------------------------------------------------------------------------
+// Helpers for selecting the best available function on the current platform.
+// ----------------------------------------------------------------------------
+
+// Store the original callback and the data for it: objects of this type are
+// used as bridges for the enumeration functions taking callbacks of different
+// types.
+struct wxEnumLoadedCallbackBridge
+{
+public:
+    wxEnumLoadedCallbackBridge(wxPENUMLOADED_MODULES_CALLBACK ptr, PVOID content)
+        : m_callback(ptr), m_data(content)
+    {
+    }
+
+    wxPENUMLOADED_MODULES_CALLBACK m_callback;
+    PVOID m_data;
+};
+
+// As mentioned near wxPENUMLOADED_MODULES_CALLBACK definition, until v11 of
+// the API the parameters were non-const.
+#if API_VERSION_NUMBER < 11
+    #define wxHAS_NON_CONST_MODULE_NAME
+
+    #define wxMODULE_NAMEA PSTR
+#else
+    #define wxMODULE_NAMEA PCSTR
+#endif
+
+#if defined(UNICODE) && defined(wxHAS_NON_CONST_MODULE_NAME)
+
+static BOOL CALLBACK
+wxEnumLoadedW64Callback(PWSTR ModuleName,
+                        DWORD64 ModuleBase,
+                        ULONG ModuleSize,
+                        PVOID UserContext)
+{
+    wxEnumLoadedCallbackBridge&
+        bridge = *static_cast<wxEnumLoadedCallbackBridge*>(UserContext);
+
+    return (*bridge.m_callback)(ModuleName, ModuleBase, ModuleSize, bridge.m_data);
+}
+
+#endif // UNICODE && wxHAS_NON_CONST_MODULE_NAME
+
+static BOOL CALLBACK
+wxEnumLoaded64Callback(wxMODULE_NAMEA ModuleName,
+                       DWORD64 ModuleBase,
+                       ULONG ModuleSize,
+                       PVOID UserContext)
+{
+    wxEnumLoadedCallbackBridge&
+        bridge = *static_cast<wxEnumLoadedCallbackBridge*>(UserContext);
+
+    return (*bridge.m_callback)
+           (
+#ifdef UNICODE
+                wxConvLocal.cMB2WC(ModuleName),
+#else // !UNICODE
+                ModuleName,
+#endif // UNICODE
+                ModuleBase, ModuleSize, bridge.m_data
+           );
+}
+
+static BOOL CALLBACK
+wxEnumLoadedCallback(wxMODULE_NAMEA ModuleName,
+                     DWORD_PTR ModuleBase,
+                     ULONG ModuleSize,
+                     PVOID UserContext)
+{
+    wxEnumLoadedCallbackBridge&
+        bridge = *static_cast<wxEnumLoadedCallbackBridge*>(UserContext);
+
+    return (*bridge.m_callback)
+           (
+#ifdef UNICODE
+                wxConvLocal.cMB2WC(ModuleName),
+#else // !UNICODE
+                ModuleName,
+#endif // UNICODE
+                ModuleBase, ModuleSize, bridge.m_data
+           );
+}
+
+/* static */
+BOOL
+wxDbgHelpDLL::CallEnumerateLoadedModules(HANDLE handle,
+                                         wxPENUMLOADED_MODULES_CALLBACK callback,
+                                         PVOID callbackParam)
+{
+#ifdef UNICODE
+    if ( EnumerateLoadedModulesW64 )
+    {
+#ifdef wxHAS_NON_CONST_MODULE_NAME
+        // We need to pass by a bridge just to convert non-const module name to
+        // the const one taken by our callback.
+        wxEnumLoadedCallbackBridge br(callback, callbackParam);
+        if ( EnumerateLoadedModulesW64(handle, &wxEnumLoadedW64Callback, &br) )
+            return TRUE;
+#else // new SDK
+        // We can use our callback directly.
+        if ( EnumerateLoadedModulesW64(handle, callback, callbackParam) )
+            return TRUE;
+#endif // old/new SDK
+    }
+#endif // UNICODE
+
+    if ( EnumerateLoadedModules64 )
+    {
+        // We need a bridge if we need to convert to Unicode or if we're using
+        // an older SDK with a non-const first argument, so just use it always
+        // to avoid having too many complicated preprocessor checks.
+        wxEnumLoadedCallbackBridge br(callback, callbackParam);
+        if ( EnumerateLoadedModules64(handle, &wxEnumLoaded64Callback, &br) )
+            return TRUE;
+    }
+
+    if ( EnumerateLoadedModules )
+    {
+        // With this function we need a bridge in any case because the type of
+        // module base argument used by EnumerateLoadedModules() differs from
+        // that used by EnumerateLoadedModules[W]64().
+        wxEnumLoadedCallbackBridge br(callback, callbackParam);
+        if ( EnumerateLoadedModules(handle, &wxEnumLoadedCallback, &br) )
+            return TRUE;
+    }
+
+    return FALSE;
+}
+
+/* static */
+BOOL
+wxDbgHelpDLL::CallSymInitialize(HANDLE hProcess, BOOL fInvadeProcess)
+{
+#ifdef UNICODE
+    if ( SymInitializeW )
+    {
+        if ( SymInitializeW(hProcess, NULL, fInvadeProcess) )
+            return TRUE;
+    }
+#endif // UNICODE
+
+    if ( SymInitialize )
+    {
+        if ( SymInitialize(hProcess, NULL, fInvadeProcess) )
+            return TRUE;
+    }
+
+    return FALSE;
+}
+
+/* static */
+BOOL
+wxDbgHelpDLL::CallSymFromAddr(HANDLE hProcess,
+                              DWORD64 Address,
+                              size_t* offset,
+                              wxString* name)
+{
+    DWORD64 dwDisplacement;
+
+#ifdef UNICODE
+    if ( SymFromAddrW )
+    {
+        VarSizedStruct<SYMBOL_INFOW> infoW;
+        if ( SymFromAddrW(hProcess, Address, &dwDisplacement, infoW) )
+        {
+            *offset = dwDisplacement;
+            *name = infoW->Name;
+            return TRUE;
+        }
+    }
+#endif // UNICODE
+
+    if ( SymFromAddr )
+    {
+        VarSizedStruct<SYMBOL_INFO> info;
+        if ( SymFromAddr(hProcess, Address, &dwDisplacement, info) )
+        {
+            *offset = dwDisplacement;
+            *name = info->Name;
+            return TRUE;
+        }
+    }
+
+    return FALSE;
+}
+
+/* static */
+BOOL
+wxDbgHelpDLL::CallSymGetLineFromAddr(HANDLE hProcess,
+                                     DWORD64 dwAddr,
+                                     wxString* fileName,
+                                     size_t* line)
+{
+    DWORD dwDisplacement;
+
+#ifdef UNICODE
+    if ( SymGetLineFromAddrW64 )
+    {
+        SizedStruct<IMAGEHLP_LINEW64> lineW64;
+        if ( SymGetLineFromAddrW64(hProcess, dwAddr, &dwDisplacement, &lineW64) )
+        {
+            *fileName = lineW64.FileName;
+            *line = lineW64.LineNumber;
+            return TRUE;
+        }
+    }
+#endif // UNICODE
+
+    if ( SymGetLineFromAddr64 )
+    {
+        SizedStruct<IMAGEHLP_LINE64> line64;
+        if ( SymGetLineFromAddr64(hProcess, dwAddr, &dwDisplacement, &line64) )
+        {
+            *fileName = line64.FileName;
+            *line = line64.LineNumber;
+            return TRUE;
+        }
+    }
+
+    if ( SymGetLineFromAddr )
+    {
+        SizedStruct<IMAGEHLP_LINE> line32;
+        if ( SymGetLineFromAddr(hProcess, dwAddr, &dwDisplacement, &line32) )
+        {
+            *fileName = line32.FileName;
+            *line = line32.LineNumber;
+            return TRUE;
+        }
+    }
+
+    return FALSE;
+}
+
+#ifdef UNICODE
+
+// Allow to adapt callback supposed to be used with SymEnumSymbolsW() with
+// SymEnumSymbols().
+struct wxEnumSymbolsCallbackBridge
+{
+public:
+    wxEnumSymbolsCallbackBridge(PSYM_ENUMERATESYMBOLS_CALLBACKW ptr, PVOID content)
+        : m_callback(ptr), m_data(content)
+    {
+    }
+
+    PSYM_ENUMERATESYMBOLS_CALLBACKW m_callback;
+    PVOID m_data;
+};
+
+static BOOL CALLBACK
+wxEnumSymbolsCallback(PSYMBOL_INFO pSymInfo, ULONG SymbolSize, PVOID UserContext)
+{
+    wxEnumSymbolsCallbackBridge&
+        bridge = *static_cast<wxEnumSymbolsCallbackBridge*>(UserContext);
+
+    const wxWCharBuffer buf = wxConvLocal.cMB2WC(pSymInfo->Name);
+    const size_t len = buf.length();
+
+    // Allocate enough space for the wide char version of the struct.
+    //
+    // Note that there is no +1 here because sizeof(SYMBOL_INFOW) already
+    // includes 1 byte of the name.
+    wxScopedArray<BYTE> symbolBuffer(sizeof(SYMBOL_INFOW) + len*sizeof(WCHAR));
+
+    SYMBOL_INFOW* const infoW = (SYMBOL_INFOW*)symbolBuffer.get();
+
+    // Copy the original struct contents except for the last byte, which is the
+    // first byte of its (narrow) name that we don't need.
+    CopyMemory(infoW, pSymInfo, sizeof(SYMBOL_INFO) - sizeof(CHAR));
+
+    // Size is different for narrow and wide variants of the struct, so fix it
+    // up now.
+    infoW->SizeOfStruct = sizeof(SYMBOL_INFOW);
+
+    // Finally copy the converted name, which is the reason for doing all this.
+    wxStrncpy(infoW->Name, buf.data(), len);
+
+    return (*bridge.m_callback)(infoW, SymbolSize, bridge.m_data);
+}
+
+#endif // UNICODE
+
+/* static */
+BOOL
+wxDbgHelpDLL::CallSymEnumSymbols(HANDLE hProcess,
+                                 ULONG64 baseOfDll,
+                                 wxPSYM_ENUMERATESYMBOLS_CALLBACK callback,
+                                 const PVOID callbackParam)
+{
+#ifdef UNICODE
+    if ( SymEnumSymbolsW )
+    {
+        if ( SymEnumSymbolsW(hProcess, baseOfDll, NULL, callback, callbackParam) )
+            return TRUE;
+    }
+
+    if ( SymEnumSymbols )
+    {
+        wxEnumSymbolsCallbackBridge br(callback, callbackParam);
+
+        if ( SymEnumSymbols(hProcess, baseOfDll, NULL, wxEnumSymbolsCallback, &br) )
+            return TRUE;
+    }
+#else // !UNICODE
+    if ( SymEnumSymbols )
+    {
+        if ( SymEnumSymbols(hProcess, baseOfDll, NULL, callback, callbackParam) )
+            return TRUE;
+    }
+#endif // UNICODE/!UNICODE
+
+    return FALSE;
+}
+
 // ----------------------------------------------------------------------------
 // debugging helpers
 // ----------------------------------------------------------------------------
@@ -550,37 +951,37 @@ static wxString TagString(wxDbgHelpDLL::SymbolTag tag)
 {
     static const wxChar *tags[] =
     {
-        _T("null"),
-        _T("exe"),
-        _T("compiland"),
-        _T("compiland details"),
-        _T("compiland env"),
-        _T("function"),
-        _T("block"),
-        _T("data"),
-        _T("annotation"),
-        _T("label"),
-        _T("public symbol"),
-        _T("udt"),
-        _T("enum"),
-        _T("function type"),
-        _T("pointer type"),
-        _T("array type"),
-        _T("base type"),
-        _T("typedef"),
-        _T("base class"),
-        _T("friend"),
-        _T("function arg type"),
-        _T("func debug start"),
-        _T("func debug end"),
-        _T("using namespace"),
-        _T("vtable shape"),
-        _T("vtable"),
-        _T("custom"),
-        _T("thunk"),
-        _T("custom type"),
-        _T("managed type"),
-        _T("dimension"),
+        wxT("null"),
+        wxT("exe"),
+        wxT("compiland"),
+        wxT("compiland details"),
+        wxT("compiland env"),
+        wxT("function"),
+        wxT("block"),
+        wxT("data"),
+        wxT("annotation"),
+        wxT("label"),
+        wxT("public symbol"),
+        wxT("udt"),
+        wxT("enum"),
+        wxT("function type"),
+        wxT("pointer type"),
+        wxT("array type"),
+        wxT("base type"),
+        wxT("typedef"),
+        wxT("base class"),
+        wxT("friend"),
+        wxT("function arg type"),
+        wxT("func debug start"),
+        wxT("func debug end"),
+        wxT("using namespace"),
+        wxT("vtable shape"),
+        wxT("vtable"),
+        wxT("custom"),
+        wxT("thunk"),
+        wxT("custom type"),
+        wxT("managed type"),
+        wxT("dimension"),
     };
 
     wxCOMPILE_TIME_ASSERT( WXSIZEOF(tags) == wxDbgHelpDLL::SYMBOL_TAG_MAX,
@@ -590,7 +991,7 @@ static wxString TagString(wxDbgHelpDLL::SymbolTag tag)
     if ( tag < WXSIZEOF(tags) )
         s = tags[tag];
     else
-        s.Printf(_T("unrecognized tag (%d)"), tag);
+        s.Printf(wxT("unrecognized tag (%d)"), tag);
 
     return s;
 }
@@ -599,16 +1000,16 @@ static wxString KindString(wxDbgHelpDLL::DataKind kind)
 {
     static const wxChar *kinds[] =
     {
-         _T("unknown"),
-         _T("local"),
-         _T("static local"),
-         _T("param"),
-         _T("object ptr"),
-         _T("file static"),
-         _T("global"),
-         _T("member"),
-         _T("static member"),
-         _T("constant"),
+         wxT("unknown"),
+         wxT("local"),
+         wxT("static local"),
+         wxT("param"),
+         wxT("object ptr"),
+         wxT("file static"),
+         wxT("global"),
+         wxT("member"),
+         wxT("static member"),
+         wxT("constant"),
     };
 
     wxCOMPILE_TIME_ASSERT( WXSIZEOF(kinds) == wxDbgHelpDLL::DATA_MAX,
@@ -618,7 +1019,7 @@ static wxString KindString(wxDbgHelpDLL::DataKind kind)
     if ( kind < WXSIZEOF(kinds) )
         s = kinds[kind];
     else
-        s.Printf(_T("unrecognized kind (%d)"), kind);
+        s.Printf(wxT("unrecognized kind (%d)"), kind);
 
     return s;
 }
@@ -627,9 +1028,9 @@ static wxString UdtKindString(wxDbgHelpDLL::UdtKind kind)
 {
     static const wxChar *kinds[] =
     {
-         _T("struct"),
-         _T("class"),
-         _T("union"),
+         wxT("struct"),
+         wxT("class"),
+         wxT("union"),
     };
 
     wxCOMPILE_TIME_ASSERT( WXSIZEOF(kinds) == wxDbgHelpDLL::UDT_MAX,
@@ -639,7 +1040,7 @@ static wxString UdtKindString(wxDbgHelpDLL::UdtKind kind)
     if ( kind < WXSIZEOF(kinds) )
         s = kinds[kind];
     else
-        s.Printf(_T("unrecognized UDT (%d)"), kind);
+        s.Printf(wxT("unrecognized UDT (%d)"), kind);
 
     return s;
 }
@@ -648,38 +1049,38 @@ static wxString TypeString(wxDbgHelpDLL::BasicType bt)
 {
     static const wxChar *types[] =
     {
-        _T("no type"),
-        _T("void"),
-        _T("char"),
-        _T("wchar"),
-        _T(""),
-        _T(""),
-        _T("int"),
-        _T("uint"),
-        _T("float"),
-        _T("bcd"),
-        _T("bool"),
-        _T(""),
-        _T(""),
-        _T("long"),
-        _T("ulong"),
-        _T(""),
-        _T(""),
-        _T(""),
-        _T(""),
-        _T(""),
-        _T(""),
-        _T(""),
-        _T(""),
-        _T(""),
-        _T(""),
-        _T("CURRENCY"),
-        _T("DATE"),
-        _T("VARIANT"),
-        _T("complex"),
-        _T("bit"),
-        _T("BSTR"),
-        _T("HRESULT"),
+        wxT("no type"),
+        wxT("void"),
+        wxT("char"),
+        wxT("wchar"),
+        wxT(""),
+        wxT(""),
+        wxT("int"),
+        wxT("uint"),
+        wxT("float"),
+        wxT("bcd"),
+        wxT("bool"),
+        wxT(""),
+        wxT(""),
+        wxT("long"),
+        wxT("ulong"),
+        wxT(""),
+        wxT(""),
+        wxT(""),
+        wxT(""),
+        wxT(""),
+        wxT(""),
+        wxT(""),
+        wxT(""),
+        wxT(""),
+        wxT(""),
+        wxT("CURRENCY"),
+        wxT("DATE"),
+        wxT("VARIANT"),
+        wxT("complex"),
+        wxT("bit"),
+        wxT("BSTR"),
+        wxT("HRESULT"),
     };
 
     wxCOMPILE_TIME_ASSERT( WXSIZEOF(types) == wxDbgHelpDLL::BASICTYPE_MAX,
@@ -690,7 +1091,7 @@ static wxString TypeString(wxDbgHelpDLL::BasicType bt)
         s = types[bt];
 
     if ( s.empty() )
-        s.Printf(_T("unrecognized type (%d)"), bt);
+        s.Printf(wxT("unrecognized type (%d)"), bt);
 
     return s;
 }
@@ -707,11 +1108,11 @@ extern "C" void DumpTI(ULONG ti)
     DoGetTypeInfo(&sym, TI_GET_SYMTAG, &tag);
     DoGetTypeInfo(&sym, TI_GET_TYPEID, &ti);
 
-    OutputDebugString(wxString::Format(_T("Type 0x%x: "), sym.TypeIndex));
+    OutputDebugString(wxString::Format(wxT("Type 0x%x: "), sym.TypeIndex));
     wxString name = wxDbgHelpDLL::GetSymbolName(&sym);
     if ( !name.empty() )
     {
-        OutputDebugString(wxString::Format(_T("name=\"%s\", "), name.c_str()));
+        OutputDebugString(wxString::Format(wxT("name=\"%s\", "), name.c_str()));
     }
 
     DWORD nested;
@@ -720,15 +1121,15 @@ extern "C" void DumpTI(ULONG ti)
         nested = FALSE;
     }
 
-    OutputDebugString(wxString::Format(_T("tag=%s%s"),
-                      nested ? _T("nested ") : wxEmptyString,
+    OutputDebugString(wxString::Format(wxT("tag=%s%s"),
+                      nested ? wxT("nested ") : wxEmptyString,
                       TagString(tag).c_str()));
     if ( tag == wxDbgHelpDLL::SYMBOL_TAG_UDT )
     {
         wxDbgHelpDLL::UdtKind udtKind;
         if ( DoGetTypeInfo(&sym, TI_GET_UDTKIND, &udtKind) )
         {
-            OutputDebugString(_T(" (") + UdtKindString(udtKind) + _T(')'));
+            OutputDebugString(wxT(" (") + UdtKindString(udtKind) + wxT(')'));
         }
     }
 
@@ -736,13 +1137,13 @@ extern "C" void DumpTI(ULONG ti)
     if ( DoGetTypeInfo(&sym, TI_GET_DATAKIND, &kind) )
     {
         OutputDebugString(wxString::Format(
-            _T(", kind=%s"), KindString(kind).c_str()));
+            wxT(", kind=%s"), KindString(kind).c_str()));
         if ( kind == wxDbgHelpDLL::DATA_MEMBER )
         {
             DWORD ofs = 0;
             if ( DoGetTypeInfo(&sym, TI_GET_OFFSET, &ofs) )
             {
-                OutputDebugString(wxString::Format(_T(" (ofs=0x%x)"), ofs));
+                OutputDebugString(wxString::Format(wxT(" (ofs=0x%x)"), ofs));
             }
         }
     }
@@ -750,16 +1151,16 @@ extern "C" void DumpTI(ULONG ti)
     wxDbgHelpDLL::BasicType bt = GetBasicType(&sym);
     if ( bt )
     {
-        OutputDebugString(wxString::Format(_T(", type=%s"),
+        OutputDebugString(wxString::Format(wxT(", type=%s"),
                                 TypeString(bt).c_str()));
     }
 
     if ( ti != sym.TypeIndex )
     {
-        OutputDebugString(wxString::Format(_T(", next ti=0x%x"), ti));
+        OutputDebugString(wxString::Format(wxT(", next ti=0x%x"), ti));
     }
 
-    OutputDebugString(_T("\r\n"));
+    OutputDebugString(wxT("\r\n"));
 }
 
 #endif // NDEBUG
