@@ -47,7 +47,7 @@
 // wxWin macros
 // ----------------------------------------------------------------------------
 
-IMPLEMENT_DYNAMIC_CLASS(wxImageList, wxObject)
+wxIMPLEMENT_DYNAMIC_CLASS(wxImageList, wxObject);
 
 #define GetHImageList()     ((HIMAGELIST)m_hImageList)
 
@@ -70,11 +70,13 @@ static HBITMAP GetMaskForImage(const wxBitmap& bitmap, const wxBitmap& mask);
 wxImageList::wxImageList()
 {
     m_hImageList = 0;
+    m_size = wxSize(0,0);
 }
 
 // Creates an image list
 bool wxImageList::Create(int width, int height, bool mask, int initial)
 {
+    m_size = wxSize(width, height);
     UINT flags = 0;
 
     // as we want to be able to use 32bpp bitmaps in the image lists, we always
@@ -82,11 +84,7 @@ bool wxImageList::Create(int width, int height, bool mask, int initial)
     // will make the best effort to show the bitmap if we do this resulting in
     // quite acceptable display while using a lower depth ILC_COLOR constant
     // (e.g. ILC_COLOR16) shows completely broken bitmaps
-#ifdef __WXWINCE__
-    flags |= ILC_COLOR;
-#else
     flags |= ILC_COLOR32;
-#endif
 
     // For comctl32.dll < 6 always use masks as it doesn't support alpha.
     if ( mask || wxApp::GetComCtl32Version() < 600 )
@@ -232,6 +230,19 @@ int wxImageList::Add(const wxBitmap& bitmap, const wxColour& maskColour)
 // Adds a bitmap and mask from an icon.
 int wxImageList::Add(const wxIcon& icon)
 {
+    // ComCtl32 prior 6.0 doesn't support images with alpha
+    // channel so if we have 32-bit icon with transparency
+    // we need to add it as a wxBitmap via dedicated method
+    // where alpha channel will be converted to the mask.
+    if ( wxApp::GetComCtl32Version() < 600 )
+    {
+        wxBitmap bmp(icon);
+        if ( bmp.HasAlpha() )
+        {
+            return Add(bmp);
+        }
+    }
+
     int index = ImageList_AddIcon(GetHImageList(), GetHiconOf(icon));
     if ( index == -1 )
     {
@@ -294,6 +305,19 @@ bool wxImageList::Replace(int index,
 // Replaces a bitmap and mask from an icon.
 bool wxImageList::Replace(int i, const wxIcon& icon)
 {
+    // ComCtl32 prior 6.0 doesn't support images with alpha
+    // channel so if we have 32-bit icon with transparency
+    // we need to replace it as a wxBitmap via dedicated method
+    // where alpha channel will be converted to the mask.
+    if ( wxApp::GetComCtl32Version() < 600 )
+    {
+        wxBitmap bmp(icon);
+        if ( bmp.HasAlpha() )
+        {
+            return Replace(i, bmp);
+        }
+    }
+
     bool ok = ImageList_ReplaceIcon(GetHImageList(), i, GetHiconOf(icon)) != -1;
     if ( !ok )
     {
@@ -378,10 +402,11 @@ wxBitmap wxImageList::GetBitmap(int index) const
     GetSize(index, bmp_width, bmp_height);
 
     wxBitmap bitmap(bmp_width, bmp_height);
+
+#if wxUSE_WXDIB && wxUSE_IMAGE
     wxMemoryDC dc;
     dc.SelectObject(bitmap);
 
-#if wxUSE_WXDIB && wxUSE_IMAGE
     IMAGEINFO ii;
     ImageList_GetImageInfo(GetHImageList(), index, &ii);
     if ( ii.hbmMask )
@@ -425,8 +450,6 @@ wxBitmap wxImageList::GetBitmap(int index) const
         // even if it requires more work (and takes more time).
         bitmap.MSWUpdateAlpha();
     }
-#else
-    wxBitmap bitmap;
 #endif
     return bitmap;
 }
@@ -438,11 +461,10 @@ wxIcon wxImageList::GetIcon(int index) const
     if (hIcon)
     {
         wxIcon icon;
-        icon.SetHICON((WXHICON)hIcon);
 
         int iconW, iconH;
         GetSize(index, iconW, iconH);
-        icon.SetSize(iconW, iconH);
+        icon.InitFromHICON((WXHICON)hIcon, iconW, iconH);
 
         return icon;
     }
