@@ -29,7 +29,7 @@
 //
 // The ActiveX control itself is particularly stubborn, calling
 // IOleInPlaceSite::OnPosRectChange every file change trying to set itself
-// to something different then what we told it to before.
+// to something different from what we told it to before.
 //
 // The docs are at
 // http://msdn.microsoft.com/library/en-us/wmplay10/mmp_sdk/windowsmediaplayer10sdk.asp
@@ -105,6 +105,7 @@ const IID IID_IWMPPlayer2 = {0x0E6B01D1,0xD407,0x4C85,{0xBF,0x5F,0x1C,0x01,0xF6,
 const IID IID_IWMPCore2 = {0xBC17E5B7,0x7561,0x4C18,{0xBB,0x90,0x17,0xD4,0x85,0x77,0x56,0x59}};
 const IID IID_IWMPCore3 = {0x7587C667,0x628F,0x499F,{0x88,0xE7,0x6A,0x6F,0x4E,0x88,0x84,0x64}};
 const IID IID_IWMPNetwork = {0xEC21B779,0xEDEF,0x462D,{0xBB,0xA4,0xAD,0x9D,0xDE,0x2B,0x29,0xA7}};
+const IID IID_IWMPControls2 = {0x6F030D25,0x0890,0x480F,{0x97, 0x75, 0x1F,0x7E,0x40,0xAB,0x5B,0x8E}};
 
 enum WMPOpenState
 {
@@ -265,6 +266,12 @@ public:
 
 };
 
+struct IWMPControls2 : public IWMPControls
+{
+public:
+    virtual /* [helpstring][propget][id] */ HRESULT STDMETHODCALLTYPE step(
+        /* [in] */ long lStep ) = 0;
+};
 
 struct IWMPSettings : public IDispatch
 {
@@ -665,6 +672,8 @@ public:
     IWMPPlayer* m_pWMPPlayer;       // Main activex interface
     IWMPSettings* m_pWMPSettings;   // Settings such as volume
     IWMPControls* m_pWMPControls;   // Control interface (play etc.)
+    IWMPControls2* m_pWMPControls2; // Control interface (play etc.)
+
     wxSize m_bestSize;              // Actual movie size
 
     bool m_bWasStateChanged;        // See the "introduction"
@@ -681,11 +690,9 @@ public:
     wxWMP10MediaEvtHandler(wxWMP10MediaBackend *amb) :
        m_amb(amb)
     {
-        m_amb->m_pAX->Connect(m_amb->m_pAX->GetId(),
-            wxEVT_ACTIVEX,
-            wxActiveXEventHandler(wxWMP10MediaEvtHandler::OnActiveX),
-            NULL, this
-                              );
+        m_amb->m_pAX->Bind(wxEVT_ACTIVEX,
+            &wxWMP10MediaEvtHandler::OnActiveX, this,
+            m_amb->m_pAX->GetId());
     }
 
     void OnActiveX(wxActiveXEvent& event);
@@ -719,7 +726,8 @@ wxWMP10MediaBackend::wxWMP10MediaBackend()
 #endif
                 m_pWMPPlayer(NULL),
                 m_pWMPSettings(NULL),
-                m_pWMPControls(NULL)
+                m_pWMPControls(NULL),
+                m_pWMPControls2(NULL)
 
 {
     m_evthandler = NULL;
@@ -751,6 +759,8 @@ wxWMP10MediaBackend::~wxWMP10MediaBackend()
             m_pWMPSettings->Release();
         if (m_pWMPControls)
             m_pWMPControls->Release();
+        if (m_pWMPControls2)
+            m_pWMPControls2->Release();
     }
 }
 
@@ -789,6 +799,8 @@ bool wxWMP10MediaBackend::CreateControl(wxControl* ctrl, wxWindow* parent,
             wxLogSysError(wxT("Could not obtain controls from WMP10!"));
             return false;
         }
+        if (m_pWMPControls )
+            m_pWMPControls->QueryInterface(IID_IWMPControls2, (void**)&m_pWMPControls2);
     }
 #endif
 
@@ -870,7 +882,7 @@ bool wxWMP10MediaBackend::CreateControl(wxControl* ctrl, wxWindow* parent,
 
     // don't erase the background of our control window so that resizing is a
     // bit smoother
-    m_ctrl->SetBackgroundStyle(wxBG_STYLE_CUSTOM);
+    m_ctrl->SetBackgroundStyle(wxBG_STYLE_PAINT);
 
     // success
     return true;
@@ -1138,8 +1150,18 @@ bool wxWMP10MediaBackend::Stop()
 //---------------------------------------------------------------------------
 bool wxWMP10MediaBackend::SetPosition(wxLongLong where)
 {
+    // The display does not update if only put_currentPosition is called.
+    // We have to find the time for the previous frame, set the control
+    // to that position and then tell it to step forward one frame.  This
+    // forces the control to draw the frame to the screen, otherwise we get
+    // just a black screen.
+
+    double timePerFrameInMSec = 0;
+    if (m_pWMPControls2)
+        timePerFrameInMSec = 1000 / GetPlaybackRate();
+
     HRESULT hr = m_pWMPControls->put_currentPosition(
-                        ((LONGLONG)where.GetValue()) / 1000.0
+                        ((LONGLONG)where.GetValue() - timePerFrameInMSec) / 1000.0
                                      );
     if(FAILED(hr))
     {
@@ -1147,6 +1169,10 @@ bool wxWMP10MediaBackend::SetPosition(wxLongLong where)
         return false;
     }
 
+    if (m_pWMPControls2)
+        m_pWMPControls2->step(1);
+
+
     return true;
 }
 
