@@ -4,7 +4,6 @@
 // Author:      Vadim Zeitlin
 // Modified by:
 // Created:     03.04.98
-// RCS-ID:      $Id$
 // Copyright:   (c) 1998 Vadim Zeitlin <zeitlin@dptmaths.ens-cachan.fr>
 // Licence:     wxWindows licence
 // TODO:        - parsing of registry key names
@@ -27,17 +26,13 @@
     #include "wx/intl.h"
     #include "wx/log.h"
     #include "wx/crt.h"
+    #include "wx/utils.h"
 #endif
 
+#include "wx/dynlib.h"
 #include "wx/file.h"
 #include "wx/wfstream.h"
-
-// Windows headers
-#ifdef __WXWINCE__
 #include "wx/msw/private.h"
-#include <winbase.h>
-#include <winreg.h>
-#endif
 
 // other std headers
 #include  <stdlib.h>      // for _MAX_PATH
@@ -54,16 +49,8 @@
 typedef unsigned char *RegString;
 typedef BYTE* RegBinary;
 
-#ifndef HKEY_PERFORMANCE_DATA
-    #define HKEY_PERFORMANCE_DATA ((HKEY)0x80000004)
-#endif
-
-#ifndef HKEY_CURRENT_CONFIG
-    #define HKEY_CURRENT_CONFIG ((HKEY)0x80000005)
-#endif
-
-#ifndef HKEY_DYN_DATA
-    #define HKEY_DYN_DATA ((HKEY)0x80000006)
+#ifndef KEY_WOW64_64KEY
+    #define KEY_WOW64_64KEY 0x0100
 #endif
 
 // ----------------------------------------------------------------------------
@@ -84,9 +71,9 @@ aStdKeys[] =
   { HKEY_CURRENT_USER,      wxT("HKEY_CURRENT_USER"),      wxT("HKCU") },
   { HKEY_LOCAL_MACHINE,     wxT("HKEY_LOCAL_MACHINE"),     wxT("HKLM") },
   { HKEY_USERS,             wxT("HKEY_USERS"),             wxT("HKU")  }, // short name?
-  { HKEY_PERFORMANCE_DATA,  wxT("HKEY_PERFORMANCE_DATA"),  wxT("HKPD") },
+  { HKEY_PERFORMANCE_DATA,  wxT("HKEY_PERFORMANCE_DATA"),  wxT("HKPD") }, // (Obsolete under XP and later)
   { HKEY_CURRENT_CONFIG,    wxT("HKEY_CURRENT_CONFIG"),    wxT("HKCC") },
-  { HKEY_DYN_DATA,          wxT("HKEY_DYN_DATA"),          wxT("HKDD") }, // short name?
+  { HKEY_DYN_DATA,          wxT("HKEY_DYN_DATA"),          wxT("HKDD") }, // (Obsolete under XP and later)
 };
 
 // the registry name separator (perhaps one day MS will change it to '/' ;-)
@@ -114,7 +101,19 @@ aStdKeys[] =
 static inline void RemoveTrailingSeparator(wxString& str);
 
 // returns true if given registry key exists
-static bool KeyExists(WXHKEY hRootKey, const wxString& szKey);
+static bool KeyExists(
+    WXHKEY hRootKey,
+    const wxString& szKey,
+    wxRegKey::WOW64ViewMode viewMode = wxRegKey::WOW64ViewMode_Default);
+
+// return the WOW64 registry view flag which can be used with MSW registry
+// functions for opening the key in the specified view
+static long GetMSWViewFlags(wxRegKey::WOW64ViewMode viewMode);
+
+// return the access rights which can be used with MSW registry functions for
+// opening the key in the specified mode
+static long
+GetMSWAccessFlags(wxRegKey::AccessMode mode, wxRegKey::WOW64ViewMode viewMode);
 
 // combines value and key name
 static wxString GetFullName(const wxRegKey *pKey);
@@ -125,6 +124,33 @@ static wxString GetFullName(const wxRegKey *pKey, const wxString& szValue);
 // to NULL
 static inline const wxChar *RegValueStr(const wxString& szValue);
 
+// Return the user-readable name of the given REG_XXX type constant.
+static wxString GetTypeString(DWORD dwType)
+{
+#define REG_TYPE_TO_STR(type) case REG_ ## type: return wxS(#type)
+
+    switch ( dwType )
+    {
+        REG_TYPE_TO_STR(NONE);
+        REG_TYPE_TO_STR(SZ);
+        REG_TYPE_TO_STR(EXPAND_SZ);
+        REG_TYPE_TO_STR(BINARY);
+        REG_TYPE_TO_STR(DWORD);
+        // REG_TYPE_TO_STR(DWORD_LITTLE_ENDIAN); -- same as REG_DWORD
+        REG_TYPE_TO_STR(DWORD_BIG_ENDIAN);
+        REG_TYPE_TO_STR(LINK);
+        REG_TYPE_TO_STR(MULTI_SZ);
+        REG_TYPE_TO_STR(RESOURCE_LIST);
+        REG_TYPE_TO_STR(FULL_RESOURCE_DESCRIPTOR);
+        REG_TYPE_TO_STR(RESOURCE_REQUIREMENTS_LIST);
+        REG_TYPE_TO_STR(QWORD);
+        // REG_TYPE_TO_STR(QWORD_LITTLE_ENDIAN); -- same as REG_QWORD
+
+        default:
+            return wxString::Format(_("unknown (%lu)"), dwType);
+    }
+}
+
 // ============================================================================
 // implementation of wxRegKey class
 // ============================================================================
@@ -195,14 +221,15 @@ wxRegKey::StdKey wxRegKey::GetStdKeyFromHkey(WXHKEY hkey)
 // ctors and dtor
 // ----------------------------------------------------------------------------
 
-wxRegKey::wxRegKey()
+wxRegKey::wxRegKey(WOW64ViewMode viewMode) : m_viewMode(viewMode)
 {
   m_hRootKey = (WXHKEY) aStdKeys[HKCR].hkey;
 
   Init();
 }
 
-wxRegKey::wxRegKey(const wxString& strKey) : m_strKey(strKey)
+wxRegKey::wxRegKey(const wxString& strKey, WOW64ViewMode viewMode)
+    : m_strKey(strKey), m_viewMode(viewMode)
 {
   m_hRootKey  = (WXHKEY) aStdKeys[ExtractKeyName(m_strKey)].hkey;
 
@@ -210,7 +237,10 @@ wxRegKey::wxRegKey(const wxString& strKey) : m_strKey(strKey)
 }
 
 // parent is a predefined (and preopened) key
-wxRegKey::wxRegKey(StdKey keyParent, const wxString& strKey) : m_strKey(strKey)
+wxRegKey::wxRegKey(StdKey keyParent,
+                   const wxString& strKey,
+                   WOW64ViewMode viewMode)
+    : m_strKey(strKey), m_viewMode(viewMode)
 {
   RemoveTrailingSeparator(m_strKey);
   m_hRootKey  = (WXHKEY) aStdKeys[keyParent].hkey;
@@ -220,7 +250,7 @@ wxRegKey::wxRegKey(StdKey keyParent, const wxString& strKey) : m_strKey(strKey)
 
 // parent is a normal regkey
 wxRegKey::wxRegKey(const wxRegKey& keyParent, const wxString& strKey)
-        : m_strKey(keyParent.m_strKey)
+    : m_strKey(keyParent.m_strKey), m_viewMode(keyParent.GetView())
 {
   // combine our name with parent's to get the full name
   if ( !m_strKey.empty() &&
@@ -304,7 +334,7 @@ void wxRegKey::SetHkey(WXHKEY hKey)
   m_mode = Write;
 
   // reset old data
-  m_strKey.empty();
+  m_strKey.clear();
   m_dwLastError = 0;
 }
 
@@ -316,7 +346,9 @@ void wxRegKey::SetHkey(WXHKEY hKey)
 bool wxRegKey::Exists() const
 {
   // opened key has to exist, try to open it if not done yet
-  return IsOpened() ? true : KeyExists(m_hRootKey, m_strKey.wx_str());
+  return IsOpened()
+      ? true
+      : KeyExists(m_hRootKey, m_strKey, m_viewMode);
 }
 
 // returns the full name of the key (prefix is abbreviated if bShortPrefix)
@@ -336,36 +368,50 @@ bool wxRegKey::GetKeyInfo(size_t *pnSubKeys,
                           size_t *pnValues,
                           size_t *pnMaxValueLen) const
 {
-    // old gcc headers incorrectly prototype RegQueryInfoKey()
-#if defined(__GNUWIN32_OLD__) && !defined(__CYGWIN10__)
-    #define REG_PARAM   (size_t *)
-#else
-    #define REG_PARAM   (LPDWORD)
-#endif
-
   // it might be unexpected to some that this function doesn't open the key
   wxASSERT_MSG( IsOpened(), wxT("key should be opened in GetKeyInfo") );
 
+  // We need to use intermediate variables in 64 bit build as the function
+  // parameters must be 32 bit DWORDs and not 64 bit size_t values.
+#ifdef __WIN64__
+  DWORD dwSubKeys = 0,
+        dwMaxKeyLen = 0,
+        dwValues = 0,
+        dwMaxValueLen = 0;
+
+  #define REG_PARAM(name) &dw##name
+#else // Win32
+  #define REG_PARAM(name)   (LPDWORD)(pn##name)
+#endif
+
+
   m_dwLastError = ::RegQueryInfoKey
                   (
                     (HKEY) m_hKey,
-                    NULL,           // class name
-                    NULL,           // (ptr to) size of class name buffer
+                    NULL,                   // class name
+                    NULL,                   // (ptr to) size of class name buffer
                     RESERVED,
-                    REG_PARAM
-                    pnSubKeys,      // [out] number of subkeys
-                    REG_PARAM
-                    pnMaxKeyLen,    // [out] max length of a subkey name
-                    NULL,           // longest subkey class name
-                    REG_PARAM
-                    pnValues,       // [out] number of values
-                    REG_PARAM
-                    pnMaxValueLen,  // [out] max length of a value name
-                    NULL,           // longest value data
-                    NULL,           // security descriptor
-                    NULL            // time of last modification
+                    REG_PARAM(SubKeys),     // [out] number of subkeys
+                    REG_PARAM(MaxKeyLen),   // [out] max length of a subkey name
+                    NULL,                   // longest subkey class name
+                    REG_PARAM(Values),      // [out] number of values
+                    REG_PARAM(MaxValueLen), // [out] max length of a value name
+                    NULL,                   // longest value data
+                    NULL,                   // security descriptor
+                    NULL                    // time of last modification
                   );
 
+#ifdef __WIN64__
+  if ( pnSubKeys )
+    *pnSubKeys = dwSubKeys;
+  if ( pnMaxKeyLen )
+    *pnMaxKeyLen = dwMaxKeyLen;
+  if ( pnValues )
+    *pnValues = dwValues;
+  if ( pnMaxValueLen )
+    *pnMaxValueLen = dwMaxValueLen;
+#endif // __WIN64__
+
 #undef REG_PARAM
 
   if ( m_dwLastError != ERROR_SUCCESS ) {
@@ -399,7 +445,7 @@ bool wxRegKey::Open(AccessMode mode)
                         (HKEY) m_hRootKey,
                         m_strKey.t_str(),
                         RESERVED,
-                        mode == Read ? KEY_READ : KEY_ALL_ACCESS,
+                        GetMSWAccessFlags(mode, m_viewMode),
                         &tmpKey
                     );
 
@@ -427,19 +473,16 @@ bool wxRegKey::Create(bool bOkIfExists)
     return true;
 
   HKEY tmpKey;
-#ifdef __WXWINCE__
   DWORD disposition;
-  m_dwLastError = RegCreateKeyEx((HKEY) m_hRootKey, m_strKey.wx_str(),
-      NULL, // reserved
-      NULL, // class string
-      0,
-      0,
-      NULL,
+  m_dwLastError = RegCreateKeyEx((HKEY) m_hRootKey, m_strKey.t_str(),
+      0,    // reserved and must be 0
+      NULL, // The user-defined class type of this key.
+      REG_OPTION_NON_VOLATILE, // supports other values as well; see MS docs
+      GetMSWAccessFlags(wxRegKey::Write, m_viewMode),
+      NULL, // pointer to a SECURITY_ATTRIBUTES structure
       &tmpKey,
       &disposition);
-#else
-  m_dwLastError = RegCreateKey((HKEY) m_hRootKey, m_strKey.t_str(), &tmpKey);
-#endif
+
   if ( m_dwLastError != ERROR_SUCCESS ) {
     wxLogSysError(m_dwLastError, _("Can't create registry key '%s'"),
                   GetName().c_str());
@@ -504,9 +547,10 @@ bool wxRegKey::CopyValue(const wxString& szValue,
 
     switch ( GetValueType(szValue) ) {
         case Type_String:
+        case Type_Expand_String:
             {
                 wxString strVal;
-                return QueryValue(szValue, strVal) &&
+                return QueryRawValue(szValue, strVal) &&
                        keyDst.SetValue(valueNew, strVal);
             }
 
@@ -530,7 +574,6 @@ bool wxRegKey::CopyValue(const wxString& szValue,
         // occur among the application keys (supposedly created with
         // this class)
         case Type_None:
-        case Type_Expand_String:
         case Type_Dword_big_endian:
         case Type_Link:
         case Type_Multi_String:
@@ -710,8 +753,24 @@ bool wxRegKey::DeleteSelf()
   // now delete this key itself
   Close();
 
-  m_dwLastError = RegDeleteKey((HKEY) m_hRootKey, m_strKey.t_str());
   // deleting a key which doesn't exist is not considered an error
+#if wxUSE_DYNLIB_CLASS
+  wxDynamicLibrary dllAdvapi32(wxT("advapi32"));
+  // Minimum supported OS for RegDeleteKeyEx: Vista, XP Pro x64, Win Server 2008, Win Server 2003 SP1
+  typedef LONG (WINAPI *RegDeleteKeyEx_t)(HKEY, LPCTSTR, REGSAM, DWORD);
+  RegDeleteKeyEx_t wxDL_INIT_FUNC_AW(pfn, RegDeleteKeyEx, dllAdvapi32);
+  if (pfnRegDeleteKeyEx)
+  {
+    m_dwLastError = (*pfnRegDeleteKeyEx)((HKEY) m_hRootKey, m_strKey.t_str(),
+        GetMSWViewFlags(m_viewMode),
+        0);    // This parameter is reserved and must be zero.
+  }
+  else
+#endif // wxUSE_DYNLIB_CLASS
+  {
+    m_dwLastError = RegDeleteKey((HKEY) m_hRootKey, m_strKey.t_str());
+  }
+
   if ( m_dwLastError != ERROR_SUCCESS &&
           m_dwLastError != ERROR_FILE_NOT_FOUND ) {
     wxLogSysError(m_dwLastError, _("Can't delete key '%s'"),
@@ -803,7 +862,7 @@ bool wxRegKey::HasSubKey(const wxString& szKey) const
   if ( !CONST_CAST Open(Read) )
     return false;
 
-  return KeyExists(m_hKey, szKey);
+  return KeyExists(m_hKey, szKey, m_viewMode);
 }
 
 wxRegKey::ValueType wxRegKey::GetValueType(const wxString& szValue) const
@@ -851,13 +910,15 @@ bool wxRegKey::QueryValue(const wxString& szValue, long *plValue) const
                     GetName().c_str());
       return false;
     }
-    else {
-      // check that we read the value of right type
-      wxASSERT_MSG( IsNumericValue(szValue),
-                    wxT("Type mismatch in wxRegKey::QueryValue().")  );
 
-      return true;
+    // check that we read the value of right type
+    if ( dwType != REG_DWORD_LITTLE_ENDIAN && dwType != REG_DWORD_BIG_ENDIAN ) {
+      wxLogError(_("Registry value \"%s\" is not numeric (but of type %s)"),
+                 GetFullName(this, szValue), GetTypeString(dwType));
+      return false;
     }
+
+    return true;
   }
   else
     return false;
@@ -865,10 +926,6 @@ bool wxRegKey::QueryValue(const wxString& szValue, long *plValue) const
 
 bool wxRegKey::SetValue(const wxString& szValue, const wxMemoryBuffer& buffer)
 {
-#ifdef __TWIN32__
-  wxFAIL_MSG("RegSetValueEx not implemented by TWIN32");
-  return false;
-#else
   if ( CONST_CAST Open() ) {
     m_dwLastError = RegSetValueEx((HKEY) m_hKey, RegValueStr(szValue),
                                   (DWORD) RESERVED, REG_BINARY,
@@ -880,7 +937,6 @@ bool wxRegKey::SetValue(const wxString& szValue, const wxMemoryBuffer& buffer)
   wxLogSysError(m_dwLastError, _("Can't set value of '%s'"),
                 GetFullName(this, szValue));
   return false;
-#endif
 }
 
 bool wxRegKey::QueryValue(const wxString& szValue, wxMemoryBuffer& buffer) const
@@ -893,6 +949,12 @@ bool wxRegKey::QueryValue(const wxString& szValue, wxMemoryBuffer& buffer) const
                                     &dwType, NULL, &dwSize);
 
     if ( m_dwLastError == ERROR_SUCCESS ) {
+        if ( dwType != REG_BINARY ) {
+          wxLogError(_("Registry value \"%s\" is not binary (but of type %s)"),
+                     GetFullName(this, szValue), GetTypeString(dwType));
+          return false;
+        }
+
         if ( dwSize ) {
             const RegBinary pBuf = (RegBinary)buffer.GetWriteBuf(dwSize);
             m_dwLastError = RegQueryValueEx((HKEY) m_hKey,
@@ -922,7 +984,7 @@ bool wxRegKey::QueryValue(const wxString& szValue, wxMemoryBuffer& buffer) const
 
 bool wxRegKey::QueryValue(const wxString& szValue,
                           wxString& strValue,
-                          bool WXUNUSED_IN_WINCE(raw)) const
+                          bool raw) const
 {
     if ( CONST_CAST Open(Read) )
     {
@@ -935,7 +997,16 @@ bool wxRegKey::QueryValue(const wxString& szValue,
                                         &dwType, NULL, &dwSize);
         if ( m_dwLastError == ERROR_SUCCESS )
         {
-            if ( !dwSize )
+            if ( dwType != REG_SZ && dwType != REG_EXPAND_SZ )
+            {
+                wxLogError(_("Registry value \"%s\" is not text (but of type %s)"),
+                             GetFullName(this, szValue), GetTypeString(dwType));
+                return false;
+            }
+
+            // We need length in characters, not bytes.
+            DWORD chars = dwSize / sizeof(wxChar);
+            if ( !chars )
             {
                 // must treat this case specially as GetWriteBuf() doesn't like
                 // being called with 0 size
@@ -943,15 +1014,26 @@ bool wxRegKey::QueryValue(const wxString& szValue,
             }
             else
             {
-                m_dwLastError = RegQueryValueEx((HKEY) m_hKey,
-                                                RegValueStr(szValue),
-                                                RESERVED,
-                                                &dwType,
-                                                (RegString)(wxChar*)wxStringBuffer(strValue, dwSize),
-                                                &dwSize);
+                // extra scope for wxStringBufferLength
+                {
+                    wxStringBufferLength strBuf(strValue, chars);
+                    m_dwLastError = RegQueryValueEx((HKEY) m_hKey,
+                                                    RegValueStr(szValue),
+                                                    RESERVED,
+                                                    &dwType,
+                                                    (RegString)(wxChar*)strBuf,
+                                                    &dwSize);
+
+                    // The returned string may or not be NUL-terminated,
+                    // exclude the trailing NUL if it's there (which is
+                    // typically the case but is not guaranteed to always be).
+                    if ( strBuf[chars - 1] == '\0' )
+                        chars--;
+
+                    strBuf.SetLength(chars);
+                }
 
                 // expand the var expansions in the string unless disabled
-#ifndef __WXWINCE__
                 if ( (dwType == REG_EXPAND_SZ) && !raw )
                 {
                     DWORD dwExpSize = ::ExpandEnvironmentStrings(strValue.t_str(), NULL, 0);
@@ -971,18 +1053,10 @@ bool wxRegKey::QueryValue(const wxString& szValue,
                         wxLogLastError(wxT("ExpandEnvironmentStrings"));
                     }
                 }
-#endif
-                // __WXWINCE__
             }
 
             if ( m_dwLastError == ERROR_SUCCESS )
-            {
-                // check that it was the right type
-                wxASSERT_MSG( !IsNumericValue(szValue),
-                              wxT("Type mismatch in wxRegKey::QueryValue().") );
-
               return true;
-            }
         }
     }
 
@@ -997,7 +1071,7 @@ bool wxRegKey::SetValue(const wxString& szValue, const wxString& strValue)
       m_dwLastError = RegSetValueEx((HKEY) m_hKey,
                                     RegValueStr(szValue),
                                     (DWORD) RESERVED, REG_SZ,
-                                    (RegString)strValue.wx_str(),
+                                    (RegString)wxMSW_CONV_LPCTSTR(strValue),
                                     (strValue.Len() + 1)*sizeof(wxChar));
       if ( m_dwLastError == ERROR_SUCCESS )
         return true;
@@ -1032,11 +1106,11 @@ bool wxRegKey::GetFirstValue(wxString& strValueName, long& lIndex)
 
 bool wxRegKey::GetNextValue(wxString& strValueName, long& lIndex) const
 {
-  wxASSERT( IsOpened() );
+    wxASSERT( IsOpened() );
 
-  // are we already at the end of enumeration?
-  if ( lIndex == -1 )
-    return false;
+    // are we already at the end of enumeration?
+    if ( lIndex == -1 )
+        return false;
 
     wxChar  szValueName[1024];                  // @@ use RegQueryInfoKey...
     DWORD dwValueLen = WXSIZEOF(szValueName);
@@ -1085,13 +1159,7 @@ bool wxRegKey::GetNextKey(wxString& strKeyName, long& lIndex) const
 
   wxChar szKeyName[_MAX_PATH + 1];
 
-#ifdef __WXWINCE__
-  DWORD sizeName = WXSIZEOF(szKeyName);
-  m_dwLastError = RegEnumKeyEx((HKEY) m_hKey, lIndex++, szKeyName, & sizeName,
-      0, NULL, NULL, NULL);
-#else
   m_dwLastError = RegEnumKey((HKEY) m_hKey, lIndex++, szKeyName, WXSIZEOF(szKeyName));
-#endif
 
   if ( m_dwLastError != ERROR_SUCCESS ) {
     if ( m_dwLastError == ERROR_NO_MORE_ITEMS ) {
@@ -1175,7 +1243,7 @@ bool wxRegKey::Export(const wxString& filename) const
 
     wxFFileOutputStream ostr(filename, wxT("w"));
 
-    return ostr.Ok() && Export(ostr);
+    return ostr.IsOk() && Export(ostr);
 #else
     wxUnusedVar(filename);
     return false;
@@ -1237,9 +1305,10 @@ wxString wxRegKey::FormatValue(const wxString& name) const
     switch ( type )
     {
         case Type_String:
+        case Type_Expand_String:
             {
                 wxString value;
-                if ( !QueryValue(name, value) )
+                if ( !QueryRawValue(name, value) )
                     break;
 
                 // quotes and backslashes must be quoted, linefeeds are not
@@ -1263,7 +1332,7 @@ wxString wxRegKey::FormatValue(const wxString& name) const
                         case wxT('\\'):
                             // escape special symbol
                             rhs += wxT('\\');
-                            // fall through
+                            wxFALLTHROUGH;
 
                         default:
                             rhs += *p;
@@ -1288,7 +1357,6 @@ wxString wxRegKey::FormatValue(const wxString& name) const
             }
             break;
 
-        case Type_Expand_String:
         case Type_Multi_String:
             {
                 wxString value;
@@ -1410,7 +1478,9 @@ bool wxRegKey::DoExport(wxOutputStream& ostr) const
 // implementation of global private functions
 // ============================================================================
 
-bool KeyExists(WXHKEY hRootKey, const wxString& szKey)
+bool KeyExists(WXHKEY hRootKey,
+               const wxString& szKey,
+               wxRegKey::WOW64ViewMode viewMode)
 {
     // don't close this key itself for the case of empty szKey!
     if ( szKey.empty() )
@@ -1422,7 +1492,8 @@ bool KeyExists(WXHKEY hRootKey, const wxString& szKey)
             (HKEY)hRootKey,
             szKey.t_str(),
             RESERVED,
-            KEY_READ,        // we might not have enough rights for rw access
+            // we might not have enough rights for rw access
+            GetMSWAccessFlags(wxRegKey::Read, viewMode),
             &hkeyDummy
          ) == ERROR_SUCCESS )
     {
@@ -1434,6 +1505,49 @@ bool KeyExists(WXHKEY hRootKey, const wxString& szKey)
     return false;
 }
 
+long GetMSWViewFlags(wxRegKey::WOW64ViewMode viewMode)
+{
+    long samWOW64ViewMode = 0;
+
+    switch ( viewMode )
+    {
+        case wxRegKey::WOW64ViewMode_32:
+#ifdef __WIN64__    // the flag is only needed by 64 bit apps
+            samWOW64ViewMode = KEY_WOW64_32KEY;
+#endif // Win64
+            break;
+
+        case wxRegKey::WOW64ViewMode_64:
+#ifndef __WIN64__   // the flag is only needed by 32 bit apps
+            // 64 bit registry can only be accessed under 64 bit platforms
+            if ( wxIsPlatform64Bit() )
+                samWOW64ViewMode = KEY_WOW64_64KEY;
+#endif // Win32
+            break;
+
+        default:
+            wxFAIL_MSG("Unknown registry view.");
+            wxFALLTHROUGH;
+
+        case wxRegKey::WOW64ViewMode_Default:
+            // Use default registry view for the current application,
+            // i.e. 32 bits for 32 bit ones and 64 bits for 64 bit apps
+            ;
+    }
+
+    return samWOW64ViewMode;
+}
+
+long GetMSWAccessFlags(wxRegKey::AccessMode mode,
+    wxRegKey::WOW64ViewMode viewMode)
+{
+    long sam = mode == wxRegKey::Read ? KEY_READ : KEY_ALL_ACCESS;
+
+    sam |= GetMSWViewFlags(viewMode);
+
+    return sam;
+}
+
 wxString GetFullName(const wxRegKey *pKey, const wxString& szValue)
 {
   wxString str(pKey->GetName());
