@@ -1,10 +1,9 @@
 /////////////////////////////////////////////////////////////////////////////
 // Name:        src/osx/cocoa/nonownedwnd.mm
 // Purpose:     non owned window for cocoa
-// Author:      DavidStefan Csomor
+// Author:      Stefan Csomor
 // Modified by:
 // Created:     2008-06-20
-// RCS-ID:      $Id: nonownedwnd.mm 48805 2007-09-19 14:52:25Z SC $
 // Copyright:   (c) Stefan Csomor
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -14,13 +13,26 @@
     #include "wx/nonownedwnd.h"
     #include "wx/frame.h"
     #include "wx/app.h"
+    #include "wx/dialog.h"
+    #include "wx/menuitem.h"
+    #include "wx/menu.h"
 #endif
 
 #include "wx/osx/private.h"
 
+NSScreen* wxOSXGetMenuScreen()
+{
+    if ( [NSScreen screens] == nil )
+        return [NSScreen mainScreen];
+    else 
+    {
+        return [[NSScreen screens] objectAtIndex:0];
+    }
+}
+
 NSRect wxToNSRect( NSView* parent, const wxRect& r )
 {
-    NSRect frame = parent ? [parent bounds] : [[NSScreen mainScreen] frame];
+    NSRect frame = parent ? [parent bounds] : [wxOSXGetMenuScreen() frame];
     int y = r.y;
     int x = r.x ;
     if ( parent == NULL || ![ parent isFlipped ] )
@@ -30,7 +42,7 @@ NSRect wxToNSRect( NSView* parent, const wxRect& r )
 
 wxRect wxFromNSRect( NSView* parent, const NSRect& rect )
 {
-    NSRect frame = parent ? [parent bounds] : [[NSScreen mainScreen] frame];
+    NSRect frame = parent ? [parent bounds] : [wxOSXGetMenuScreen() frame];
     int y = (int)rect.origin.y;
     int x = (int)rect.origin.x;
     if ( parent == NULL || ![ parent isFlipped ] )
@@ -40,7 +52,7 @@ wxRect wxFromNSRect( NSView* parent, const NSRect& rect )
 
 NSPoint wxToNSPoint( NSView* parent, const wxPoint& p )
 {
-    NSRect frame = parent ? [parent bounds] : [[NSScreen mainScreen] frame];
+    NSRect frame = parent ? [parent bounds] : [wxOSXGetMenuScreen() frame];
     int x = p.x ;
     int y = p.y;
     if ( parent == NULL || ![ parent isFlipped ] )
@@ -50,7 +62,7 @@ NSPoint wxToNSPoint( NSView* parent, const wxPoint& p )
 
 wxPoint wxFromNSPoint( NSView* parent, const NSPoint& p )
 {
-    NSRect frame = parent ? [parent bounds] : [[NSScreen mainScreen] frame];
+    NSRect frame = parent ? [parent bounds] : [wxOSXGetMenuScreen() frame];
     int x = (int)p.x;
     int y = (int)p.y;
     if ( parent == NULL || ![ parent isFlipped ] )
@@ -66,6 +78,7 @@ bool shouldHandleSelector(SEL selector)
             || selector == @selector(deleteForward:)
             || selector == @selector(insertNewline:)
             || selector == @selector(insertTab:)
+            || selector == @selector(insertBacktab:)
             || selector == @selector(keyDown:)
             || selector == @selector(keyUp:)
             || selector == @selector(scrollPageUp:)
@@ -78,6 +91,11 @@ bool shouldHandleSelector(SEL selector)
 
 }
 
+static bool IsUsingFullScreenApi(WXWindow macWindow)
+{
+    return ([macWindow collectionBehavior] & NSWindowCollectionBehaviorFullScreenPrimary);
+}
+
 //
 // wx category for NSWindow (our own and wrapped instances)
 //
@@ -97,7 +115,7 @@ bool shouldHandleSelector(SEL selector)
     return (wxNonOwnedWindowCocoaImpl*) wxNonOwnedWindowImpl::FindFromWXWindow( self );
 }
 
-// TODO in cocoa everything during a drag is sent to the NSWindow the mouse down occured, 
+// TODO in cocoa everything during a drag is sent to the NSWindow the mouse down occurred,
 // this does not conform to the wx behaviour if the window is not captured, so try to resend
 // or capture all wx mouse event handling at the tlw as we did for carbon
 
@@ -106,12 +124,22 @@ bool shouldHandleSelector(SEL selector)
     bool handled = false;
     if ( ([event type] >= NSLeftMouseDown) && ([event type] <= NSMouseExited) )
     {
+        WXEVENTREF formerEvent = wxTheApp == NULL ? NULL : wxTheApp->MacGetCurrentEvent();
+        WXEVENTHANDLERCALLREF formerHandler = wxTheApp == NULL ? NULL : wxTheApp->MacGetCurrentEventHandlerCallRef();
+
         wxWindow* cw = wxWindow::GetCapture();
         if ( cw != NULL )
         {
+            if (wxTheApp)
+                wxTheApp->MacSetCurrentEvent(event, NULL);
             ((wxWidgetCocoaImpl*)cw->GetPeer())->DoHandleMouseEvent( event);
             handled = true;
         }
+        if ( handled )
+        {
+            if (wxTheApp)
+                wxTheApp->MacSetCurrentEvent(formerEvent , formerHandler);
+        }
     }
     return handled;
 }
@@ -121,6 +149,9 @@ bool shouldHandleSelector(SEL selector)
 // wx native implementation 
 //
 
+static NSResponder* s_nextFirstResponder = NULL;
+static NSResponder* s_formerFirstResponder = NULL;
+
 @interface wxNSWindow : NSWindow
 {
 }
@@ -128,6 +159,7 @@ bool shouldHandleSelector(SEL selector)
 - (void) sendEvent:(NSEvent *)event;
 - (NSRect)constrainFrameRect:(NSRect)frameRect toScreen:(NSScreen *)screen;
 - (void)noResponderFor: (SEL) selector;
+- (BOOL)makeFirstResponder:(NSResponder *)aResponder;
 @end
 
 @implementation wxNSWindow
@@ -182,18 +214,41 @@ bool shouldHandleSelector(SEL selector)
     return YES;
 }
 
+- (BOOL)makeFirstResponder:(NSResponder *)aResponder
+{
+    NSResponder* tempFormer = s_formerFirstResponder;
+    NSResponder* tempNext = s_nextFirstResponder;
+    s_nextFirstResponder = aResponder;
+    s_formerFirstResponder = [[NSApp keyWindow] firstResponder];
+    BOOL retval = [super makeFirstResponder:aResponder];
+    s_nextFirstResponder = tempNext;
+    s_formerFirstResponder = tempFormer;
+    return retval;
+}
+
 @end
 
 @interface wxNSPanel : NSPanel
 {
 }
 
+- (NSRect)constrainFrameRect:(NSRect)frameRect toScreen:(NSScreen *)screen;
 - (void)noResponderFor: (SEL) selector;
 - (void)sendEvent:(NSEvent *)event;
+- (BOOL)makeFirstResponder:(NSResponder *)aResponder;
 @end
 
 @implementation wxNSPanel
 
+- (NSRect)constrainFrameRect:(NSRect)frameRect toScreen:(NSScreen *)screen
+{
+    wxNonOwnedWindowCocoaImpl* impl = (wxNonOwnedWindowCocoaImpl*) wxNonOwnedWindowImpl::FindFromWXWindow( self );
+    if (impl && impl->IsFullScreen())
+        return frameRect;
+    else
+        return [super constrainFrameRect:frameRect toScreen:screen];
+}
+
 - (BOOL)canBecomeKeyWindow
 {
     return YES;
@@ -217,7 +272,30 @@ bool shouldHandleSelector(SEL selector)
 - (void)sendEvent:(NSEvent *) event
 {
     if ( ![self WX_filterSendEvent: event] )
+    {
+        WXEVENTREF formerEvent = wxTheApp == NULL ? NULL : wxTheApp->MacGetCurrentEvent();
+        WXEVENTHANDLERCALLREF formerHandler = wxTheApp == NULL ? NULL : wxTheApp->MacGetCurrentEventHandlerCallRef();
+        
+        if (wxTheApp)
+            wxTheApp->MacSetCurrentEvent(event, NULL);
+        
         [super sendEvent: event];
+        
+        if (wxTheApp)
+            wxTheApp->MacSetCurrentEvent(formerEvent , formerHandler);
+    }
+}
+
+- (BOOL)makeFirstResponder:(NSResponder *)aResponder
+{
+    NSResponder* tempFormer = s_formerFirstResponder;
+    NSResponder* tempNext = s_nextFirstResponder;
+    s_nextFirstResponder = aResponder;
+    s_formerFirstResponder = [[NSApp keyWindow] firstResponder];
+    BOOL retval = [super makeFirstResponder:aResponder];
+    s_nextFirstResponder = tempNext;
+    s_formerFirstResponder = tempFormer;
+    return retval;
 }
 
 @end
@@ -227,7 +305,9 @@ bool shouldHandleSelector(SEL selector)
 // controller
 //
 
-@interface wxNonOwnedWindowController : NSObject wxOSX_10_6_AND_LATER(<NSWindowDelegate>)
+static void *EffectiveAppearanceContext = &EffectiveAppearanceContext;
+
+@interface wxNonOwnedWindowController : NSObject <NSWindowDelegate>
 {
 }
 
@@ -236,19 +316,119 @@ bool shouldHandleSelector(SEL selector)
 - (void)windowDidResignKey:(NSNotification *)notification;
 - (void)windowDidBecomeKey:(NSNotification *)notification;
 - (void)windowDidMove:(NSNotification *)notification;
+- (void)windowDidMiniaturize:(NSNotification *)notification;
+- (void)windowDidDeminiaturize:(NSNotification *)notification;
 - (BOOL)windowShouldClose:(id)window;
 - (BOOL)windowShouldZoom:(NSWindow *)window toFrame:(NSRect)newFrame;
+- (void)windowWillEnterFullScreen:(NSNotification *)notification;
 
 @end
 
+extern int wxOSXGetIdFromSelector(SEL action );
+
 @implementation wxNonOwnedWindowController
 
 - (id) init
 {
-    [super init];
+    self = [super init];
     return self;
 }
 
+- (BOOL) triggerMenu:(SEL) action
+{
+    wxMenuBar* mbar = wxMenuBar::MacGetInstalledMenuBar();
+    if ( mbar )
+    {
+        wxMenu* menu = NULL;
+        wxMenuItem* menuitem = mbar->FindItem(wxOSXGetIdFromSelector(action), &menu);
+        if ( menu != NULL && menuitem != NULL)
+            return menu->HandleCommandProcess(menuitem);
+    }
+    return NO;
+}
+
+- (BOOL)validateMenuItem:(NSMenuItem *)menuItem 
+{    
+    SEL action = [menuItem action];
+
+    wxMenuBar* mbar = wxMenuBar::MacGetInstalledMenuBar();
+    if ( mbar )
+    {
+        wxMenu* menu = NULL;
+        wxMenuItem* menuitem = mbar->FindItem(wxOSXGetIdFromSelector(action), &menu);
+        if ( menu != NULL && menuitem != NULL)
+        {
+            menu->HandleCommandUpdateStatus(menuitem);
+            return menuitem->IsEnabled();
+        }
+    }
+    return YES;
+}
+
+- (void)undo:(id)sender 
+{
+    wxUnusedVar(sender);
+    [self triggerMenu:_cmd];
+}
+
+- (void)redo:(id)sender 
+{
+    wxUnusedVar(sender);
+    [self triggerMenu:_cmd];
+}
+
+- (void)cut:(id)sender 
+{
+    wxUnusedVar(sender);
+    [self triggerMenu:_cmd];
+}
+
+- (void)copy:(id)sender
+{
+    wxUnusedVar(sender);
+    [self triggerMenu:_cmd];
+}
+
+- (void)paste:(id)sender
+{
+    wxUnusedVar(sender);
+    [self triggerMenu:_cmd];
+}
+
+- (void)delete:(id)sender 
+{
+    wxUnusedVar(sender);
+    [self triggerMenu:_cmd];
+}
+
+- (void)selectAll:(id)sender 
+{
+    wxUnusedVar(sender);
+    [self triggerMenu:_cmd];
+}
+
+- (void)windowDidMiniaturize:(NSNotification *)notification
+{
+    NSWindow* window = (NSWindow*) [notification object];
+    wxNonOwnedWindowCocoaImpl* windowimpl = [window WX_implementation];
+    if ( windowimpl )
+    {
+        if ( wxNonOwnedWindow* wxpeer = windowimpl->GetWXPeer() )
+            wxpeer->OSXHandleMiniaturize(0, [window isMiniaturized]);
+    }
+}
+
+- (void)windowDidDeminiaturize:(NSNotification *)notification
+{
+    NSWindow* window = (NSWindow*) [notification object];
+    wxNonOwnedWindowCocoaImpl* windowimpl = [window WX_implementation];
+    if ( windowimpl )
+    {
+        if ( wxNonOwnedWindow* wxpeer = windowimpl->GetWXPeer() )
+            wxpeer->OSXHandleMiniaturize(0, [window isMiniaturized]);
+    }
+}
+
 - (BOOL)windowShouldClose:(id)nwindow
 {
     wxNonOwnedWindowCocoaImpl* windowimpl = [(NSWindow*) nwindow WX_implementation];
@@ -314,6 +494,15 @@ bool shouldHandleSelector(SEL selector)
     wxNonOwnedWindowCocoaImpl* windowimpl = [window WX_implementation];
     if ( windowimpl )
     {
+        // See windowDidResignKey: -- we emulate corresponding focus set
+        // event for the first responder here as well:
+        NSResponder *firstResponder = [window firstResponder];
+        wxWidgetCocoaImpl *focused = firstResponder
+                ? (wxWidgetCocoaImpl*)wxWidgetImpl::FindFromWXWidget(wxOSXGetViewFromResponder(firstResponder))
+                : NULL;
+        if ( focused )
+            focused->DoNotifyFocusSet();
+
         wxNonOwnedWindow* wxpeer = windowimpl->GetWXPeer();
         if ( wxpeer )
             wxpeer->HandleActivated(0, true);
@@ -330,12 +519,26 @@ bool shouldHandleSelector(SEL selector)
         if ( wxpeer )
         {
             wxpeer->HandleActivated(0, false);
+
+            // As for wx the deactivation also means losing focus, we
+            // must emulate focus events _without_ resetting first responder
+            // (because that would subtly break other things in Cocoa/macOS):
+            NSResponder *firstResponder = [window firstResponder];
+            wxWidgetCocoaImpl *focused = firstResponder
+                    ? (wxWidgetCocoaImpl*)wxWidgetImpl::FindFromWXWidget(wxOSXGetViewFromResponder(firstResponder))
+                    : NULL;
+            if ( focused )
+                focused->DoNotifyFocusLost();
+
+            // TODO Remove if no problems arise with Popup Windows
+#if 0
             // Needed for popup window since the firstResponder
             // (focus in wx) doesn't change when this
             // TLW becomes inactive.
             wxFocusEvent event( wxEVT_KILL_FOCUS, wxpeer->GetId());
             event.SetEventObject(wxpeer);
             wxpeer->HandleWindowEvent(event);
+#endif
         }
     }
 }
@@ -352,11 +555,27 @@ bool shouldHandleSelector(SEL selector)
         {
             editor = [[wxNSTextFieldEditor alloc] init];
             [editor setFieldEditor:YES];
+            [editor setTextField:tf];
             [tf setFieldEditor:editor];
+            [editor release];
         }
         return editor;
-    }
-
+    } 
+    else if ([anObject isKindOfClass:[wxNSComboBox class]])
+    {
+        wxNSComboBox * cb = (wxNSComboBox*) anObject;
+        wxNSTextFieldEditor* editor = [cb fieldEditor];
+        if ( editor == nil )
+        {
+            editor = [[wxNSTextFieldEditor alloc] init];
+            [editor setFieldEditor:YES];
+            [editor setTextField:cb];
+            [cb setFieldEditor:editor];
+            [editor release];
+        }
+        return editor;
+    }    
+ 
     return nil;
 }
 
@@ -374,9 +593,55 @@ bool shouldHandleSelector(SEL selector)
     return true;
 }
 
+// work around OS X bug, on a secondary monitor an already fully sized window
+// (eg maximized) will not be correctly put to full screen size and keeps a 22px
+// title band at the top free, therefore we force the correct content size
+
+- (void)windowWillEnterFullScreen:(NSNotification *)notification
+{
+    NSWindow* window = (NSWindow*) [notification object];
+
+    NSView* view = [window contentView];
+    NSRect windowframe = [window frame];
+    NSRect viewframe = [view frame];
+    NSUInteger stylemask = [window styleMask] | NSFullScreenWindowMask;
+    NSRect expectedframerect = [NSWindow contentRectForFrameRect: windowframe styleMask: stylemask];
+    
+    if ( !NSEqualSizes(expectedframerect.size, viewframe.size) )
+    {
+        [view setFrameSize: expectedframerect.size];
+    }
+}
+ 
+- (void)addObservers:(NSWindow*)win
+{
+    [win addObserver:self forKeyPath:@"effectiveAppearance"
+             options:0 context:EffectiveAppearanceContext];
+}
+
+- (void)removeObservers:(NSWindow*)win
+{
+    [win removeObserver:self forKeyPath:@"effectiveAppearance" context:EffectiveAppearanceContext];
+}
+
+- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
+{
+    if (context == EffectiveAppearanceContext)
+    {
+        wxNonOwnedWindowCocoaImpl* windowimpl = [(NSWindow*)object WX_implementation];
+        wxNonOwnedWindow* wxpeer = windowimpl ? windowimpl->GetWXPeer() : NULL;
+        if (wxpeer)
+        {
+            wxSysColourChangedEvent event;
+            event.SetEventObject(wxpeer);
+            wxpeer->HandleWindowEvent(event);
+        }
+    }
+}
+
 @end
 
-IMPLEMENT_DYNAMIC_CLASS( wxNonOwnedWindowCocoaImpl , wxNonOwnedWindowImpl )
+wxIMPLEMENT_DYNAMIC_CLASS(wxNonOwnedWindowCocoaImpl , wxNonOwnedWindowImpl);
 
 wxNonOwnedWindowCocoaImpl::wxNonOwnedWindowCocoaImpl( wxNonOwnedWindow* nonownedwnd) :
     wxNonOwnedWindowImpl(nonownedwnd)
@@ -395,8 +660,16 @@ wxNonOwnedWindowCocoaImpl::~wxNonOwnedWindowCocoaImpl()
 {
     if ( !m_wxPeer->IsNativeWindowWrapper() )
     {
+        [(wxNonOwnedWindowController*)[m_macWindow delegate] removeObservers:m_macWindow];
         [m_macWindow setDelegate:nil];
-        [m_macWindow release];
+     
+        // make sure we remove this first, otherwise the ref count will not lead to the 
+        // native window's destruction
+        if ([m_macWindow parentWindow] != 0)
+            [[m_macWindow parentWindow] removeChildWindow: m_macWindow];
+
+        [m_macWindow setReleasedWhenClosed:YES];
+        [m_macWindow close];
     }
 }
 
@@ -404,11 +677,12 @@ void wxNonOwnedWindowCocoaImpl::WillBeDestroyed()
 {
     if ( !m_wxPeer->IsNativeWindowWrapper() )
     {
+        [(wxNonOwnedWindowController*)[m_macWindow delegate] removeObservers:m_macWindow];
         [m_macWindow setDelegate:nil];
     }
 }
 
-void wxNonOwnedWindowCocoaImpl::Create( wxWindow* WXUNUSED(parent), const wxPoint& pos, const wxSize& size,
+void wxNonOwnedWindowCocoaImpl::Create( wxWindow* parent, const wxPoint& pos, const wxSize& size,
 long style, long extraStyle, const wxString& WXUNUSED(name) )
 {
     static wxNonOwnedWindowController* controller = NULL;
@@ -421,41 +695,21 @@ long style, long extraStyle, const wxString& WXUNUSED(name) )
 
     if ( style & wxFRAME_TOOL_WINDOW || ( style & wxPOPUP_WINDOW ) ||
             GetWXPeer()->GetExtraStyle() & wxTOPLEVEL_EX_DIALOG )
-    {
         m_macWindow = [wxNSPanel alloc];
-    }
     else
         m_macWindow = [wxNSWindow alloc];
 
-    CGWindowLevel level = kCGNormalWindowLevel;
+    [m_macWindow setAcceptsMouseMovedEvents:YES];
+
+    NSInteger level = NSNormalWindowLevel;
 
     if ( style & wxFRAME_TOOL_WINDOW )
     {
         windowstyle |= NSUtilityWindowMask;
-        if ( ( style & wxMINIMIZE_BOX ) || ( style & wxMAXIMIZE_BOX ) ||
-            ( style & wxCLOSE_BOX ) || ( style & wxSYSTEM_MENU ) )
-        {
-            windowstyle |= NSTitledWindowMask ;
-        }
     }
     else if ( ( style & wxPOPUP_WINDOW ) )
     {
-        level = kCGPopUpMenuWindowLevel;
-        /*
-        if ( ( style & wxBORDER_NONE ) )
-        {
-            wclass = kHelpWindowClass ;   // has no border
-            attr |= kWindowNoShadowAttribute;
-        }
-        else
-        {
-            wclass = kPlainWindowClass ;  // has a single line border, it will have to do for now
-        }
-        */
-    }
-    else if ( ( style & wxCAPTION ) )
-    {
-        windowstyle |= NSTitledWindowMask ;
+        level = NSPopUpMenuWindowLevel;
     }
     else if ( ( style & wxFRAME_DRAWER ) )
     {
@@ -463,48 +717,32 @@ long style, long extraStyle, const wxString& WXUNUSED(name) )
         wclass = kDrawerWindowClass;
         */
     }
-    else
-    {
-        // set these even if we have no title, otherwise the controls won't be visible
-        if ( ( style & wxMINIMIZE_BOX ) || ( style & wxMAXIMIZE_BOX ) ||
-            ( style & wxCLOSE_BOX ) || ( style & wxSYSTEM_MENU ) )
-        {
-            windowstyle |= NSTitledWindowMask ;
-        }
-        /*
-        else if ( ( style & wxNO_BORDER ) )
-        {
-            wclass = kSimpleWindowClass ;
-        }
-        else
-        {
-            wclass = kPlainWindowClass ;
-        }
-        */
-    }
-
-    if ( windowstyle & NSTitledWindowMask )
+ 
+    if ( ( style & wxMINIMIZE_BOX ) || ( style & wxMAXIMIZE_BOX ) ||
+        ( style & wxCLOSE_BOX ) || ( style & wxSYSTEM_MENU ) || ( style & wxCAPTION ) )
     {
+        windowstyle |= NSTitledWindowMask ;
         if ( ( style & wxMINIMIZE_BOX ) )
             windowstyle |= NSMiniaturizableWindowMask ;
-
+        
         if ( ( style & wxMAXIMIZE_BOX ) )
-            windowstyle |= NSResizableWindowMask ; // TODO showing ZOOM ?
-
-        if ( ( style & wxRESIZE_BORDER ) )
             windowstyle |= NSResizableWindowMask ;
-
+        
         if ( ( style & wxCLOSE_BOX) )
             windowstyle |= NSClosableWindowMask ;
     }
+    
+    if ( ( style & wxRESIZE_BORDER ) )
+        windowstyle |= NSResizableWindowMask ;
+
     if ( extraStyle & wxFRAME_EX_METAL)
         windowstyle |= NSTexturedBackgroundWindowMask;
 
     if ( ( style & wxFRAME_FLOAT_ON_PARENT ) || ( style & wxFRAME_TOOL_WINDOW ) )
-        level = kCGFloatingWindowLevel;
+        level = NSFloatingWindowLevel;
 
     if ( ( style & wxSTAY_ON_TOP ) )
-        level = kCGUtilityWindowLevel;
+        level = NSModalPanelWindowLevel;
 
     NSRect r = wxToNSRect( NULL, wxRect( pos, size) );
 
@@ -515,21 +753,32 @@ long style, long extraStyle, const wxString& WXUNUSED(name) )
         backing:NSBackingStoreBuffered
         defer:NO
         ];
-
-    [m_macWindow setLevel:level];
+    
+    // if we just have a title bar with no buttons needed, hide them
+    if ( (windowstyle & NSTitledWindowMask) && 
+        !(style & wxCLOSE_BOX) && !(style & wxMAXIMIZE_BOX) && !(style & wxMINIMIZE_BOX) )
+    {
+        [[m_macWindow standardWindowButton:NSWindowZoomButton] setHidden:YES];
+        [[m_macWindow standardWindowButton:NSWindowCloseButton] setHidden:YES];
+        [[m_macWindow standardWindowButton:NSWindowMiniaturizeButton] setHidden:YES];
+    }
+    
+    m_macWindowLevel = level;
+    SetUpForModalParent();
+    [m_macWindow setLevel:m_macWindowLevel];
 
     [m_macWindow setDelegate:controller];
-
-    [m_macWindow setAcceptsMouseMovedEvents: YES];
-    
-    if ( ( style & wxFRAME_SHAPED) )
-    {
-        [m_macWindow setOpaque:NO];
-        [m_macWindow setAlphaValue:1.0];
-    }
+    [controller addObservers:m_macWindow];
     
     if ( !(style & wxFRAME_TOOL_WINDOW) )
         [m_macWindow setHidesOnDeactivate:NO];
+    
+    if ( GetWXPeer()->GetBackgroundStyle() == wxBG_STYLE_TRANSPARENT )
+    {
+        [m_macWindow setOpaque:NO];
+        [m_macWindow setAlphaValue:1.0];
+        [m_macWindow setBackgroundColor:[NSColor clearColor]];
+    }
 }
 
 void wxNonOwnedWindowCocoaImpl::Create( wxWindow* WXUNUSED(parent), WXWindow nativeWindow )
@@ -544,7 +793,7 @@ WXWindow wxNonOwnedWindowCocoaImpl::GetWXWindow() const
 
 void wxNonOwnedWindowCocoaImpl::Raise()
 {
-    [m_macWindow orderWindow:NSWindowAbove relativeTo:0];
+    [m_macWindow makeKeyAndOrderFront:nil];
 }
 
 void wxNonOwnedWindowCocoaImpl::Lower()
@@ -552,8 +801,41 @@ void wxNonOwnedWindowCocoaImpl::Lower()
     [m_macWindow orderWindow:NSWindowBelow relativeTo:0];
 }
 
+void wxNonOwnedWindowCocoaImpl::SetUpForModalParent()
+{
+    wxNonOwnedWindow* wxpeer = GetWXPeer();
+    if (wxpeer)
+    {
+        // If the parent is modal, windows with wxFRAME_FLOAT_ON_PARENT style need
+        // to be in NSModalPanelWindowLevel and not NSFloatingWindowLevel to stay
+        // above the parent.
+        wxDialog* const parentDialog = wxDynamicCast(wxGetTopLevelParent(wxpeer->GetParent()), wxDialog);
+        if (parentDialog && parentDialog->IsModal())
+        {
+            if (m_macWindowLevel == NSFloatingWindowLevel)
+            {
+                m_macWindowLevel = NSModalPanelWindowLevel;
+                if ([m_macWindow level] == NSFloatingWindowLevel)
+                    [m_macWindow setLevel:m_macWindowLevel];
+            }
+
+            // Cocoa's modal loop does not process other windows by default, but
+            // don't call this on normal window levels so nested modal dialogs will
+            // still behave modally.
+            if (m_macWindowLevel != NSNormalWindowLevel)
+            {
+                if ([m_macWindow isKindOfClass:[NSPanel class]])
+                {
+                    [(NSPanel*)m_macWindow setWorksWhenModal:YES];
+                }
+            }
+        }
+    }
+}
+
 void wxNonOwnedWindowCocoaImpl::ShowWithoutActivating()
 {
+    SetUpForModalParent();
     [m_macWindow orderFront:nil];
     [[m_macWindow contentView] setNeedsDisplay: YES];
 }
@@ -563,14 +845,45 @@ bool wxNonOwnedWindowCocoaImpl::Show(bool show)
     if ( show )
     {
         wxNonOwnedWindow* wxpeer = GetWXPeer(); 
-        if (wxpeer && !(wxpeer->GetWindowStyle() & wxFRAME_TOOL_WINDOW)) 
-            [m_macWindow makeKeyAndOrderFront:nil];
-        else 
-            [m_macWindow orderFront:nil]; 
+        if ( wxpeer )
+        {
+            // add to parent window before showing
+            wxDialog * const dialog = wxDynamicCast(wxpeer, wxDialog);
+            if ( wxpeer->GetParent() && dialog )
+            {
+                NSView * parentView = wxpeer->GetParent()->GetPeer()->GetWXWidget();
+                if ( parentView )
+                {
+                    NSWindow* parentNSWindow = [parentView window];
+                    if ( parentNSWindow ) {
+                        [parentNSWindow addChildWindow:m_macWindow ordered:NSWindowAbove];
+                        // If the parent is modal, windows with wxFRAME_FLOAT_ON_PARENT style need
+                        // to be in NSModalPanelWindowLevel and not NSFloatingWindowLevel to stay
+                        // above the parent.
+                        if ([m_macWindow level] == NSFloatingWindowLevel ||
+                            [m_macWindow level] == NSModalPanelWindowLevel) {
+                            m_macWindowLevel = NSModalPanelWindowLevel;
+                            [m_macWindow setLevel:m_macWindowLevel];
+                        }
+                    }
+                }
+            }
+            
+            SetUpForModalParent();
+            if (!(wxpeer->GetWindowStyle() & wxFRAME_TOOL_WINDOW))
+                [m_macWindow makeKeyAndOrderFront:nil];
+            else 
+                [m_macWindow orderFront:nil]; 
+        }
         [[m_macWindow contentView] setNeedsDisplay: YES];
     }
     else
+    {
+        // avoid propagation of orderOut to parent 
+        if ([m_macWindow parentWindow] != 0)
+            [[m_macWindow parentWindow] removeChildWindow: m_macWindow];
         [m_macWindow orderOut:nil];
+    }
     return true;
 }
 
@@ -593,8 +906,9 @@ bool wxNonOwnedWindowCocoaImpl::SetTransparent(wxByte alpha)
     return true;
 }
 
-bool wxNonOwnedWindowCocoaImpl::SetBackgroundColour(const wxColour& WXUNUSED(col) )
+bool wxNonOwnedWindowCocoaImpl::SetBackgroundColour(const wxColour& col )
 {
+    [m_macWindow setBackgroundColor:col.OSXGetNSColor()];
     return true;
 }
 
@@ -617,16 +931,18 @@ void wxNonOwnedWindowCocoaImpl::SetExtraStyle( long exStyle )
 
 void wxNonOwnedWindowCocoaImpl::SetWindowStyleFlag( long style )
 {
-    if (m_macWindow)
+    // don't mess with native wrapped windows, they might throw an exception when their level is changed
+    if (!m_wxPeer->IsNativeWindowWrapper() && m_macWindow)
     {
-        CGWindowLevel level = kCGNormalWindowLevel;
+        NSInteger level = NSNormalWindowLevel;
         
         if (style & wxSTAY_ON_TOP)
-            level = kCGUtilityWindowLevel;
+            level = NSModalPanelWindowLevel;
         else if (( style & wxFRAME_FLOAT_ON_PARENT ) || ( style & wxFRAME_TOOL_WINDOW ))
-            level = kCGFloatingWindowLevel;
+            level = NSFloatingWindowLevel;
         
         [m_macWindow setLevel: level];
+        m_macWindowLevel = level;
     }
 }
 
@@ -669,8 +985,6 @@ void wxNonOwnedWindowCocoaImpl::GetSize( int &width, int &height ) const
 
 void wxNonOwnedWindowCocoaImpl::GetContentArea( int& left, int &top, int &width, int &height ) const
 {
-    NSRect outer = NSMakeRect(100,100,100,100);
-    NSRect content = [NSWindow contentRectForFrameRect:outer styleMask:[m_macWindow styleMask] ];
     NSRect rect = [[m_macWindow contentView] frame];
     left = (int)rect.origin.x;
     top = (int)rect.origin.y;
@@ -691,11 +1005,37 @@ void wxNonOwnedWindowCocoaImpl::SetTitle( const wxString& title, wxFontEncoding
     [m_macWindow setTitle:wxCFStringRef( title , encoding ).AsNSString()];
 }
 
+bool wxNonOwnedWindowCocoaImpl::EnableCloseButton(bool enable)
+{
+    [[m_macWindow standardWindowButton:NSWindowCloseButton] setEnabled:enable];
+
+    return true;
+}
+
+bool wxNonOwnedWindowCocoaImpl::EnableMaximizeButton(bool enable)
+{
+    [[m_macWindow standardWindowButton:NSWindowZoomButton] setEnabled:enable];
+
+    return true;
+}
+
+bool wxNonOwnedWindowCocoaImpl::EnableMinimizeButton(bool enable)
+{
+    [[m_macWindow standardWindowButton:NSWindowMiniaturizeButton] setEnabled:enable];
+
+    return true;
+}
+
 bool wxNonOwnedWindowCocoaImpl::IsMaximized() const
 {
     if (([m_macWindow styleMask] & NSResizableWindowMask) != 0)
     {
-        return [m_macWindow isZoomed];
+        // isZoomed internally calls windowWillResize which would trigger
+        // an wxEVT_SIZE. Setting ignore resizing supresses the event
+        m_wxPeer->OSXSetIgnoreResizing(true);
+        BOOL result = [m_macWindow isZoomed];
+        m_wxPeer->OSXSetIgnoreResizing(false);
+        return result;
     }
     else
     {
@@ -731,17 +1071,57 @@ void wxNonOwnedWindowCocoaImpl::Maximize(bool WXUNUSED(maximize))
 
 typedef struct
 {
+    NSUInteger m_formerStyleMask;
     int m_formerLevel;
     NSRect m_formerFrame;
 } FullScreenData ;
 
 bool wxNonOwnedWindowCocoaImpl::IsFullScreen() const
 {
+    if ( IsUsingFullScreenApi(m_macWindow) )
+    {
+        return [m_macWindow styleMask] & NSFullScreenWindowMask;
+    }
+
     return m_macFullScreenData != NULL ;
 }
 
+bool wxNonOwnedWindowCocoaImpl::EnableFullScreenView(bool enable)
+{
+    NSUInteger collectionBehavior = [m_macWindow collectionBehavior];
+    if (enable)
+    {
+        collectionBehavior |= NSWindowCollectionBehaviorFullScreenPrimary;
+        collectionBehavior &= ~NSWindowCollectionBehaviorFullScreenAuxiliary;
+    }
+    else
+    {
+        // Note that just turning "Full Screen Primary" is not enough, the
+        // window would still be made full screen when the green button in the
+        // title bar is pressed, and we need to explicitly turn on the "Full
+        // Screen Auxiliary" style to prevent this from happening. This works,
+        // at least under 10.11 and 10.14, even though it's not really clear
+        // from the documentation that it should.
+        collectionBehavior &= ~NSWindowCollectionBehaviorFullScreenPrimary;
+        collectionBehavior |= NSWindowCollectionBehaviorFullScreenAuxiliary;
+    }
+    [m_macWindow setCollectionBehavior: collectionBehavior];
+
+    return true;
+}
+
 bool wxNonOwnedWindowCocoaImpl::ShowFullScreen(bool show, long WXUNUSED(style))
 {
+    if ( IsUsingFullScreenApi(m_macWindow) )
+    {
+        if ( show != IsFullScreen() )
+        {
+            [m_macWindow toggleFullScreen: nil];
+        }
+
+        return true;
+    }
+
     if ( show )
     {
         FullScreenData *data = (FullScreenData *)m_macFullScreenData ;
@@ -751,18 +1131,38 @@ bool wxNonOwnedWindowCocoaImpl::ShowFullScreen(bool show, long WXUNUSED(style))
         m_macFullScreenData = data ;
         data->m_formerLevel = [m_macWindow level];
         data->m_formerFrame = [m_macWindow frame];
-        CGDisplayCapture( kCGDirectMainDisplay );
-        [m_macWindow setLevel:CGShieldingWindowLevel()];
-        [m_macWindow setFrame:[[NSScreen mainScreen] frame] display:YES];
+        data->m_formerStyleMask = [m_macWindow styleMask];
+
+        NSRect screenframe = [[NSScreen mainScreen] frame];
+        NSRect frame = NSMakeRect (0, 0, 100, 100);
+        NSRect contentRect;
+
+        [m_macWindow setStyleMask:data->m_formerStyleMask & ~ NSResizableWindowMask];
+        
+        contentRect = [NSWindow contentRectForFrameRect: frame
+                                styleMask: [m_macWindow styleMask]];
+        screenframe.origin.y += (frame.origin.y - contentRect.origin.y);
+        screenframe.size.height += (frame.size.height - contentRect.size.height);
+        [m_macWindow setFrame:screenframe display:YES];
+
+        SetSystemUIMode(kUIModeAllHidden,
+                                kUIOptionDisableAppleMenu
+                        /*
+                                | kUIOptionDisableProcessSwitch
+                                | kUIOptionDisableForceQuit
+                         */); 
     }
     else if ( m_macFullScreenData != NULL )
     {
         FullScreenData *data = (FullScreenData *) m_macFullScreenData ;
-        CGDisplayRelease( kCGDirectMainDisplay );
-        [m_macWindow setLevel:data->m_formerLevel];
+        
         [m_macWindow setFrame:data->m_formerFrame display:YES];
+        [m_macWindow setStyleMask:data->m_formerStyleMask];
+
         delete data ;
         m_macFullScreenData = NULL ;
+
+        SetSystemUIMode(kUIModeNormal, 0); 
     }
 
     return true;
@@ -792,9 +1192,10 @@ void wxNonOwnedWindowCocoaImpl::RequestUserAttention(int flagsWX)
 void wxNonOwnedWindowCocoaImpl::ScreenToWindow( int *x, int *y )
 {
     wxPoint p((x ? *x : 0), (y ? *y : 0) );
-    NSPoint nspt = wxToNSPoint( NULL, p );
-    nspt = [m_macWindow convertScreenToBase:nspt];
-    nspt = [[m_macWindow contentView] convertPoint:nspt fromView:nil];
+    NSRect nsrect = NSZeroRect;
+    nsrect.origin = wxToNSPoint( NULL, p );
+    nsrect = [m_macWindow convertRectFromScreen:nsrect];
+    NSPoint nspt = [[m_macWindow contentView] convertPoint:nsrect.origin fromView:nil];
     p = wxFromNSPoint([m_macWindow contentView], nspt);
     if ( x )
         *x = p.x;
@@ -807,8 +1208,10 @@ void wxNonOwnedWindowCocoaImpl::WindowToScreen( int *x, int *y )
     wxPoint p((x ? *x : 0), (y ? *y : 0) );
     NSPoint nspt = wxToNSPoint( [m_macWindow contentView], p );
     nspt = [[m_macWindow contentView] convertPoint:nspt toView:nil];
-    nspt = [m_macWindow convertBaseToScreen:nspt];
-    p = wxFromNSPoint( NULL, nspt);
+    NSRect nsrect = NSZeroRect;
+    nsrect.origin = nspt;
+    nsrect = [m_macWindow convertRectToScreen:nsrect];
+    p = wxFromNSPoint( NULL, nsrect.origin);
     if ( x )
         *x = p.x;
     if ( y )
@@ -830,6 +1233,31 @@ bool wxNonOwnedWindowCocoaImpl::IsModified() const
     return [m_macWindow isDocumentEdited];
 }
 
+void wxNonOwnedWindowCocoaImpl::SetRepresentedFilename(const wxString& filename)
+{
+    [m_macWindow setRepresentedFilename:wxCFStringRef(filename).AsNSString()];
+}
+
+void wxNonOwnedWindowCocoaImpl::RestoreWindowLevel()
+{
+    if ( [m_macWindow level] != m_macWindowLevel )
+        [m_macWindow setLevel:m_macWindowLevel];
+}
+
+WX_NSResponder wxNonOwnedWindowCocoaImpl::GetNextFirstResponder()
+{
+    return s_nextFirstResponder;
+}
+
+WX_NSResponder wxNonOwnedWindowCocoaImpl::GetFormerFirstResponder()
+{
+    return s_formerFirstResponder;
+}
+
+//
+//
+//
+
 wxNonOwnedWindowImpl* wxNonOwnedWindowImpl::CreateNonOwnedWindow( wxNonOwnedWindow* wxpeer, wxWindow* parent, WXWindow nativeWindow)
 {
     wxNonOwnedWindowCocoaImpl* now = new wxNonOwnedWindowCocoaImpl( wxpeer );
@@ -844,3 +1272,4 @@ wxNonOwnedWindowImpl* wxNonOwnedWindowImpl::CreateNonOwnedWindow( wxNonOwnedWind
     now->Create( parent, pos, size, style , extraStyle, name );
     return now;
 }
+
