@@ -229,7 +229,7 @@ bool wxTextCtrl::Create( wxWindow *parent,
     m_editable = ((m_windowStyle & wxTE_READONLY) == 0);
 
     if (HasFlag(wxTE_PASSWORD))
-        m_sourceFont = wxFont( 12, wxMODERN, wxNORMAL, wxNORMAL );
+        m_sourceFont = wxFont( 12, wxFONTFAMILY_MODERN, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL );
     else
         m_sourceFont = GetFont();
 
@@ -1711,7 +1711,7 @@ void wxTextCtrl::OnPaint( wxPaintEvent &event )
     GetClientSize( &size_x, &size_y );
 
     dc.SetPen( *wxTRANSPARENT_PEN );
-    dc.SetBrush( wxBrush( wxTHEME_COLOUR(HIGHLIGHT), wxSOLID ) );
+    dc.SetBrush( wxBrush( wxTHEME_COLOUR(HIGHLIGHT), wxBRUSHSTYLE_SOLID ) );
     int upper = wxMin( (int)m_lines.GetCount(), scroll_y+(size_y/m_lineHeight)+2 );
     for (int i = scroll_y; i < upper; i++)
     {
