@@ -3,7 +3,6 @@
 // Purpose:     wxString unit test
 // Author:      Vadim Zeitlin, Wlodzimierz ABX Skiba
 // Created:     2004-05-07
-// RCS-ID:      $Id$
 // Copyright:   (c) 2004 Vadim Zeitlin, Wlodzimierz Skiba
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -21,6 +20,8 @@
     #include "wx/wx.h"
 #endif // WX_PRECOMP
 
+#include <algorithm>
+
 // ----------------------------------------------------------------------------
 // test class
 // ----------------------------------------------------------------------------
@@ -34,6 +35,7 @@ private:
     CPPUNIT_TEST_SUITE( StdStringTestCase );
         CPPUNIT_TEST( StdConstructors );
         CPPUNIT_TEST( StdIterators );
+        CPPUNIT_TEST( StdIteratorsCmp );
         CPPUNIT_TEST( StdAppend );
         CPPUNIT_TEST( StdAssign );
         CPPUNIT_TEST( StdCompare );
@@ -50,10 +52,12 @@ private:
 #if wxUSE_STD_STRING
         CPPUNIT_TEST( StdConversion );
 #endif
+        CPPUNIT_TEST( StdAlgo );
     CPPUNIT_TEST_SUITE_END();
 
     void StdConstructors();
     void StdIterators();
+    void StdIteratorsCmp();
     void StdAppend();
     void StdAssign();
     void StdCompare();
@@ -70,14 +74,15 @@ private:
 #if wxUSE_STD_STRING
     void StdConversion();
 #endif
+    void StdAlgo();
 
-    DECLARE_NO_COPY_CLASS(StdStringTestCase)
+    wxDECLARE_NO_COPY_CLASS(StdStringTestCase);
 };
 
 // register in the unnamed registry so that these tests are run by default
 CPPUNIT_TEST_SUITE_REGISTRATION( StdStringTestCase );
 
-// also include in it's own registry so that these tests can be run alone
+// also include in its own registry so that these tests can be run alone
 CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( StdStringTestCase, "StdStringTestCase" );
 
 StdStringTestCase::StdStringTestCase()
@@ -119,6 +124,30 @@ void StdStringTestCase::StdIterators()
     wxString::const_reverse_iterator i4;
 }
 
+void StdStringTestCase::StdIteratorsCmp()
+{
+    wxString s("foobar");
+    wxString::iterator i = s.begin();
+    wxString::const_iterator ci = s.begin();
+
+    CPPUNIT_ASSERT( i == ci );
+    CPPUNIT_ASSERT( i >= ci );
+    CPPUNIT_ASSERT( i <= ci );
+    CPPUNIT_ASSERT( ci == i );
+    CPPUNIT_ASSERT( ci >= i );
+    CPPUNIT_ASSERT( ci <= i );
+
+    ci++;
+
+    CPPUNIT_ASSERT( i != ci );
+    CPPUNIT_ASSERT( i < ci );
+    CPPUNIT_ASSERT( !(i > ci) );
+
+    CPPUNIT_ASSERT( ci != i );
+    CPPUNIT_ASSERT( ci > i );
+    CPPUNIT_ASSERT( !(ci < i) );
+}
+
 void StdStringTestCase::StdAppend()
 {
     wxString s1, s2, s3, s4, s5, s6, s7, s8;
@@ -489,10 +518,12 @@ void StdStringTestCase::StdResize()
     CPPUNIT_ASSERT_EQUAL( wxT("abcABCdefDEF  "), s3 );
     CPPUNIT_ASSERT_EQUAL( wxT("abcABCdefDEFWW"), s4 );
 
+#if wxUSE_UNICODE
     wxString s =
         wxString::FromUTF8("\xd0\x9f\xd1\x80\xd0\xb8\xd0\xb2\xd0\xb5\xd1\x82");
     s.resize(3);
     CPPUNIT_ASSERT_EQUAL( wxString::FromUTF8("\xd0\x9f\xd1\x80\xd0\xb8"), s);
+#endif // wxUSE_UNICODE
 }
 
 void StdStringTestCase::StdRiter()
@@ -562,7 +593,7 @@ void StdStringTestCase::StdConversion()
     // notice that implicit wxString -> std::string conversion is only
     // available in wxUSE_STL case, because it conflicts with conversion to
     // const char*/wchar_t*
-#if wxUSE_STL
+#if wxUSE_STL && wxUSE_UNSAFE_WXSTRING_CONV
     std::string s5 = s4;
 #else
     std::string s5 = s4.ToStdString();
@@ -576,10 +607,32 @@ void StdStringTestCase::StdConversion()
 #endif
     CPPUNIT_ASSERT_EQUAL( "hello", s6 );
 
+#if wxUSE_UNSAFE_WXSTRING_CONV
     std::string s7(s4);
     CPPUNIT_ASSERT( s7 == "hello" );
+#endif
 
     wxStdWideString s8(s4);
     CPPUNIT_ASSERT( s8 == "hello" );
+
+    std::string s9("\xF0\x9F\x90\xB1\0\xE7\x8C\xAB", 9); /* U+1F431 U+0000 U+732B */
+    wxString s10 = wxString::FromUTF8(s9);
+    CPPUNIT_ASSERT_EQUAL( s9, s10.ToStdString(wxConvUTF8) );
+
+    std::string s11("xyz\0\xFF", 5); /* an invalid UTF-8 sequence */
+    CPPUNIT_ASSERT_EQUAL( wxString::FromUTF8(s11), "" );
 }
 #endif // wxUSE_STD_STRING
+
+void StdStringTestCase::StdAlgo()
+{
+    // Unfortunately this currently doesn't work with libc++ in C++11 mode, see
+    // comment near iter_swap() definition in wx/string.h.
+#if __cplusplus < 201103L || !defined(_LIBCPP_VERSION)
+    wxString s("AB");
+    std::reverse(s.begin(), s.end());
+    CPPUNIT_ASSERT_EQUAL( "BA", s );
+#else
+    wxLogWarning("Skipping std::reverse() test broken with C++11/libc++");
+#endif
+}
