@@ -5,7 +5,6 @@
 //              (part of this file was taken from CMP.c of TRIO package
 //               written by Bjorn Reese and Daniel Stenberg)
 // Created:     2006-04-01
-// RCS-ID:      $Id$
 // Copyright:   (c) 2006 Francesco Montorsi, Bjorn Reese and Daniel Stenberg
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -34,7 +33,6 @@
 //             http://www.gnu.org/software/libc/manual/html_node/Formatted-Output.html
 
 
-
 // ----------------------------------------------------------------------------
 // global utilities for testing
 // ----------------------------------------------------------------------------
@@ -118,7 +116,7 @@ class VsnprintfTestCase : public CppUnit::TestCase
 public:
     VsnprintfTestCase() {}
 
-    virtual void setUp();
+    virtual void setUp() wxOVERRIDE;
 
 private:
     CPPUNIT_TEST_SUITE( VsnprintfTestCase );
@@ -139,9 +137,7 @@ private:
 #endif
 
         CPPUNIT_TEST( BigToSmallBuffer );
-#if wxUSE_WXVSNPRINTF
         CPPUNIT_TEST( WrongFormatStrings );
-#endif // wxUSE_WXVSNPRINTF
         CPPUNIT_TEST( Miscellaneous );
         CPPUNIT_TEST( GlibcMisc1 );
         CPPUNIT_TEST( GlibcMisc2 );
@@ -164,16 +160,14 @@ private:
 #endif
     void Unicode();
 
-    template<typename T> 
+    template<typename T>
         void DoBigToSmallBuffer(T *buffer, int size);
     void BigToSmallBuffer();
 
-#if wxUSE_WXVSNPRINTF
     void WrongFormatStrings();
-#endif // wxUSE_WXVSNPRINTF
 
     // compares the expectedString and the result of wxVsnprintf() char by char
-    // for all its lenght (not only for first expectedLen chars) and also
+    // for all its length (not only for first expectedLen chars) and also
     // checks the return value
     void DoMisc(int expectedLen, const wxString& expectedString,
                 size_t max, const wxChar *format, ...);
@@ -182,13 +176,13 @@ private:
     void GlibcMisc1();
     void GlibcMisc2();
 
-    DECLARE_NO_COPY_CLASS(VsnprintfTestCase)
+    wxDECLARE_NO_COPY_CLASS(VsnprintfTestCase);
 };
 
 // register in the unnamed registry so that these tests are run by default
 CPPUNIT_TEST_SUITE_REGISTRATION( VsnprintfTestCase );
 
-// also include in it's own registry so that these tests can be run alone
+// also include in its own registry so that these tests can be run alone
 CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( VsnprintfTestCase, "VsnprintfTestCase" );
 
 void VsnprintfTestCase::setUp()
@@ -235,13 +229,14 @@ void VsnprintfTestCase::O()
 
 void VsnprintfTestCase::P()
 {
-    // WARNING: printing of pointers is not fully standard.
-    //          GNU prints them as %#x except for NULL pointers which are
-    //          printed as '(nil)'.
-    //          MSVC always print them as %8X on 32 bit systems and as %16X
-    //          on 64 bit systems
-    //          mingw32 uses MSVC CRT by default so uses the same rules
-#if defined(__VISUALC__) || (defined(__MINGW32__) && !__USE_MINGW_ANSI_STDIO)
+    // The exact format used for "%p" is not specified by the standard and so
+    // varies among different platforms, so we need to expect different results
+    // here (remember that while we test our own wxPrintf() code here, it uses
+    // the system sprintf() for actual formatting so the results are still
+    // different under different systems).
+
+#if defined(__VISUALC__) || (defined(__MINGW32__) && \
+        (!defined(__USE_MINGW_ANSI_STDIO) || !__USE_MINGW_ANSI_STDIO))
     #if SIZEOF_VOID_P == 4
         CMP3("00ABCDEF", "%p", (void*)0xABCDEF);
         CMP3("00000000", "%p", (void*)NULL);
@@ -249,10 +244,17 @@ void VsnprintfTestCase::P()
         CMP3("0000ABCDEFABCDEF", "%p", (void*)0xABCDEFABCDEF);
         CMP3("0000000000000000", "%p", (void*)NULL);
     #endif
-#elif defined(__MINGW32__) 
-    CMP3("0xabcdef", "%p", (void*)0xABCDEF); 
-    CMP3("0", "%p", (void*)NULL); 
+#elif defined(__MINGW32__)
+    #if SIZEOF_VOID_P == 4
+        CMP3("00abcdef", "%p", (void*)0xABCDEF);
+        CMP3("00000000", "%p", (void*)NULL);
+    #elif SIZEOF_VOID_P == 8
+        CMP3("0000abcdefabcdef", "%p", (void*)0xABCDEFABCDEF);
+        CMP3("0000000000000000", "%p", (void*)NULL);
+    #endif
 #elif defined(__GNUG__)
+    // glibc prints pointers as %#x except for NULL pointers which are printed
+    // as '(nil)'.
     CMP3("0xabcdef", "%p", (void*)0xABCDEF);
     CMP3("(nil)", "%p", (void*)NULL);
 #endif
@@ -268,13 +270,10 @@ void VsnprintfTestCase::N()
 
 void VsnprintfTestCase::E()
 {
-    // NB: there are no standards about the minimum exponent width
-    //     (and the width of the %e conversion specifier refers to the
-    //      mantissa, not to the exponent).
-    //     Since newer MSVC versions use 3 digits as minimum exponent
-    //     width while GNU libc uses 2 digits as minimum width, here we
-    //     workaround this problem using for the exponent values with at
-    //     least three digits.
+    // NB: Use at least three digits for the exponent to workaround
+    //     differences between MSVC, MinGW and GNU libc.
+    //     See wxUSING_MANTISSA_SIZE_3 in testprec.h as well.
+    //
     //     Some examples:
     //       printf("%e",2.342E+02);
     //     -> under MSVC7.1 prints:      2.342000e+002
@@ -417,48 +416,26 @@ void VsnprintfTestCase::LongLong()
 
     CMP3("123456789", "%llu", (wxULongLong_t)123456789);
 
-#ifdef __WXMSW__
+#ifdef __WINDOWS__
     CMP3("123456789", "%I64d", (wxLongLong_t)123456789);
     CMP3("123456789abcdef", "%I64x", wxLL(0x123456789abcdef));
 #endif
 }
 #endif
 
-// this test is only for our own implementation, the system implementation
-// doesn't always give errors for invalid format strings (e.g. glibc doesn't)
-// and as it's not required too (the behaviour is "undefined" according to the
-// spec), there is really no sense in testing for it
-#if wxUSE_WXVSNPRINTF
-
 void VsnprintfTestCase::WrongFormatStrings()
 {
     // test how wxVsnprintf() behaves with wrong format string:
 
-#if 0
-    // NB: the next 2 tests currently return an error but it would be nice
-    //     if they didn't (see ticket #9367)
-
-    // two positionals with the same index:
-    r = wxSnprintf(buf, MAX_TEST_LEN, wxT("%1$s %1$s"), "hello");
-    CPPUNIT_ASSERT(r != -1);
-
-    // three positionals with the same index mixed with other pos args:
-    r = wxSnprintf(buf, MAX_TEST_LEN, wxT("%4$d %2$f %1$s %2$s %3$d"), "hello", "world", 3, 4);
-    CPPUNIT_ASSERT(r != -1);
-#endif
-
-    // a missing positional arg: this should result in an error but not all
-    // implementations detect it (e.g. glibc doesn't)
-    r = wxSnprintf(buf, MAX_TEST_LEN, wxT("%1$d %3$d"), 1, 2, 3);
-    CPPUNIT_ASSERT_EQUAL(-1, r);
+    // a missing positional arg should result in an assert
+    WX_ASSERT_FAILS_WITH_ASSERT(
+            wxSnprintf(buf, MAX_TEST_LEN, wxT("%1$d %3$d"), 1, 2, 3) );
 
     // positional and non-positionals in the same format string:
     r = wxSnprintf(buf, MAX_TEST_LEN, wxT("%1$d %d %3$d"), 1, 2, 3);
     CPPUNIT_ASSERT_EQUAL(-1, r);
 }
 
-#endif // wxUSE_WXVSNPRINTF
-
 // BigToSmallBuffer() test case helper:
 template<typename T>
 void VsnprintfTestCase::DoBigToSmallBuffer(T *buffer, int size)
@@ -516,8 +493,6 @@ void VsnprintfTestCase::DoBigToSmallBuffer(T *buffer, int size)
 
 void VsnprintfTestCase::BigToSmallBuffer()
 {
-    // VC6 can't compile this code
-#if !defined(__VISUALC__) || (__VISUALC__ >= 1310)
 #if wxUSE_UNICODE
     wchar_t bufw[1024], bufw2[16], bufw3[4], bufw4;
     DoBigToSmallBuffer(bufw, 1024);
@@ -531,7 +506,6 @@ void VsnprintfTestCase::BigToSmallBuffer()
     DoBigToSmallBuffer(bufa2, 16);
     DoBigToSmallBuffer(bufa3, 4);
     DoBigToSmallBuffer(&bufa4, 1);
-#endif // !VC6
 }
 
 // Miscellaneous() test case helper:
@@ -570,8 +544,10 @@ void VsnprintfTestCase::DoMisc(
     std::string errMsg(errStr.mb_str());
     std::string overflowMsg(overflowStr.mb_str());
 
-    CPPUNIT_ASSERT_MESSAGE(errMsg,
-            (expectedLen == -1 && size_t(n) >= max) || expectedLen == n);
+    if ( size_t(n) < max )
+        CPPUNIT_ASSERT_MESSAGE(errMsg, expectedLen == n);
+    else
+        CPPUNIT_ASSERT_MESSAGE(errMsg, expectedLen == -1);
 
     CPPUNIT_ASSERT_MESSAGE(errMsg, expectedString == buf);
 
@@ -617,8 +593,7 @@ void VsnprintfTestCase::GlibcMisc1()
 {
     CMP3("     ",    "%5.s", "xyz");
     CMP3("   33",    "%5.f", 33.3);
-#if defined(__VISUALC__)
-    // see the previous notes about the minimum width of mantissa:
+#if defined(wxDEFAULT_MANTISSA_SIZE_3)
     CMP3("  3e+008", "%8.e", 33.3e7);
     CMP3("  3E+008", "%8.E", 33.3e7);
     CMP3("3e+001",    "%.g",  33.3);
